@@ -101,17 +101,20 @@
 
 #include <stdio.h>
 
+#ifdef HAVE_X11_XAW_COMMAND_H
+
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/Command.h>
-#include <X11/Xaw/Viewport.h>
 
 #include <m17n-gui.h>
 #include <m17n-misc.h>
 #include <m17n-X.h>
 
+#include <X11/Xaw/Form.h>
+#include <X11/Xaw/Command.h>
+#include <X11/Xaw/Viewport.h>
+
 #define VERSION "1.2.0"
 
 /* Global m17n variables.  */
@@ -420,4 +423,17 @@ main (int argc, char **argv)
 
   exit (0);
 }
+
+#else  /* not HAVE_X11_XAW_COMMAND_H */
+
+int
+main (int argc, char **argv)
+{
+  fprintf (stderr,
+	   "Building of this program failed (lack of some header files)\n");
+  exit (1);
+}
+
+#endif /* not HAVE_X11_XAW_COMMAND_H */
+
 #endif /* not FOR_DOXYGEN */
