@@ -44,19 +44,19 @@
     @addtogroup m17nCharacter
     @brief ʸ֥ȤȤ˴ؤ API.
 
-    m17n 饤֥ @e ʸ ʸɡˤɽ롣Ǿʸ
-    ɤ @c 0 ǡʸɤϥޥ #MCHAR_MAX ˤä
-    Ƥ롣#MCHAR_MAX  @c 0x3FFFFF22ӥåȡ ʾǤ
-    ȤݾڤƤ롣
+    m17n 饤֥ @e ʸ ʸɡˤɽ롣
+    Ǿʸɤ @c 0 Ǥꡢʸɤϥޥ #MCHAR_MAX 
+    ˤäƤ롣#MCHAR_MAX  @c 0x3FFFFF22ӥåȡ
+    ʾǤ뤳ȤݾڤƤ롣
 
     @c 0  @c 0x10FFFF ޤǤʸϡƱͤ Unicode 
     ʸ˳ƤƤ롣
 
-    ʸ @e ʸץѥƥ ȸƤ֥ץѥƥ 0 İʾĤȤ
-    롣ʸץѥƥ @e   @e  ʤ롣ϥܥ
-    ꡢͤ <tt>(void *)</tt> ˥㥹ȤǤΤʤ鲿Ǥ褤 
-    ʸ C ʸץѥƥΤ K ǤΡפñ
-    ʸ C  K ץѥƥפȸƤ֤Ȥ롣  */
+    ʸ @e ʸץѥƥ ȸƤ֥ץѥƥ 0 İʾĤȤǤ롣
+    ʸץѥƥ @e   @e  ʤ롣
+    ϥܥǤꡢͤ <tt>(void *)</tt> ˥㥹ȤǤΤʤ鲿Ǥ褤 
+    ʸ C ʸץѥƥΤ K ǤΡפñˡʸ C 
+     K ץѥƥפȸƤ֤Ȥ롣  */
 /*=*/
 
 #if !defined (FOR_DOXYGEN) || defined (DOXYGEN_INTERNAL_MODULE)
@@ -95,22 +95,13 @@ free_string (int from, int to, void *str, void *arg)
 int
 mchar__init ()
 {
-  char_prop_list = mplist ();
-
-  Mname
-    = mchar_define_property ("name", Mstring);
-  Mcategory
-    = mchar_define_property ("category", Msymbol);
-  Mcombining_class
-    = mchar_define_property ("combining-class", Minteger);
-  Mbidi_category
-    = mchar_define_property ("bidirectional-category", Msymbol);
-  Msimple_case_folding
-    = mchar_define_property ("simple-case-folding", Minteger);
-  Mcomplicated_case_folding
-    = mchar_define_property ("complicated-case-folding", Mtext);
-  Mscript
-    = mchar_define_property ("script", Msymbol);
+  Mname = msymbol ("name");
+  Mcategory = msymbol ("category");
+  Mcombining_class = msymbol ("combining-class");
+  Mbidi_category = msymbol ("bidirectional-category");
+  Msimple_case_folding = msymbol ("simple-case-folding");
+  Mcomplicated_case_folding = msymbol ("complicated-case-folding");
+  Mscript = msymbol ("script");
 
   return 0;
 }
@@ -120,28 +111,68 @@ mchar__fini (void)
 {
   MPlist *p;
 
-  for (p = char_prop_list; mplist_key (p) != Mnil; p = mplist_next (p))
+  if (char_prop_list)
     {
-      MCharPropRecord *record = mplist_value (p);
-
-      if (record->table)
+      for (p = char_prop_list; mplist_key (p) != Mnil; p = mplist_next (p))
 	{
-	  if (record->type == Mstring)
-	    mchartable_map (record->table, NULL, free_string, NULL);
-	  M17N_OBJECT_UNREF (record->table);
+	  MCharPropRecord *record = mplist_value (p);
+
+	  if (record->table)
+	    {
+	      if (record->type == Mstring)
+		mchartable_map (record->table, NULL, free_string, NULL);
+	      M17N_OBJECT_UNREF (record->table);
+	    }
+	  free (record);
 	}
-      free (record);
+      M17N_OBJECT_UNREF (char_prop_list);
     }
-  M17N_OBJECT_UNREF (char_prop_list);
 }
 
+void
+mchar__define_prop (MSymbol key, MSymbol type, void *mdb)
+{
+  MCharPropRecord *record;
+
+  if (char_prop_list)
+    record = mplist_get (char_prop_list, key);
+  else
+    char_prop_list = mplist (), record = NULL;
+  if (record)
+    {
+      if (record->table)
+	M17N_OBJECT_UNREF (record->table);
+    }
+  else
+    {
+      MSTRUCT_CALLOC (record, MERROR_CHAR);
+      mplist_put (char_prop_list, key, record);
+    }
+
+  record->type = type;
+  record->mdb = mdb;
+  if (mdb)
+    {
+      record->table = NULL;
+    }
+  else
+    {
+      void *default_value = NULL;
+
+      if (type == Minteger)
+	default_value = (void *) -1;
+      record->table = mchartable (type, default_value);
+    }
+}
+
+
 /*** @} */
 #endif /* !FOR_DOXYGEN || DOXYGEN_INTERNAL_MODULE */
 
 /* External API */
 
-/*** @addtogroup m17nCharacter */
-/*** @{ */
+/*** @addtogroup m17nCharacter
+     @{ */
 /*=*/
 
 #ifdef FOR_DOXYGEN
@@ -153,23 +184,22 @@ mchar__fini (void)
 /***ja
     @brief ʸɤκ.
 
-    ޥ #MCHAR_MAX ʸɤκͤͿ롣  */
+    ޥ #MCHAR_MAX ʸɤκͤɽ  */
 
 #define MCHAR_MAX
 /*=*/
 #endif /* FOR_DOXYGEN */
 
 /***en
-    @ingroup m17nCharacter
     @name Variables: Keys of character properties
 
     These symbols are used as keys of character properties.  */
 
 /***ja
-     @name ѿ: ʸץѥƥΥ
+    @ingroup m17nCharacter
+    @name ѿ: ʸץѥƥΥ
 
-     ΥܥʸץѥƥΥȤƻȤ롣*/
-/*=*/
+    ΥܥʸץѥƥΥȤƻȤ롣*/
 /*** @{ */
 
 /***en
@@ -185,9 +215,9 @@ mchar__fini (void)
 /***ja
     @brief ץȤɽ魯.
 
-    ܥ #Mscript  <tt>"script"</tt> Ȥ̾ʸ
-    ѥƥΥȤƻȤ롣Υץѥƥͤϡʸ°
-    륹ץȤɽ魯ܥǤ롣
+    ܥ #Mscript  <tt>"script"</tt> 
+    Ȥ̾ʸץѥƥΥȤƻȤ롣
+    Υץѥƥͤϡʸ°륹ץȤɽ魯ܥǤ롣
 
     ץȤɽ魯ܥ̾ϡ<em>Unicode Technical Report
     #24</em> ˥ꥹȤƤΤΤ줫Ǥ롣  */
@@ -206,9 +236,9 @@ MSymbol Mscript;
 /***ja
     @brief ̾ɽ魯.
 
-    ܥ #Mname  <tt>"name"</tt> Ȥ̾ʸץ
-    ƥΥȤƻȤ롣Υץѥƥͤ C-string Ǥꡢ
-    ʸ̾ɽ魯  */
+    ܥ #Mname  <tt>"name"</tt> 
+    Ȥ̾ʸץѥƥΥȤƻȤ롣
+    ΥץѥƥͤϤʸ̾ɽ魯 C ʸǤ롣  */
 
 MSymbol Mname;
 
@@ -229,12 +259,12 @@ MSymbol Mname;
 /***ja
     @brief ̥ƥɽ魯.
 
-    ܥ #Mcategory  <tt>"category"</tt> Ȥ̾ʸ
-    ץѥƥΥȤƻȤ롣Υץѥƥͤϡб
-    <em>̥ƥ</em> ɽ魯ܥǤ롣
+    ܥ #Mcategory  <tt>"category"</tt> 
+    Ȥ̾ʸץѥƥΥȤƻȤ롣
+    Υץѥƥͤϡб <em>̥ƥ</em> ɽ魯ܥǤ롣
 
-    ̥ƥɽ魯ܥ̾ϡ<em>General Category</em>
-    άȤ Unicode ƤΤǤ롣  */
+    ̥ƥɽ魯ܥ̾ϡ<em>General Category</em> 
+    ξάȤ Unicode ƤΤǤ롣  */
 
 MSymbol Mcategory;
 
@@ -254,12 +284,12 @@ MSymbol Mcategory;
 /***ja
     @brief ɸ祯饹ɽ魯.
 
-    ܥ #Mcombining_class  <tt>"combining-class"</tt> Ȥ
-    ̾ʸץѥƥΥȤƻȤ롣Υץѥƥ
-    ϡб @e ɸ祯饹 ɽ魯Ǥ롣
+    ܥ #Mcombining_class  <tt>"combining-class"</tt> 
+    Ȥ̾ʸץѥƥΥȤƻȤ롣
+    Υץѥƥͤϡб @e ɸ祯饹 ɽ魯Ǥ롣
 
-    ɸ祯饹ɽ魯ΰ̣ϡUnicode Ƥ
-    ƱǤ롣  */
+    ɸ祯饹ɽ魯ΰ̣ϡUnicode 
+    ƤΤƱǤ롣  */
 
 MSymbol Mcombining_class;
 /*=*/
@@ -279,9 +309,9 @@ MSymbol Mcombining_class;
 /***ja
     @brief ƥɽ魯.
 
-    ܥ #Mbidi_category  <tt>"bidi-category"</tt> Ȥ̾
-    ʸץѥƥΥȤƻȤ롣Υץѥƥͤϡ
-    б @e ƥ ɽ魯ܥǤ롣
+    ܥ #Mbidi_category  <tt>"bidi-category"</tt> 
+    Ȥ̾ʸץѥƥΥȤƻȤ롣
+    Υץѥƥͤϡб @e ƥ ɽ魯ܥǤ롣
 
     ƥɽ魯ܥ̾ϡ<em>Bidirectional
     Category</em> ηȤ Unicode ƤΤǤ롣  */
@@ -307,14 +337,13 @@ MSymbol Mbidi_category;
     @brief б뾮ʸʸɽ魯.
 
     ܥ #Msimple_case_folding  <tt>"simple-case-folding"</tt> 
-    Ȥ̾ʸץѥƥΥȤƻȤ롣Υץѥƥ
-    ͤϡб뾮ʸʸǤꡢʸʸζ̵̤뤷
-    ʸӤκݤ˻Ȥ롣
+    Ȥ̾ʸץѥƥΥȤƻȤ롣
+    Υץѥƥͤϡб뾮ʸʸǤꡢʸʸζ̵̤뤷ʸӤκݤ˻Ȥ롣
 
-    ʣˡɬפȤʸǤä̤ΰʸбդ
-    ȤˤäӤǤʤˡΥץѥƥͤ @c 0xFFFF 
-    ʤ롣ξ礽ʸϡ#Mcomplicated_case_folding Ȥ
-    ʸץѥƥġ  */
+    ʣˡɬפȤʸǤä
+    ̤ΰʸбդ뤳ȤˤäӤǤʤˡΥץѥƥͤ 
+    @c 0xFFFF ˤʤ롣ξ礽ʸϡ#Mcomplicated_case_folding
+    Ȥʸץѥƥġ  */
 
 MSymbol Msimple_case_folding;
 /***en
@@ -330,9 +359,9 @@ MSymbol Msimple_case_folding;
     @brief б뾮ʸɽ魯.
 
     ܥ #Mcomplicated_case_folding  
-    <tt>"complicated-case-folding"</tt> Ȥ̾ʸץѥƥ
-    ΥȤƻȤ롣Υץѥƥͤϡб뾮ʸ󤫤
-     M-text Ǥꡢʸʸζ̵̤뤷ʸӤκݤ˻
+    <tt>"complicated-case-folding"</tt> 
+    Ȥ̾ʸץѥƥΥȤƻȤ롣
+    Υץѥƥͤϡб뾮ʸ󤫤ʤ M-text Ǥꡢʸʸζ̵̤뤷ʸӤκݤ˻
     롣
       */
 
@@ -376,36 +405,13 @@ MSymbol
 mchar_define_property (const char *name, MSymbol type)
 {
   MSymbol key = msymbol (name);
-  MCharPropRecord *record;
+  void *mdb;
 
-  record = mplist_get (char_prop_list, key);
-  if (record)
-    {
-      if (record->table)
-	M17N_OBJECT_UNREF (record->table);
-    }
+  if (mdatabase__finder)
+    mdb = (*mdatabase__finder) (Mchar_table, type, key, Mnil);
   else
-    {
-      MSTRUCT_CALLOC (record, MERROR_CHAR);
-      mplist_put (char_prop_list, key, record);
-    }
-
-  record->type = type;
-  if (mdatabase__finder
-      && (record->mdb = (*mdatabase__finder) (Mchar_table, type, key, Mnil)))
-    {
-      record->table = NULL;
-    }
-  else
-    {
-      void *default_value = NULL;
-
-      record->mdb = NULL;
-      if (type == Minteger)
-	default_value = (void *) -1;
-      record->table = mchartable (type, default_value);
-    }
-
+    mdb = NULL;
+  mchar__define_prop (key, type, mdb);
   return key;
 }
 
@@ -429,7 +435,7 @@ mchar_define_property (const char *name, MSymbol type)
     $KEY ǤΤõ
 
     @return
-     mchar_get_prop () ϸĤäץѥƥͤ
+     mchar_get_prop () ϸĤäץѥƥ֤ͤ
     Ԥ @c NULL ֤
 
     @latexonly \IPAlabel{mchar_get_prop} @endlatexonly
@@ -446,6 +452,8 @@ mchar_get_prop (int c, MSymbol key)
 {
   MCharPropRecord *record;
 
+  if (! char_prop_list)
+    return NULL;
   record = mplist_get (char_prop_list, key);
   if (! record)
     return NULL;
@@ -475,11 +483,10 @@ mchar_get_prop (int c, MSymbol key)
     @brief ʸץѥƥͤꤹ.
 
     ؿ mchar_put_prop () ϡʸ $C ʸץѥƥΤ $KEY 
-    ΤõͤȤ $VAL ꤹ롣
+    ǤΤõͤȤ $VAL ꤹ롣
 
     @return
-     mchar_put_prop () 0֤Ԥ-1
-      */
+     mchar_put_prop () 0֤Ԥ-1֤  */
 /***
     @errors
     @c MERROR_SYMBOL, @c MERROR_DB
@@ -492,6 +499,8 @@ mchar_put_prop (int c, MSymbol key, void *val)
 {
   MCharPropRecord *record;
 
+  if (! char_prop_list)
+    MERROR (MERROR_CHAR, -1);
   record = mplist_get (char_prop_list, key);
   if (! record)
     return -1;
@@ -505,6 +514,55 @@ mchar_put_prop (int c, MSymbol key, void *val)
   return mchartable_set (record->table, c, val);
 }
 
+/*=*/
+
+/***en
+    @brief Get the char-table for a character property.
+
+    The mchar_get_prop_table () function returns a char-table that
+    contains the character property whose key is $KEY.  If $TYPE is
+    not NULL, this function stores the type of the property in the
+    place pointed by $TYPE.  See mchar_define_property () for types of
+    character property.
+
+    @return
+    If $KEY is a valid character property key, this function returns a
+    char-table.  Otherwise NULL is retuned.  */
+
+/***ja
+    @brief ʸץѥƥʸơ֥.
+
+    ؿ mchar_get_prop_table () ϡ $KEY Ǥʸץѥƥ
+    ޤʸơ֥֤⤷ $TYPE  NULL ǤʤС $TYPE 
+    ؤˤʸΥץѥƥǼ롣ʸץѥƥμ
+    ˴ؤƤ mchar_define_property () 򸫤衣
+
+    @return
+    ⤷ $KEY ʸץѥƥΥǤСʸơ֥뤬֤
+    롣Ǥʤ NULL ֤롣  */
+
+MCharTable *
+mchar_get_prop_table (MSymbol key, MSymbol *type)
+{
+  MCharPropRecord *record;
+
+  if (! char_prop_list)
+    return NULL;
+  record = mplist_get (char_prop_list, key);
+  if (! record)
+    return NULL;
+  if (record->mdb)
+    {
+      record->table = (*mdatabase__loader) (record->mdb);
+      if (! record->table)
+	MERROR (MERROR_DB, NULL);
+      record->mdb = NULL;
+    }
+  if (type)
+    *type = record->type;
+  return record->table;
+}
+
 /*** @} */
 
 /*
