@@ -54,31 +54,27 @@
     @addtogroup m17nCharset
     @brief ʸåȥ֥ȤȤ˴ؤ API.
 
-    m17n 饤֥ϡ沽ʸ (CCS)  @e ʸå ȸƤ֥
-    ֥Ȥɽ롣m17n 饤֥¿沽ʸ򤢤餫
-    ݡȤƤ뤷ץꥱץबȼʸåȤ
-    ɲä뤳ȤǽǤ롣ĤʸʣʸåȤ°Ƥ
-    
+    m17n 饤֥ϡ沽ʸ (CCS)  @e ʸå 
+    ȸƤ֥֥Ȥɽ롣
+    m17n 饤֥¿沽ʸ򤢤餫᥵ݡȤƤ뤷ץꥱץबȼʸåȤɲä뤳ȤǽǤ롣
+    ĤʸʣʸåȤ°Ƥ褤
 
     m17n 饤֥ϡʲγǰ̤Ƥ:
 
-    @li @e ɥݥ ȤϡCCS θġʸФ
-    ͤǤ롣ɥݥȤϢ³ƤȤϸ¤ʤɥݥȤ
+    @li @e ɥݥ ȤϡCCS θġʸФͤǤ롣
+    ɥݥȤϢ³ƤȤϸ¤ʤɥݥȤ
     @c unsigned ˤäɽ롣̵ʥɥݥȤϥޥ
     @c MCHAR_INVALID_CODE ɽ롣
 
-    @li @e ʸǥå ȤϡCCS ǳʸ˳Ƥ
-    줿ǥåǤ롣ʸǥå N ʸϡCCS 
-    ʸ򥳡ɥݥȽ¤٤Ȥ N ܤ˸롣CCS 
-    ʸǥåϢ³Ƥꡢ0 Ϥޤ롣
+    @li @e ʸǥå ȤϡCCS ǳʸ˳Ƥ줿ǥåǤ롣
+    ʸǥå N ʸϡCCS ʸ򥳡ɥݥȽ¤٤Ȥ N ܤ˸롣
+    CCS ʸǥåϢ³Ƥꡢ0 Ϥޤ롣
 
-    @li @e ʸ Ȥϡm17n 饤֥ˤʸɽǤ
-    ꡢ21 ӥåȰʾĹդǤ롣
+    @li @e ʸ Ȥϡm17n 饤֥ˤʸɽǤꡢ21 ӥåȰʾĹդǤ롣
 
-    ʸåȥ֥ȤϡʸåȤ°ʸΥɥݥ
-    ȤʸɤȤδ֤Ѵꤹ롣ɥݥȤʸ
-    ɤؤѴ @e ǥ ȸƤӡʸɤ饳ɥݥȤؤ
-    Ѵ @e 󥳡 ȸƤ֡  */
+    ʸåȥ֥ȤϡʸåȤ°ʸΥɥݥȤʸɤȤδ֤Ѵꤹ롣
+    ɥݥȤʸɤؤѴ @e ǥ
+    ȸƤӡʸɤ饳ɥݥȤؤѴ @e 󥳡 ȸƤ֡  */
 
 /*=*/
 #if !defined (FOR_DOXYGEN) || defined (DOXYGEN_INTERNAL_MODULE)
@@ -647,7 +643,7 @@ mcharset__load_from_database ()
 /***ja
     @brief ̵ʥɥݥ.
 
-    ޥ #MCHAR_INVALID_CODE ̵ʥɥݥȤͿ롣  */
+    ޥ #MCHAR_INVALID_CODE ̵ʥɥݥȤ򼨤  */
 
 #define MCHAR_INVALID_CODE
 #endif
@@ -662,9 +658,9 @@ mcharset__load_from_database ()
 /***ja
     @brief ܥ @c Mcharset.
 
-    ǥɤ줿 M-text ϡ @c Mcharset Ǥ褦ʥƥ
-    ץѥƥġܥ @c Mcharset  <tt>"charset"</tt> Ȥ
-    ̾ġ  */
+    ǥɤ줿 M-text ϡ @c Mcharset
+    Ǥ褦ʥƥȥץѥƥġ
+    ܥ @c Mcharset  <tt>"charset"</tt> Ȥ̾ġ  */
 
 MSymbol Mcharset;
 /*=*/
@@ -675,7 +671,7 @@ MSymbol Mcharset;
     Each of the following symbols represents a predefined charset.  */
 
 /***ja
-    @name ѿ: ʸåȤɽѤߥܥ
+    @name ѿ: ʸåȤɽѤߥܥ.
 
     ʲγƥܥϡѤʸåȤɽ롣  */
 /*=*/
@@ -704,8 +700,8 @@ MSymbol Mcharset_ascii;
 /***ja
     @brief ISO/IEC 8859-1:1998 ʸåȤɽ륷ܥ.
 
-    ܥ #Mcharset_iso_8859_1  <tt>"iso-8859-1"</tt> Ȥ̾
-    ISO/IEC 8859-1:1998 ʸåȤɽ롣
+    ܥ #Mcharset_iso_8859_1  <tt>"iso-8859-1"</tt> 
+    Ȥ̾ISO/IEC 8859-1:1998 ʸåȤɽ롣
     */
 
 MSymbol Mcharset_iso_8859_1;
@@ -718,8 +714,8 @@ MSymbol Mcharset_iso_8859_1;
 /***ja
     @brief Unicode ʸåȤɽ륷ܥ.
 
-    ܥ #Mcharset_unicode  <tt>"unicode"</tt> Ȥ̾
-    Unicode ʸåȤɽ롣 */
+    ܥ #Mcharset_unicode  <tt>"unicode"</tt> 
+    Ȥ̾Unicode ʸåȤɽ롣 */
 
 MSymbol Mcharset_unicode;
 
@@ -745,14 +741,16 @@ MSymbol Mcharset_m17n;
     The symbol #Mcharset_binary has name <tt>"binary"</tt> and
     represents the fake charset which the decoding functions put to an
     M-text as a text property when they encounter an invalid byte
-    (sequence).  See @ref m17nConv for more details.  */
+    (sequence).  
+
+    See @ref m17nConv for more details.  */
 
 /***ja
     @brief ǥɤǤʤʸʸåȤɽ륷ܥ.
 
-    ܥ #Mcharset_binary  <tt>"binary"</tt> Ȥ̾
-     (fake) ʸåȤɽ롣ǥɴؿϡM-text Υƥ
-    ȥץѥƥȤơ̵ʥХȡʥ󥹡ˤ֤ղä롣
+    ܥ #Mcharset_binary  <tt>"binary"</tt> 
+    Ȥ̾ (fake) ʸåȤɽ롣
+    ǥɴؿϡM-text ΥƥȥץѥƥȤơ̵ʥХȡʥ󥹡ˤ֤ղä롣
 
      ܺ٤ @ref m17nConv ȤΤȡ */
 
@@ -771,8 +769,8 @@ MSymbol Mcharset_binary;
 /***ja
     @name ѿ: mchar_define_charset ѤΥѥ᡼
 
-    ϡؿ mchar_define_charset () ѤΥѥ᡼Ȥ
-    Ȥ륷ܥǤ롣 ܤϤδؿβ򻲾ȤΤȡ*/
+    ϡؿ mchar_define_charset () ѤΥѥ᡼ȤƻȤ륷ܥǤ롣 
+    ܤϤδؿβ򻲾ȤΤȡ*/
 /*** @{ */
 /*=*/
 
@@ -780,7 +778,8 @@ MSymbol Mcharset_binary;
     Parameter key for mchar_define_charset () (which see). */ 
 
 /***ja
-    ؿ mchar_define_charset () ѤΥѥ᡼. */ 
+    ؿ mchar_define_charset () ѤΥѥ᡼. 
+    ܤϤδؿβ򻲾ȤΤȡ*/ 
 
 MSymbol Mmethod;
 MSymbol Mdimension;
@@ -815,12 +814,12 @@ MSymbol Maliases;
 /***ja
     @name ѿ: ʸåȤΥ᥽åɻ˻Ȥ륷ܥ
 
-    ϡʸåȤ @e ᥽å ꤹ뤿Ѥߥ
-    ǤꡢʸåȤ #Mmethod ѥ᡼ͤȤʤ뤳ȤǤ롣
+    ϡʸåȤ @e ᥽å ꤹ뤿ѤߥܥǤꡢʸåȤ
+    #Mmethod ѥ᡼ͤȤʤ뤳ȤǤ롣
     ͤϴؿ mchar_define_charset () ΰȤƻȤ롣
 
-    ᥽åɤȤϡɥݥȤʸɤѴݤΤ
-    ȤǤ롣ܤϴؿ mchar_define_charset () β򻲾ȤΤȡ  */
+    ᥽åɤȤϡɥݥȤʸɤѴݤΤȤǤ롣
+    ܤϴؿ mchar_define_charset () β򻲾ȤΤȡ  */
 /*** @{ */
 /*=*/
 /***en
@@ -841,10 +840,8 @@ CHARACTER-CODE = CODE-POINT - MIN-CODE + MIN-CHAR
 /***ja
     @brief եåȷΥ᥽åɤ򼨤ܥ.
 
-    ܥ #Moffset  <tt>"offset"</tt> Ȥ̾ʸ
-    Ȥ #Mmethod ѥ᡼ͤȤѤ줿ˤϡɥݥ
-    ȤʸåȤʸɤδ֤Ѵʲμ˽äƹԤ뤳
-    ̣롣
+    ܥ #Moffset  <tt>"offset"</tt> Ȥ̾ʸåȤ
+    #Mmethod ѥ᡼ͤȤѤ줿ˤϡɥݥȤʸåȤʸɤδ֤Ѵʲμ˽äƹԤ뤳Ȥ̣롣
 
 @verbatim
 ʸ = ɥݥ - MIN-CODE + MIN-CHAR
@@ -867,10 +864,8 @@ MSymbol Moffset;
 /***ja @brief ޥå׷Υ᥽åɤ򼨤ܥ.
 
     ܥ #Mmap  <tt>"map"</tt> Ȥ̾ʸåȤ 
-    #Mmethod ѥ᡼ͤȤѤ줿ˤϡɥݥȤ
-    ʸåȤʸɤδ֤Ѵޥåפ򻲾Ȥ뤳ȤˤäƹԤ
-    뤳Ȥ̣롣ޥåפ #Mmapfile ѥ᡼ȤͿʤ
-    Фʤʤ */
+    #Mmethod ѥ᡼ͤȤѤ줿ˤϡɥݥȤʸåȤʸɤδ֤Ѵޥåפ򻲾Ȥ뤳ȤˤäƹԤ뤳Ȥ̣롣
+    ޥåפ #Mmapfile ѥ᡼ȤͿʤФʤʤ */
 
 MSymbol Mmap;
 /*=*/
@@ -898,23 +893,20 @@ CHARACTER-CODE = CODE-POINT - MIN-CODE + LOWEST-CHAR-CODE
 
 /***ja @brief ˥եΥ᥽åɤ򼨤ܥ.
 
-    ܥ #Minherit  <tt>"unify"</tt> Ȥ̾ʸ
-    Ȥ #Mmethod ѥ᡼ͤȤѤ줿ˤϡɥݥ
-    ȤʸåȤʸɤδ֤ѴޥåפλȤȥեåȤ
-    Ȥ߹碌ˤäƹԤ뤳Ȥ̣롣ޥåפ #Mmapfile ѥ
-    ᡼ȤͿʤФʤʤμγʸåȤˤϡʸ
-    ФϢ³륳ɥڡ줾Ƥ롣
+    ܥ #Minherit  <tt>"unify"</tt> Ȥ̾ʸåȤ 
+    #Mmethod ѥ᡼ͤȤѤ줿ˤϡɥݥȤʸåȤʸɤδ֤ѴޥåפλȤȥեåȤȤ߹碌ˤäƹԤ뤳Ȥ̣롣
+    ޥåפ #Mmapfile ѥ᡼ȤͿʤФʤʤ
+    μγʸåȤˤϡʸФϢ³륳ɥڡ줾Ƥ롣
 
-    ɥݥȤޥåפ˴ޤޤƤСѴϥޥå׻Ȥˤäƹ
-    롣ǤʤСʲμ˽
+    ɥݥȤޥåפ˴ޤޤƤСѴϥޥå׻ȤˤäƹԤ롣
+    ǤʤСʲμ˽
 
 @verbatim
 CHARACTER-CODE = CODE-POINT - MIN-CODE + LOWEST-CHAR-CODE
 @endverbatim
     
     ǡMIN-CODE ʸåȤ #Mmin_code ѥ᡼ͤǤꡢ
-    LOWEST-CHAR-CODE ϳƤ줿ɥڡκǤ⾮ʸ
-    ɤǤ롣
+    LOWEST-CHAR-CODE ϳƤ줿ɥڡκǤ⾮ʸɤǤ롣
     */
 
 MSymbol Munify;
@@ -940,20 +932,18 @@ CHARACTER-CODE = PARENT-CODE (CODE-POINT) + SUBSET-OFFSET
 
 /***ja @brief ֥åȷΥ᥽åɤ򼨤ܥ.
 
-    ܥ #Msubset  <tt>"subset"</tt> Ȥ̾ʸ
-    Ȥ #Mmethod ѥ᡼ͤȤѤ줿ˤϡʸ
-    Ȥ̤ʸåȡʿʸåȡˤʬǤ뤳Ȥ̣롣
+    ܥ #Msubset  <tt>"subset"</tt> Ȥ̾ʸåȤ
+    #Mmethod ѥ᡼ͤȤѤ줿ˤϡʸåȤ̤ʸåȡʿʸåȡˤʬǤ뤳Ȥ̣롣
     ʸåȤ #Mparents ѥ᡼ˤäͿʤƤϤʤʤ
-    ɥݥȤʸåȤʸɤδ֤ѴϡǰŪˤ
-    ʲμ˽
+    ɥݥȤʸåȤʸɤδ֤ѴϡǰŪˤϰʲμ˽
 
 @verbatim
 CHARACTER-CODE = PARENT-CODE (CODE-POINT) + SUBSET-OFFSET
 @endverbatim
 
-     PARENT-CODE  CODE-POINT οʸåǤʸɤ
-    ֤ؿǤꡢSUBSET-OFFSET  #Msubset_offset ѥ᡼Ϳ
-    ͤǤ롣
+     PARENT-CODE  CODE-POINT 
+    οʸåǤʸɤ֤ؿǤꡢSUBSET-OFFSET  
+    #Msubset_offset ѥ᡼ͿͤǤ롣
     */
 
 MSymbol Msubset;
@@ -970,11 +960,9 @@ MSymbol Msubset;
 /***ja
     @brief ѡåȷΥ᥽åɤ򼨤ܥ.
 
-    ܥ #Msuperset  <tt>"superset"</tt> Ȥ̾ʸ
-    åȤ #Mmethod ѥ᡼ͤȤѤ줿ˤϡʸ
-    åȤ̤ʸåȡʿʸåȡˤξ̽Ǥ뤳Ȥ̣
-    롣ʸåȤ #Mparents ѥ᡼ˤäͿʤƤϤ
-    ʤ
+    ܥ #Msuperset  <tt>"superset"</tt> Ȥ̾ʸåȤ
+    #Mmethod ѥ᡼ͤȤѤ줿ˤϡʸåȤ̤ʸåȡʿʸåȡˤξ̽Ǥ뤳Ȥ̣롣
+    ʸåȤ #Mparents ѥ᡼ˤäͿʤƤϤʤʤ
     */
 
 MSymbol Msuperset;
@@ -1054,7 +1042,7 @@ MSymbol Msuperset;
 
     If the method is #Mmap or #Munify, a data that contains
     mapping information is added to the m17n database by calling
-    mdatabase_define () with the value as an argument $EXTRA_INFO,
+    the function mdatabase_define () with the value as an argument $EXTRA_INFO,
     i.e. the value is used as a file name of the data.
 
     Otherwise, this parameter is ignored.
@@ -1098,26 +1086,25 @@ MSymbol Msuperset;
     <li>  #Mmethod ͤܥλ
 
     ͤϡ#Moffset, #Mmap (ǥե), #Munify, #Msubset,
-    #Msuperset Τ줫ǤꡢʸåȤΥɥݥȤǥɡ
-    󥳡ɤݤΥ᥽åɤꤹ롣
+    #Msuperset Τ줫ǤꡢʸåȤΥɥݥȤǥɡ󥳡ɤݤΥ᥽åɤꤹ롣
 
     <li>  #Mdimension ͤͤλ
 
-    ͤϡ1 (ǥե), 2, 3, 4 Τ줫ǤꡢʸåȤΥ
-    ɥݥȤμǤ롣
+    ͤϡ1 (ǥե), 2, 3, 4 
+    Τ줫ǤꡢʸåȤΥɥݥȤμǤ롣
 
     <li>  #Mmin_range ͤͤλ
 
-    ͤϥɥݥȤκǾͤǤ롣ʤͤ N ܤΥ
-    ȤϤʸåȤΥɥݥȤ N ܤΥХȤκǾΤ
-    Ȥʤ롣ǥեͤ 0 
+    ͤϥɥݥȤκǾͤǤ롣ʤͤ N 
+    ܤΥХȤϤʸåȤΥɥݥȤ N ܤΥХȤκǾΤΤȤʤ롣
+    ǥեͤ 0 
 
     <li>  #Mmax_range ͤͤλ
 
-    ͤϥɥݥȤκͤǤ롣ʤͤ N ܤΥ
-    ȤϤʸåȤΥɥݥȤ N ܤΥХȤκΤ
-    Ȥʤ롣ǥեͤϡɥݥȤμ 1, 2, 3, 4 λ
-    줾 0xFF, 0xFFFF, 0xFFFFFF, 0xFFFFFFFF 
+    ͤϥɥݥȤκͤǤ롣ʤͤ N 
+    ܤΥХȤϤʸåȤΥɥݥȤ N ܤΥХȤκΤΤȤʤ롣
+    ǥեͤϡɥݥȤμ 1, 2, 3, 4 λ줾
+    0xFF, 0xFFFF, 0xFFFFFF, 0xFFFFFFFF 
 
     <li>  #Mmin_code ͤͤλ
 
@@ -1143,9 +1130,9 @@ MSymbol Msuperset;
     <li>   #Mrevision ͤͤλ
 
     ͤ The International Registry ϿƤ @e revision @e
-    number Ǥꡢ0..127 Ǥ롣ʸåȤϿƤʤˤ
-    ̵ͤ뤵롣 0  revision number ¸ߤʤȤ̣
-    롣
+    number Ǥꡢ0..127 Ǥ롣
+    ʸåȤϿƤʤˤϤ̵ͤ뤵롣
+    0  revision number ¸ߤʤȤ̣롣
 
     <li>   #Mmin_char ͤͤλ
 
@@ -1153,36 +1140,35 @@ MSymbol Msuperset;
 
     <li>  #Mmapfile ͤ M-text λ
 
-    ᥽åɤ #Mmap  #Munify λؿ mdatabase_define () 򤳤
-    ͤ $EXTRA_INFO ȤƸƤ֤Ȥˤäơޥåԥ󥰤˴ؤǡ
-     m17n ǡ١ɲä롣ʤͤϥǡե
-    ̾Ǥ롣
+    ᥽åɤ #Mmap  #Munify λؿ mdatabase_define () 
+    򤳤ͤ $EXTRA_INFO ȤƸƤ֤Ȥˤäơޥåԥ󥰤˴ؤǡ
+    m17n ǡ١ɲä롣
+    ʤͤϥǡե̾Ǥ롣
 
     ǤʤСΥѥ᡼̵뤵롣
 
     <li>  #Mparents ͤ plist λ
 
-    ᥽åɤ #Msubset ʤСͤĹ 1  plist ǤꡢͤϤ
-    ʸåȤξ̽ȤʤʸåȤ򼨤ܥǤ롣
+    ᥽åɤ #Msubset ʤСͤĹ 1  plist 
+    ǤꡢͤϤʸåȤξ̽ȤʤʸåȤ򼨤ܥǤ롣
 
-    ᥽åɤ #Msuperset ʤСͤĹ 8 ʲ plist Ǥꡢ
-    ͤϤʸåȤβ̽ǤʸåȤ򼨤ܥǤ
-    롣
+    ᥽åɤ #Msuperset ʤСͤĹ 8 ʲ plist 
+    ǤꡢͤϤʸåȤβ̽ǤʸåȤ򼨤ܥǤ롣
 
     ǤʤСΥѥ᡼̵뤵롣
 
     <li>   #Mdefine_coding ͤܥλ
 
-    ʸåȤμ 1 ʤСͤ #Mcharset פƱ̾Υ
-    ɷϤ뤫ɤꤹ롣ͤ #Mnil ʳξ롣
+    ʸåȤμ 1 ʤСͤ #Mnil ʳξ #Mcharset 
+    Ʊ̾ĥɷϤ롣
 
     ǤʤСΥѥ᡼̵뤵롣
 
     </ul>
 
     @return 
-    Сmchar_define_charset()  $NAME Ȥ̾
-    Υ ܥ֤Ǥʤ #Mnil ֤ѿ 
+    Сmchar_define_charset()  $NAME 
+    Ȥ̾Υܥ֤Ǥʤ #Mnil ֤ѿ 
     #merror_code ˥顼ɤꤹ롣*/
 
 /***
@@ -1306,12 +1292,11 @@ mchar_define_charset (const char *name, MPlist *plist)
 /***ja
     @brief ʸå̾褹.
 
-    ؿ mchar_resolve_charset ()  $SYMBOL ʸåȤ򼨤Ƥ
-    Ф֤
+    ؿ mchar_resolve_charset ()  $SYMBOL 
+    ʸåȤ򼨤ƤФ֤
 
-    ǤʤС$SYMBOL ʸå̾Ȥ줬ʸ
-    Ȥ򼨤ƤƤΤ֤ǤʤС#Mnil 
-    ֤ */
+    ǤʤС$SYMBOL ʸå̾Ȥ줬ʸåȤ򼨤ƤƤΤ֤
+    ǤʤС#Mnil ֤ */
 
 MSymbol
 mchar_resolve_charset (MSymbol symbol)
@@ -1339,9 +1324,9 @@ mchar_resolve_charset (MSymbol symbol)
 /***ja
     @brief ʸåȤɽ魯ܥ󤹤.
 
-    ؿ mchar_list_charsets () ϡʸåȤ򼨤ܥ¤٤
-    ꡢ$SYMBOLS ǥݥȤ줿ˤؤΥݥ󥿤
-    Ĺ֤ */
+    ؿ mchar_list_charsets () 
+    ϡʸåȤ򼨤ܥ¤٤ꡢ$SYMBOLS 
+    ǥݥȤ줿ˤؤΥݥ󥿤֤Ĺ֤ */
 
 int
 mchar_list_charset (MSymbol **symbols)
@@ -1370,12 +1355,12 @@ mchar_list_charset (MSymbol **symbols)
 /***ja
     @brief ɥݥȤǥɤ.
 
-    ؿ mchar_decode () ϡܥ $CHARSET_NAME Ǽʸ
-     $CODE ȤɥݥȤǥɤʸɤ롣
+    ؿ mchar_decode () ϡܥ $CHARSET_NAME Ǽʸå
+    $CODE ȤɥݥȤǥɤʸɤ롣
 
     @return
-    ǥɤСmchar_decode () ϥǥɤ줿ʸɤ
-    ֤Ǥʤ -1 ֤  */
+    ǥɤСmchar_decode () ϥǥɤ줿ʸɤ֤
+    Ǥʤ -1 ֤  */
 
 /***
     @seealso
@@ -1410,8 +1395,8 @@ mchar_decode (MSymbol charset_name, unsigned code)
     $CHARSET_NAME Ǽʸåˤ륳ɥݥȤ롣
 
     @return
-    󥳡ɤСmchar_encode () ϥ󡼥ɤ줿ɥݥ
-    Ȥ֤Ǥʤ #MCHAR_INVALID_CODE ֤  */
+    󥳡ɤСmchar_encode () ϥ󡼥ɤ줿ɥݥȤ֤
+    Ǥʤ #MCHAR_INVALID_CODE ֤  */
 
 /***
     @seealso
@@ -1449,17 +1434,17 @@ mchar_encode (MSymbol charset_name, int c)
 /***ja
     @brief ꤷʸåȤΤ٤ƤʸФƴؿƤ.
 
-    ؿ mcharset_map_chars ()  $CHARSET_NAME Ȥ̾ʸ
-    Τ٤ƤʸФ $FUNC Ƥ֡ƤӽФϰʸǤϤ
-    Ϣ³ʸΤޤȤޤñ̤ǹԤʤ롣
+    ؿ mcharset_map_chars ()  $CHARSET_NAME 
+    Ȥ̾ʸåΤ٤ƤʸФ $FUNC Ƥ֡
+    ƤӽФϰʸǤϤʤϢ³ʸΤޤȤޤñ̤ǹԤʤ롣
 
     ؿ $FUNC ˤ$FROM, $TO, $ARG ΣϤ롣$FROM  $TO 
-     $CHARSET ʸɤϰϤꤹ롣$ARG  $FUNC_ARG Ʊ
-    Ǥ롣
+     $CHARSET ʸɤϰϤꤹ롣$ARG  $FUNC_ARG 
+    ƱǤ롣
 
     @return
-     mcharset_map_chars ()  0 ֤Ǥʤ 
-    -1 ֤ѿ #merror_code ˥顼ɤꤹ롣  */
+     mcharset_map_chars ()  0 ֤
+    Ǥʤ -1 ֤ѿ #merror_code ˥顼ɤꤹ롣  */
 
 /*** 
     @errors
