@@ -70,36 +70,36 @@
     m17n 饤֥ɬפ˱ưŪ @e m17n @e ǡ١ 
     롣ޤץꥱץȼΥǡ 
     m17n ǡ١ɲäưŪ˼뤳ȤǤ롣m17n 
-    ǡ١ˤʣ¿ͤʥǡޤޤƤꡢƥǡϣĤ
-     TAG0, TAG1, TAG2, TAG3ʤ٤ƥܥˤˤäƼ̤롣
+    ǡ١ˤʣ¿ͤʥǡޤޤƤꡢƥǡ
+    TAG0, TAG1, TAG2, TAG3ʤ٤ƥܥˤΣĤΥˤäƼ̤롣
 
-    TAG0 ϥǡ١ΥǡΥפʲΤ褦˻ꤹ롣
+    TAG0 ˤäơǡ١ΥǡΥפϼΤ褦˻ꤵ롣
 
     @li 
-    TAG0  #Mchar_table Ǥǡ @e chartable ȸƤФ졢
-    ʸ˴ؤ󶡤롣ξ TAG1 Ͼμꤹ
-    ܥǤꡢ#Msymbol, #Minteger, #Mstring, #Mtext, #Mplist 
-    줫Ǥ롣TAG2  TAG3 ǤդΥܥǤ褤
+    TAG0  #Mchar_table Ǥǡ @e chartable 
+    ȸƤФ졢ʸ˴ؤ󶡤롣ξ
+    TAG1 Ͼμꤹ륷ܥǤꡢ#Msymbol, #Minteger, #Mstring,
+    #Mtext, #Mplist Τ줫Ǥ롣TAG2  TAG3 ǤդΥܥǤ褤
 
     @li 
-    TAG0  #Mcharset Ǥǡ @e charset ȸƤФ졢ʸ
-    åѤΥǥɡ󥳡ɥޥåפ󶡤롣ξ TAG1 
-    ʸåȤΥܥǤʤФʤʤTAG2  TAG3 ǤդΥ
-    ܥǤ褤
+    TAG0  #Mcharset Ǥǡ @e charset 
+    ȸƤФ졢ʸåѤΥǥɡ󥳡ɥޥåפ󶡤롣ξ TAG1
+    ʸåȤΥܥǤʤФʤʤTAG2  TAG3
+    ǤդΥܥǤ褤
 
     @li
     TAG0  #Mchar_table Ǥ #Mcharset Ǥʤ硢Υǡ @e
-    plist  Ǥ롣ܺ٤˴ؤƤϴؿ mdatabase_load () 
-    ȤΤȡξ TAG1TAG2TAG3 ǤդΥܥǤ褤
+    plist Ǥ롣ܺ٤˴ؤƤϴؿ mdatabase_load () 
+    򻲾ȤΤȡξ TAG1TAG2TAG3 ǤդΥܥǤ褤
 
-    Υĥǡ١ \<TAG0, TAG1, TAG2, TAG3\> Ȥ
-    ɽ魯
+    Υĥǡ١ \<TAG0, TAG1, TAG2, TAG3\> 
+    Ȥɽ
 
-    ץꥱץϡޤؿ mdatabase_find () Ȥäƥǡ
-    ١˴ؤݻ륪֥ȡ#MDatabase ˤؤ
-    ݥ󥿤롣顢 mdatabase_load () ˤäƼº
-    ˥ǡ١ɤ롣¤ #MDatabase Ȥɤ
-    Ƥ뤫ϡץꥱץफϸʤ
+    ץꥱץϡޤؿ mdatabase_find () 
+    Ȥäƥǡ١˴ؤݻ륪֥ȡ#MDatabase
+    ˤؤΥݥ󥿤롣顢 mdatabase_load () 
+    ˤäƼºݤ˥ǡ١ɤ롣¤ #MDatabase 
+    ȤɤƤ뤫ϡץꥱץफϸʤ
 
     @latexonly \IPAlabel{database} @endlatexonly
 */
@@ -119,6 +119,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <limits.h>
+#include <glob.h>
 
 #include "m17n.h"
 #include "m17n-misc.h"
@@ -134,8 +135,9 @@
 #define MDB_DIR "mdb.dir"
 #define MDB_DIR_LEN 8
 
-/** List of database directories.  */ 
-static MPlist *mdb_dir_list;
+#define MAX_TIME(TIME1, TIME2) ((TIME1) >= (TIME2) ? (TIME1) : (TIME2))
+
+static MSymbol Masterisk;
 
 /** Structure for a data in the m17n database.  */
 
@@ -155,6 +157,12 @@ struct MDatabase
   void *extra_info;
 };
 
+typedef struct
+{
+  char *filename;
+  time_t time;
+} MDatabaseInfo;
+
 /** List of all data.  */
 struct MDatabaseList
 {
@@ -249,13 +257,13 @@ load_chartable (FILE *fp, MSymbol type)
 	i++, to = read_number (buf, &i);
       else
 	to = from;
-      if (from < 0 || to < 0)
-	goto label_error;
+      if (from < 0 || to < from)
+	continue;
 
       while (buf[i] && isspace ((unsigned) buf[i])) i++;
       c = buf[i];
       if (!c)
-	break;
+	continue;
 
       if (type == Mstring)
 	{
@@ -294,6 +302,18 @@ load_chartable (FILE *fp, MSymbol type)
 	}
       else if (type == Msymbol)
 	{
+	  char *p = buf + i;
+
+	  while (*p && ! isspace (*p)) 
+	    {
+	      if (*p == '\\' && p[1] != '\0')
+		{
+		  memmove (p, p + 1, buf + len - (p + 1));
+		  len--;
+		}
+	      p++;
+	    }
+	  *p = '\0';
 	  if (! strcmp (buf + i, "nil"))
 	    val = (void *) Mnil;
 	  else
@@ -412,30 +432,56 @@ gen_database_name (char *buf, MSymbol *tags)
   return buf;
 }
 
-static void *
-load_database (MSymbol *tags, void *extra_info)
+static char *
+get_database_filename (MDatabaseInfo *db_info)
 {
-  FILE *fp;
-  char *filename = (char *) extra_info;
-  void *value;
+  char *filename = NULL;
+  struct stat buf;
 
-  if (filename[0] == '/')
-    fp = fopen (filename, "r");
+  if (db_info->filename[0] == '/')
+    {
+      if (stat (db_info->filename, &buf) == 0)
+	{
+	  filename = db_info->filename;
+	  db_info->time = MAX_TIME (buf.st_mtime, buf.st_ctime);
+	}
+    }
   else
     {
       MPlist *plist;
-      char path[PATH_MAX];
+      char *path;
+      int filelen = strlen (db_info->filename);
+      USE_SAFE_ALLOCA;
 
-      MPLIST_DO (plist, mdb_dir_list)
+      MPLIST_DO (plist, mdatabase__dir_list)
 	{
-	  strcpy (path, (char *) MPLIST_VAL (plist));
-	  strcat (path, filename);
-	  fp = fopen (path, "r");
-	  if (fp)
-	    break;
+	  MDatabaseInfo *dir_info = MPLIST_VAL (plist);
+	  int require = strlen (dir_info->filename) + filelen + 1;
+
+	  SAFE_ALLOCA (path, require);
+	  strcpy (path, dir_info->filename);
+	  strcat (path, db_info->filename);
+	  if (stat (path, &buf) == 0)
+	    {
+	      free (db_info->filename);
+	      filename = db_info->filename = strdup (path);
+	      db_info->time = MAX_TIME (buf.st_mtime, buf.st_ctime);
+	      break;
+	    }
 	}
+      SAFE_FREE (path);
     }
-  if (! fp)
+  return filename;
+}
+
+static void *
+load_database (MSymbol *tags, void *extra_info)
+{
+  char *filename = get_database_filename ((MDatabaseInfo *) extra_info);
+  FILE *fp;
+  void *value;
+
+  if (! filename || ! (fp = fopen (filename, "r")))
     MERROR (MERROR_DB, NULL);
 
   if (tags[0] == Mchar_table)
@@ -443,7 +489,7 @@ load_database (MSymbol *tags, void *extra_info)
   else if (tags[0] == Mcharset)
     value = load_charset (fp, tags[1]);
   else
-    value = mplist__from_file (fp);
+    value = mplist__from_file (fp, NULL);
   fclose (fp);
 
   if (! value)
@@ -452,76 +498,91 @@ load_database (MSymbol *tags, void *extra_info)
 }
 
 
-/** Copy DIRNAME to a newly allocated memory and return it.  If
+/** If DIRNAME is a readable directory, allocate MDatabaseInfo and
+    copy DIRNAME to a newly allocated memory and return it.  If
     DIRNAME does not end with a slash, append a slash to the new memory.  */
 
-static char *
-duplicate_dirname (char *dirname)
+static MDatabaseInfo *
+get_dir_info (char *dirname)
 {
   struct stat buf;
   int len;
-  char *str;
+  MDatabaseInfo *dir_info;
 
   if (! dirname
-      || stat (dirname, &buf) < 0)
+      || stat (dirname, &buf) < 0
+      || ! (buf.st_mode & S_IFDIR))
     return NULL;
 
+  MSTRUCT_MALLOC (dir_info, MERROR_DB);
   len = strlen (dirname);
-  MTABLE_MALLOC (str, len + 2, MERROR_DB);
-  memcpy (str, dirname, len + 1);
-  if (str[len - 1] != '/')
+  MTABLE_MALLOC (dir_info->filename, len + 2, MERROR_DB);
+  memcpy (dir_info->filename, dirname, len + 1);
+  if (dir_info->filename[len - 1] != '/')
     {
-      str[len] = '/';
-      str[len + 1] = '\0';
+      dir_info->filename[len] = '/';
+      dir_info->filename[len + 1] = '\0';
     }
-  return str;
+  /* Set this to zero so that the first call of update_database_list
+     surely checks this directory.  */
+  dir_info->time = 0;
+  return dir_info;
 }
 
-
-/* Internal API */
-
-int
-mdatabase__init ()
+static MDatabase *
+find_database (MSymbol tag0, MSymbol tag1, MSymbol tag2, MSymbol tag3)
 {
-  char *dir;
   int i;
-  MPlist *plist;
-  FILE *fp;
 
-  Mchar_table = msymbol ("char-table");
-
-  mdb_dir_list = mplist ();
-  /** The macro M17NDIR specifies a directory where the system-wide
-    MDB_DIR file exists.  */
-  if ((dir = duplicate_dirname (M17NDIR)))
-    mplist_set (mdb_dir_list, Mt, dir);
-
-  /* The variable mdatabase_dir specifies a directory where an
-     application program specific MDB_DIR file exists.  */
-  if ((dir = duplicate_dirname (mdatabase_dir)))
-    mplist_push (mdb_dir_list, Mt, dir);
-
-  /* The environment variable M17NDIR (if non-NULL) specifies a
-     directory where a user specific MDB_DIR file exists.  */
-  if ((dir = duplicate_dirname (getenv ("M17NDIR"))))
-    mplist_push (mdb_dir_list, Mt, dir);
-
-  MLIST_INIT1 (&mdb_list, mdbs, 256);
-  MPLIST_DO (plist, mdb_dir_list)
+  for (i = 0; i < mdb_list.used; i++)
     {
-      MPlist *pl, *p;
-      int len;
-      char path[PATH_MAX];
+      MDatabase *mdb = mdb_list.mdbs + i;
 
-      dir = (char *) MPLIST_VAL (plist);
-      len = strlen (dir);
+      if (tag0 == mdb->tag[0]
+	  && tag1 == mdb->tag[1]
+	  && tag2 == mdb->tag[2]
+	  && tag3 == mdb->tag[3])
+	return mdb;
+    }
+  return NULL;
+}
+
+static void
+update_database_list ()
+{
+  MPlist *plist;
+  char *path;
+  USE_SAFE_ALLOCA;
+
+  /* Usually this avoids path to be reallocated.  */
+  SAFE_ALLOCA (path, 256);
+
+  MPLIST_DO (plist, mdatabase__dir_list)
+    {
+      MDatabaseInfo *dir_info = MPLIST_VAL (plist);
+      struct stat statbuf;
+      MPlist *pl, *p;
+      int i, j, len;
+      FILE *fp;
+
+      if (stat (dir_info->filename, &statbuf) < 0)
+	continue;
+      if (dir_info->time >= statbuf.st_ctime
+	  && dir_info->time >= statbuf.st_mtime)
+	continue;
+      dir_info->time = MAX_TIME (statbuf.st_ctime, statbuf.st_mtime);
+      len = strlen (dir_info->filename);
+#ifdef PATH_MAX
       if (len + MDB_DIR_LEN >= PATH_MAX)
 	continue;
-      memcpy (path, dir, len);
+#endif	/* PATH_MAX */
+      SAFE_ALLOCA (path, len + MDB_DIR_LEN + 1);
+      memcpy (path, dir_info->filename, len);
       memcpy (path + len, MDB_DIR, MDB_DIR_LEN);
+      path[len + MDB_DIR_LEN] = '\0';
       if (! (fp = fopen (path, "r")))
 	continue;
-      pl = mplist__from_file (fp);
+      pl = mplist__from_file (fp, NULL);
       fclose (fp);
       if (! pl)
 	continue;
@@ -529,35 +590,164 @@ mdatabase__init ()
 	{
 	  MDatabase mdb;
 	  MPlist *p1;
+	  MText *mt;
 	  int nbytes;
+	  int with_wildcard = 0;
 
 	  if (! MPLIST_PLIST_P (p))
 	    continue;
+	  p1 = MPLIST_PLIST (p);
+	  if (! MPLIST_SYMBOL_P (p1))
+	      continue;
 	  for (i = 0, p1 = MPLIST_PLIST (p);
 	       i < 4 && MPLIST_KEY (p1) == Msymbol;
 	       i++, p1 = MPLIST_NEXT (p1))
-	    mdb.tag[i] = MPLIST_SYMBOL (p1);
+	    with_wildcard |= ((mdb.tag[i] = MPLIST_SYMBOL (p1)) == Masterisk);
 	  if (i == 0
+	      || mdb.tag[0] == Masterisk
 	      || ! MPLIST_MTEXT_P (p1))
 	    continue;
 	  for (; i < 4; i++)
 	    mdb.tag[i] = Mnil;
-	  if (mdatabase_find (mdb.tag[0], mdb.tag[1],
-			      mdb.tag[2], mdb.tag[3]))
-	    continue;
-
 	  mdb.loader = load_database;
-	  nbytes = mconv_encode_buffer (Mcoding_utf_8,  MPLIST_MTEXT (p1),
-					(unsigned char *) path, PATH_MAX);
-	  if (nbytes < 0 || nbytes >= PATH_MAX)
+	  mt = MPLIST_MTEXT (p1);
+	  if (mt->format >= MTEXT_FORMAT_UTF_16LE)
+	    mtext__adjust_format (mt, MTEXT_FORMAT_UTF_8);
+	  nbytes = mtext_nbytes (mt);
+#ifdef PATH_MAX
+	  if (nbytes > PATH_MAX)
 	    continue;
-	  path[nbytes++] = '\0';
-	  mdb.extra_info = (void *) strdup (path);
-	  MLIST_APPEND1 (&mdb_list, mdbs, mdb, MERROR_DB);
+#endif	/* PATH_MAX */
+	  if (with_wildcard
+	      && mdb.tag[0] != Mchar_table
+	      && mdb.tag[0] != Mcharset)
+	    {
+	      glob_t globbuf;
+	      MPlist *load_key;
+
+	      SAFE_ALLOCA (path, len + nbytes + 1);
+	      memcpy (path, dir_info->filename, len);
+	      memcpy (path + len, mt->data, nbytes);
+	      path[len + nbytes] = '\0';
+
+	      if (glob (path, GLOB_NOSORT | GLOB_NOCHECK, NULL, &globbuf) != 0)
+		continue;
+	      load_key = mplist ();
+	      for (i = 0; i < globbuf.gl_pathc; i++)
+		{
+		  if (! (fp = fopen (globbuf.gl_pathv[i], "r")))
+		    continue;
+		  p1 = mplist__from_file (fp, load_key);
+		  fclose (fp);
+		  if (! p1)
+		    continue;
+		  if (MPLIST_PLIST_P (p1))
+		    {
+		      MPlist *p0;
+		      MDatabase mdb2;
+
+		      for (j = 0, p0 = MPLIST_PLIST (p1);
+			   j < 4 && MPLIST_SYMBOL_P (p0);
+			   j++, p0 = MPLIST_NEXT (p0))
+			mdb2.tag[j] = MPLIST_SYMBOL (p0);
+		      for (; j < 4; j++)
+			mdb2.tag[j] = Mnil;
+		      for (j = 0; j < 4; j++)
+			if (mdb.tag[j] == Masterisk
+			    ? mdb2.tag[j] == Mnil
+			    : (mdb.tag[j] != Mnil && mdb.tag[j] != mdb2.tag[j]))
+			  break;
+		      if (j == 4
+			  && ! find_database (mdb2.tag[0], mdb2.tag[1],
+					      mdb2.tag[2], mdb2.tag[3]))
+			{
+			  mdb2.loader = load_database;
+			  mdb2.extra_info = calloc (1, sizeof (MDatabaseInfo));
+			  if (! mdb2.extra_info)
+			    MEMORY_FULL (MERROR_DB);
+			  ((MDatabaseInfo*) mdb2.extra_info)->filename
+			    = strdup (globbuf.gl_pathv[i]);
+			  MLIST_APPEND1 (&mdb_list, mdbs, mdb2, MERROR_DB);
+			  if (mdb2.tag[0] == Mchar_table
+			      && mdb2.tag[2] != Mnil
+			      && (mdb2.tag[1] == Mstring
+				  || mdb2.tag[1] == Mtext
+				  || mdb2.tag[1] == Msymbol
+				  || mdb2.tag[1] == Minteger
+				  || mdb2.tag[1] == Mplist))
+			    mchar__define_prop (mdb2.tag[2], mdb2.tag[1],
+						(mdb_list.mdbs
+						 + mdb_list.used - 1));
+			}
+		    }
+		  M17N_OBJECT_UNREF (p1);
+		}
+	      M17N_OBJECT_UNREF (load_key);
+	      globfree (&globbuf);
+	    }
+	  else
+	    {
+	      if (find_database (mdb.tag[0], mdb.tag[1],
+				 mdb.tag[2], mdb.tag[3]))
+		continue;
+	      SAFE_ALLOCA (path, nbytes + 1);
+	      memcpy (path, mt->data, nbytes);
+	      path[nbytes] = '\0';
+	      mdb.extra_info = calloc (1, sizeof (MDatabaseInfo));
+	      if (! mdb.extra_info)
+		MEMORY_FULL (MERROR_DB);
+	      ((MDatabaseInfo*) mdb.extra_info)->filename = strdup (path);
+	      MLIST_APPEND1 (&mdb_list, mdbs, mdb, MERROR_DB);
+	      if (mdb.tag[0] == Mchar_table
+		  && mdb.tag[2] != Mnil
+		  && (mdb.tag[1] == Mstring
+		      || mdb.tag[1] == Mtext
+		      || mdb.tag[1] == Msymbol
+		      || mdb.tag[1] == Minteger
+		      || mdb.tag[1] == Mplist))
+		mchar__define_prop (mdb.tag[2], mdb.tag[1], 
+				    mdb_list.mdbs + mdb_list.used - 1);
+	    }
 	}
       M17N_OBJECT_UNREF (pl);
     }
 
+  SAFE_FREE (path);
+}
+
+
+/* Internal API */
+
+/** List of database directories.  */ 
+MPlist *mdatabase__dir_list;
+
+int
+mdatabase__init ()
+{
+  MDatabaseInfo *dir_info;
+
+  Mchar_table = msymbol ("char-table");
+  Masterisk = msymbol ("*");
+
+  mdatabase__dir_list = mplist ();
+  /** The macro M17NDIR specifies a directory where the system-wide
+    MDB_DIR file exists.  */
+  if ((dir_info = get_dir_info (M17NDIR)))
+    mplist_set (mdatabase__dir_list, Mt, dir_info);
+
+  /* The variable mdatabase_dir specifies a directory where an
+     application program specific MDB_DIR file exists.  */
+  if ((dir_info = get_dir_info (mdatabase_dir)))
+    mplist_push (mdatabase__dir_list, Mt, dir_info);
+
+  /* The environment variable M17NDIR (if non-NULL) specifies a
+     directory where a user specific MDB_DIR file exists.  */
+  if ((dir_info = get_dir_info (getenv ("M17NDIR"))))
+    mplist_push (mdatabase__dir_list, Mt, dir_info);
+
+  MLIST_INIT1 (&mdb_list, mdbs, 256);
+  update_database_list ();
+
   mdatabase__finder = ((void *(*) (MSymbol, MSymbol, MSymbol, MSymbol))
 		       mdatabase_find);
   mdatabase__loader = (void *(*) (void *)) mdatabase_load;
@@ -571,20 +761,88 @@ mdatabase__fini (void)
   int i;
   MPlist *plist; 
 
-  MPLIST_DO (plist, mdb_dir_list)
-    free (MPLIST_VAL (plist));
-  M17N_OBJECT_UNREF (mdb_dir_list);
+  MPLIST_DO (plist, mdatabase__dir_list)
+    {
+      MDatabaseInfo *dir_info = MPLIST_VAL (plist);
+
+      free (dir_info->filename);
+      free (dir_info);
+    }
+  M17N_OBJECT_UNREF (mdatabase__dir_list);
 
   for (i = 0; i < mdb_list.used; i++)
     {
       MDatabase *mdb = mdb_list.mdbs + i;
 
       if (mdb->loader == load_database)
-	free (mdb->extra_info);
+	{
+	  MDatabaseInfo *db_info = mdb->extra_info;
+
+	  free (db_info->filename);
+	  free (db_info);
+	}
     }
   MLIST_FREE1 (&mdb_list, mdbs);
 }
 
+MPlist *
+mdatabase__load_for_keys (MDatabase *mdb, MPlist *keys)
+{
+  int mdebug_mask = MDEBUG_DATABASE;
+  FILE *fp;
+  MPlist *plist;
+  char buf[256];
+  char *filename;
+
+  if (mdb->loader != load_database
+      || mdb->tag[0] == Mchar_table
+      || mdb->tag[0] == Mcharset)
+    MERROR (MERROR_DB, NULL);
+  MDEBUG_PRINT1 (" [DATABASE] loading <%s>.\n",
+		 gen_database_name (buf, mdb->tag));
+  filename = get_database_filename ((MDatabaseInfo *) mdb->extra_info);
+  if (! filename || ! (fp = fopen (filename, "r")))
+    MERROR (MERROR_DB, NULL);
+  plist = mplist__from_file (fp, keys);
+  fclose (fp);
+  return plist;
+}
+
+
+/* Check if the database MDB should be reloaded or not.  */
+
+int
+mdatabase__check (MDatabase *mdb)
+{
+  MDatabaseInfo *db_info = (MDatabaseInfo *) mdb->extra_info;
+  struct stat buf;
+
+  if (stat (db_info->filename, &buf) < 0)
+    return -1;
+  return (db_info->time >= buf.st_ctime
+	  && db_info->time >= buf.st_mtime);
+}
+
+/* Find a file FILENAME in mdatabase__dir_list.  If the file exist,
+   return the absolute pathname.  If FILENAME is already absolute,
+   return a copy of it.  */
+
+char *
+mdatabase__find_file (char *filename)
+{
+  MDatabaseInfo db_info;
+
+  db_info.filename = strdup (filename);
+  db_info.time = 0;
+  if (! get_database_filename (&db_info))
+    {
+      free (db_info.filename);
+      return NULL;
+    }
+  return db_info.filename;
+}
+
+
 /*** @} */
 #endif /* !FOR_DOXYGEN || DOXYGEN_INTERNAL_MODULE */
 
@@ -609,12 +867,11 @@ mdatabase__fini (void)
 /***ja
     @brief ץꥱͭΥǡѥǥ쥯ȥ.
 
-    ץꥱץबΥץͭΥǡ m17n ǡ
-    ١񤭤ǡ󶡤ˤϡޥ M17N_INIT () 
-    Ƥˤѿǡեޤǥ쥯ȥ̾˥åȤ
-    ƤϤʤʤǥ쥯ȥˤ "mdb.dir" ե򤪤Ȥ
-    롣"mdb.dir"եˤϡ @ref mdbDir "mdbDir(5)" 
-    ƤեޥåȤǥǡΥꥹȤ򵭽Ҥ롣
+    ץꥱץबΥץͭΥǡ m17n 
+    ǡ١񤭤ǡ󶡤ˤϡޥ M17N_INIT () 
+    Ƥˤѿǡեޤǥ쥯ȥ̾˥åȤʤƤϤʤʤǥ쥯ȥˤ
+    "mdb.dir" ե򤪤ȤǤ롣"mdb.dir"եˤϡ 
+    @ref mdbDir "mdbDir(5)" ƤեޥåȤǥǡΥꥹȤ򵭽Ҥ롣
 
     ǥեȤͤ NULL Ǥ롣  */
 
@@ -632,27 +889,16 @@ char *mdatabase_dir;
     @brief ǡ١Υǡõ.
 
     ؿ mdatabase_find () ϡ m17n ١ $TAG0  
-    $TAG3 ޤǤΥĥǡõؤΥݥ󥿤֤Τ
-    ʥǡʤ @c NULL ֤
+    $TAG3 ޤǤΥĥǡõؤΥݥ󥿤֤Τ褦ʥǡʤ
+    @c NULL ֤
 
     @latexonly \IPAlabel{mdatabase_find} @endlatexonly  */
 
 MDatabase *
 mdatabase_find (MSymbol tag0, MSymbol tag1, MSymbol tag2, MSymbol tag3)
 {
-  int i;
-
-  for (i = 0; i < mdb_list.used; i++)
-    {
-      MDatabase *mdb = mdb_list.mdbs + i;
-
-      if (tag0 == mdb->tag[0]
-	  && tag1 == mdb->tag[1]
-	  && tag2 == mdb->tag[2]
-	  && tag3 == mdb->tag[3])
-	return mdb;
-    }
-  return NULL;
+  update_database_list ();
+  return find_database (tag0, tag1, tag2, tag3);
 }
 
 /*=*/
@@ -668,11 +914,9 @@ mdatabase_find (MSymbol tag0, MSymbol tag1, MSymbol tag2, MSymbol tag3)
     @brief m17n ǡ١ΥǡꥹȤ֤.
 
     ؿ mdatabase_list ()  m17n ǡ١椫 $TAG0 $TAG3 
-    ޤǤΥĥǡõΥꥹȤplist Ȥ֤ $TAGn 
-     #Mnil ǤäˤϡǤդΥ˥ޥå磻ɥɤ
-    Ƽ갷롣֤ plist γǤϥ Ȥ #Mt 
-    Ȥ #MDatabase ؤΥݥ󥿤ġ  */
-
+    ޤǤΥĥǡõΥꥹȤplist Ȥ֤ $TAGn  #Mnil
+    ǤäˤϡǤդΥ˥ޥå磻ɥɤȤƼ갷롣֤
+    plist γǤϥ Ȥ #Mt ͤȤ #MDatabase ؤΥݥ󥿤ġ  */
 
 MPlist *
 mdatabase_list (MSymbol tag0, MSymbol tag1, MSymbol tag2, MSymbol tag3)
@@ -680,6 +924,8 @@ mdatabase_list (MSymbol tag0, MSymbol tag1, MSymbol tag2, MSymbol tag3)
   int i;
   MPlist *plist = NULL, *pl;
 
+  update_database_list ();
+
   for (i = 0; i < mdb_list.used; i++)
     {
       MDatabase *mdb = mdb_list.mdbs + i;
@@ -697,8 +943,6 @@ mdatabase_list (MSymbol tag0, MSymbol tag1, MSymbol tag2, MSymbol tag3)
   return plist;
 }
 
-
-
 /*=*/
 /***en
     @brief Define a data of the m17n database.
@@ -723,22 +967,20 @@ mdatabase_list (MSymbol tag0, MSymbol tag1, MSymbol tag2, MSymbol tag3)
 /***ja
     @brief m17n ǡ١Υǡ.
 
-    ؿ mdatabase_define ()  $TAG0  $TAG3 ޤǤΥղ
-     $EXTRA_INFO ĥǡ롣
+    ؿ mdatabase_define ()  $TAG0  $TAG3 ޤǤΥղþ 
+    $EXTRA_INFO ĥǡ롣
 
-    $LOADER ϤΥǡΥɤѤؿؤΥݥ󥿤Ǥ롣
-    δؿ mdatabase_load ()  $TAGS  $EXTRA_INFO Ȥ2 Ĥ
-    դǸƤӽФ롣 $TAGS  $TAG0  $TAG3 ޤǤ
-    Ǥ롣
+    $LOADER ϤΥǡΥɤѤؿؤΥݥ󥿤Ǥ롣δؿ
+    mdatabase_load ()  $TAGS  $EXTRA_INFO ȤĤΰդǸƤӽФ롣 
+    $TAGS  $TAG0  $TAG3 ޤǤǤ롣
 
-    ⤷ $LOADER  @c NULL ʤ顢m17n 饤֥ɸΥȤ
-    롣ξˤ $EXTRA_INFO ϥǡޤե̾ǤʤƤϤ
-    ʤ
+    ⤷ $LOADER  @c NULL ʤ顢m17n 饤֥ɸΥȤ롣ξˤ
+    $EXTRA_INFO ϥǡޤե̾ǤʤƤϤʤʤ
 
     @return
-      mdatabase_define () 줿ǡ١
-    ؤΥݥ󥿤֤Υݥ󥿤ϴؿ mdatabase_load () ΰ
-    ȤѤ뤳ȤǤ롣Ǥʤ @c NULL ֤
+     mdatabase_define () 
+    줿ǡ١ؤΥݥ󥿤֤Υݥ󥿤ϴؿ mdatabase_load () 
+    ΰȤѤ뤳ȤǤ롣Ǥʤ @c NULL ֤
 
     @latexonly \IPAlabel{mdatabase_define} @endlatexonly  */
 
@@ -760,13 +1002,24 @@ mdatabase_define (MSymbol tag0, MSymbol tag1, MSymbol tag2, MSymbol tag3,
 
       template.tag[0] = tag0, template.tag[1] = tag1;
       template.tag[2] = tag2, template.tag[3] = tag3;
+      template.extra_info = NULL;
       MLIST_APPEND1 (&mdb_list, mdbs, template, MERROR_DB);
       mdb = mdb_list.mdbs + (mdb_list.used - 1);
     }
   mdb->loader = loader ? loader : load_database;
-  mdb->extra_info = extra_info;
   if (mdb->loader == load_database)
-    mdb->extra_info = strdup ((char *) extra_info);
+    {
+      MDatabaseInfo *db_info = mdb->extra_info;
+
+      if (db_info)
+	free (db_info->filename);
+      else
+	db_info = mdb->extra_info = malloc (sizeof (MDatabaseInfo));
+      db_info->filename = strdup ((char *) extra_info);
+      db_info->time = 0;
+    }
+  else
+    mdb->extra_info = extra_info;
   return (&(mdb_list.mdbs[mdb_list.used - 1]));
 }
 
@@ -778,10 +1031,10 @@ mdatabase_define (MSymbol tag0, MSymbol tag1, MSymbol tag2, MSymbol tag3,
     returns the contents.  The type of contents depends on the type of
     the data.
 
-    If the data is of the @e plist type, this function returns a
+    If the data is of the @e plist @e type, this function returns a
     pointer to @e plist.
 
-    If the database is of the @e chartable type, it returns a
+    If the database is of the @e chartable @e type, it returns a
     chartable.  The default value of the chartable is set according to
     the second tag of the data as below:
 
@@ -789,7 +1042,7 @@ mdatabase_define (MSymbol tag0, MSymbol tag1, MSymbol tag2, MSymbol tag3,
     @li If the tag is #Minteger, the default value is -1.
     @li Otherwise, the default value is @c NULL.
 
-    If the data is of the @e charset type, it returns a plist of length 2
+    If the data is of the @e charset @e type, it returns a plist of length 2
     (keys are both #Mt).  The value of the first element is an array
     of integers that maps code points to the corresponding character
     codes.  The value of the second element is a chartable of integers
@@ -800,22 +1053,22 @@ mdatabase_define (MSymbol tag0, MSymbol tag1, MSymbol tag2, MSymbol tag3,
 /***ja
     @brief ǡ١ǡɤ.
 
-    ؿ mdatabase_load ()  $MDB ؤǡɤ
-    Ȥ֤֤ΤϥǡΥפˤäưۤʤ롣
+    ؿ mdatabase_load ()  $MDB 
+    ؤǡɤȤ֤֤ΤϥǡΥפˤäưۤʤ롣
 
-    ǡ @e plist פʤС @e plist ؤΥݥ󥿤֤
+    ǡ @e plist ʤС @e plist ؤΥݥ󥿤֤
 
-    ǡ @e chartable פʤʸơ֥֤ʸơ֥
-    Υǥեͤϡǡ2ˤäưʲΤ褦˷ޤ롣
+    ǡ @e chartable ʤʸơ֥֤
+    ʸơ֥Υǥեͤϡǡ2ˤäưʲΤ褦˷ޤ롣
 
     @li  #Msymbol ʤ顢ǥեͤ #Mnil
     @li  #Minteger ʤ顢ǥեͤ -1
     @li ʳʤ顢ǥեͤ @c NULL
 
-    ǡ @e charset פʤĹ 2  plist ֤ʥ϶ 
-    #Mt ˡǽǤͤϥɥݥȤбʸɤ˥ޥå
-    Ǥ롣ܤǤͤϵդΥޥåפ򤹤ʸơ
-    Ǥ롣ʸåȤͽƤʤФʤʤ
+    ǡ @e charset ʤĹ 2  plist ֤ʥ϶#Mt ˡ
+    ǽǤͤϥɥݥȤбʸɤ˥ޥåפǤ롣
+    ܤǤͤϵդΥޥåפ򤹤ʸơ֥Ǥ롣
+    ʸåȤͽƤʤФʤʤ
 
     @latexonly \IPAlabel{mdatabase_load} @endlatexonly
   */
@@ -846,8 +1099,8 @@ mdatabase_load (MDatabase *mdb)
 /***ja
     @brief ǡΥ.
 
-    ؿ mdatabase_tag () ϡǡ $MDB Υʥܥˤ
-    Ĺ 4 Ǥ롣
+    ؿ mdatabase_tag () ϡǡ $MDB Υʥܥˤ֤Ĺ
+    4 Ǥ롣
 
     @latexonly \IPAlabel{mdatabase_tag} @endlatexonly  */
 
