@@ -31,7 +31,7 @@
     appearance of M-texts, i.e. font size, color, underline, etc.
 
     The drawing format of M-texts can be controlled in a variety of
-    ways, which provides powerful 2-dimensional layouting
+    ways, which provides powerful 2-dimensional layout
     facility.  */
 
 /***ja
@@ -40,12 +40,11 @@
 
     m17n-gui API ˤϡM-text ɽ뤿δؿѰդƤ롣
 
-    ɽѤեȤϡեȥåȤ face ץѥƥ˴
-    ƼưŪ˷ꤵ롣ޤեȤΥ俧䲼ʤɤθ
-     face ˤäƷޤ롣
+    ɽѤեȤϡեȥåȤ face 
+    Υץѥƥ˴ŤƼưŪ˷ꤵ롣ޤեȤΥ俧䲼ʤɤθɤ
+    face ˤäƷޤ롣
 
-    M-text եޥåȤ¿ͤˡǤΤǡϤ
-    쥤ȵǽ¸Ǥ롣
+    M-text եޥåȤ¿ͤˡǤΤǡϤ󼡸쥤ȵǽ¸Ǥ롣
     */
 
 /*=*/
@@ -65,6 +64,7 @@
 #include "m17n-misc.h"
 #include "internal.h"
 #include "symbol.h"
+#include "mtext.h"
 #include "textprop.h"
 #include "internal-gui.h"
 #include "face.h"
@@ -80,10 +80,14 @@
 static MSymbol M_glyph_string;
 
 /* Special scripts */
-static MSymbol Minherited;
+static MSymbol Minherited, Mcommon;
 /* Special categories */
 static MSymbol McatCc, McatCf;
 
+static MCharTable *linebreak_table;
+static MSymbol M_break_at_space, M_break_at_word, M_break_at_any;
+static MSymbol M_kinsoku_bol, M_kinsoku_eol;
+
 
 /* Glyph-string composer.  */
 
@@ -179,7 +183,8 @@ visual_order (MGlyphString *gstring)
 	  /* Mirrored.  */
 	  g->c = visual[j];
 	  if (g->rface->rfont)
-	    g->code = mfont__encode_char (g->rface->rfont, g->c);
+	    g->code = mfont__encode_char (NULL, (MFont *) g->rface->rfont,
+					  NULL, g->c);
 	}
 #endif /* HAVE_FRIBIDI */
       g->bidi_level = levels[i];
@@ -232,7 +237,7 @@ compose_glyph_string (MFrame *frame, MText *mt, int from, int to,
 		      MGlyphString *gstring)
 {
   MRealizedFace *default_rface = frame->rface;
-  int stop, face_change, language_change, charset_change;
+  int stop, face_change, language_change, charset_change, font_change;
   MGlyph g_tmp, *g, *last_g;
   int pos;
   MSymbol language = Mnil, script = Mnil, charset = Mnil;
@@ -253,7 +258,7 @@ compose_glyph_string (MFrame *frame, MText *mt, int from, int to,
   g_tmp.type = GLYPH_ANCHOR;
   g_tmp.pos = g_tmp.to = from;
   APPEND_GLYPH (gstring, g_tmp);
-  stop = face_change = pos = from;
+  stop = face_change = font_change = pos = from;
   while (1)
     {
       int c;
@@ -263,19 +268,39 @@ compose_glyph_string (MFrame *frame, MText *mt, int from, int to,
 	{
 	  if (pos == to)
 	    break;
-	  if (pos == face_change)
+	  if (pos < mtext_nchars (mt))
 	    {
+	      MFont *font = rface->font;
 	      MFace *faces[64];
-	      int num = mtext_get_prop_values (mt, pos, Mface,
-					       (void **) faces, 64);
+	      int num;
 
-	      mtext_prop_range (mt, Mface, pos, NULL, &face_change, 1);
-	      if (face_change == mtext_nchars (mt))
-		face_change++;
-	      rface = (num > 0 ? mface__realize (frame, faces, num, size)
-		       : default_rface);
+	      if (pos == font_change)
+		{
+		  font = mtext_get_prop (mt, pos, Mfont);
+		  mtext_prop_range (mt, Mfont, pos, NULL, &font_change, 0);
+		  if (font_change == mtext_nchars (mt))
+		    font_change++;
+		}
+	      if (pos == face_change)
+		{
+		  num = mtext_get_prop_values (mt, pos, Mface,
+					       (void **) faces, 64);
+		  mtext_prop_range (mt, Mface, pos, NULL, &face_change, 1);
+		  if (face_change == mtext_nchars (mt))
+		    face_change++;
+		}
+	      else
+		{
+		  faces[0] = &rface->face;
+		  num = 1;
+		}
+	      rface = mface__realize (frame, faces, num, size, font);
 	    }
+	  else
+	    rface = default_rface;
 	  stop = to;
+	  if (stop > font_change)
+	    stop = font_change;		
 	  if (stop > face_change)
 	    stop = face_change;		
 	}
@@ -335,11 +360,11 @@ compose_glyph_string (MFrame *frame, MText *mt, int from, int to,
       else
 	{
 	  this_script = (MSymbol) mchar_get_prop (c, Mscript);
-	  if (this_script == Minherited || this_script == Mnil)
+	  if (this_script == Minherited || this_script == Mcommon)
 	    this_script = script;
-	  if (this_script == Mnil)
+	  if (this_script == Mcommon)
 	    this_script = non_latin_script;
-	  if (this_script == Mnil)
+	  if (this_script == Mcommon)
 	    {
 	      /* Search forward for a character that explicitly
 		 specifies a non-latin script.  */
@@ -348,7 +373,7 @@ compose_glyph_string (MFrame *frame, MText *mt, int from, int to,
 
 	      for (g1 = g + 1; g1->type != GLYPH_ANCHOR; g1++)
 		if (g1->c >= 0x100
-		    && (sym = mchar_get_prop (g1->c, Mscript)) != Mnil
+		    && (sym = mchar_get_prop (g1->c, Mscript)) != Mcommon
 		    && sym != Minherited)
 		  {
 		    this_script = sym;
@@ -402,7 +427,7 @@ compose_glyph_string (MFrame *frame, MText *mt, int from, int to,
 	{
 	  int start = i++;
 
-	  if (this->rface->rfont->layouter != Mnil)
+	  if (this->rface->layouter != Mnil)
 	    {
 	      MGlyph *prev;
 	      unsigned code;
@@ -410,16 +435,25 @@ compose_glyph_string (MFrame *frame, MText *mt, int from, int to,
 	      for (prev = MGLYPH (start - 1);
 		   (prev->type == GLYPH_CHAR
 		    && prev->category == GLYPH_CATEGORY_FORMATTER
-		    && (code = mfont__encode_char (this->rface->rfont, prev->c)
+		    && (code = mfont__encode_char (NULL,
+						   (MFont *) this->rface->rfont,
+						   NULL, prev->c)
 			!= MCHAR_INVALID_CODE));
 		   start--, prev--)
-		prev->code = code;
+		if (prev->rface->rfont != this->rface->rfont)
+		  {
+		    prev->rface->rfont = this->rface->rfont;
+		    prev->code = code;
+		  }
 
 	      for (g++;
 		   (g->type == GLYPH_CHAR
+		    && g->rface->layouter == this->rface->layouter
 		    && (g->rface->rfont == this->rface->rfont
 			|| (g->category == GLYPH_CATEGORY_FORMATTER
-			    && ((code = mfont__encode_char (this->rface->rfont,
+			    && ((code = mfont__encode_char (NULL,
+							    (MFont *) this->rface->rfont,
+							    NULL,
 							    g->c))
 				!= MCHAR_INVALID_CODE))));
 		   i++, g++)
@@ -547,7 +581,7 @@ layout_glyphs (MFrame *frame, MGlyphString *gstring, int from, int to,
     {
       MGlyph *base = g++;
       MRealizedFont *rfont = base->rface->rfont;
-      int size = rfont->font.property[MFONT_SIZE];
+      int size = rfont->spec.size;
       int width, lbearing, rbearing;
 
       if (g == last_g || ! g->combining_code)
@@ -571,7 +605,10 @@ layout_glyphs (MFrame *frame, MGlyphString *gstring, int from, int to,
 	  if (base->left_padding && base->lbearing < 0)
 	    {
 	      base->xoff = - base->lbearing;
-	      base->width += base->xoff;
+	      if (base->rbearing < 0)
+		base->width = base->rbearing - base->lbearing;
+	      else
+		base->width += base->xoff;
 	      base->rbearing += base->xoff;
 	      base->lbearing = 0;
 	    }
@@ -579,9 +616,8 @@ layout_glyphs (MFrame *frame, MGlyphString *gstring, int from, int to,
 	    {
 	      base->width = base->rbearing;
 	    }
-	  lbearing = (base->xoff + base->lbearing < 0
-		      ? base->xoff + base->lbearing : 0);
-	  rbearing = base->xoff + base->rbearing;
+	  lbearing = base->lbearing;
+	  rbearing = base->rbearing;
 	}
       else
 	{
@@ -618,7 +654,7 @@ layout_glyphs (MFrame *frame, MGlyphString *gstring, int from, int to,
 						   (combining_code));
 
 		  rfont = g->rface->rfont;
-		  size = rfont->font.property[MFONT_SIZE];
+		  size = rfont->spec.size;
 		  off_x = (size * (COMBINING_CODE_OFF_X (combining_code) - 128)
 			   / 1000);
 		  off_y = (size * (COMBINING_CODE_OFF_Y (combining_code) - 128)
@@ -1231,7 +1267,8 @@ render_glyphs (MFrame *frame, MDrawWindow win, int x, int y, int width,
 		gend--;
 	    }
 	  if (g != gend)
-	    while (gend[-1].to == gend->to) gend++;
+	    while (gend->type != GLYPH_ANCHOR && gend[-1].to == gend->to)
+	      gend++;
 	}
     }
 
@@ -1494,6 +1531,9 @@ alloc_gstring (MFrame *frame, MText *mt, int pos, MDrawControl *control,
   return gstring;
 }
 
+static MGlyph *find_glyph_in_gstring (MGlyphString *gstring, int pos,
+				      int forwardp);
+
 /* Truncate the line width of GSTRING to GSTRING->width_limit.  */
 
 static void
@@ -1527,16 +1567,26 @@ truncate_gstring (MFrame *frame, MText *mt, MGlyphString *gstring)
   if (gstring->control.line_break)
     {
       pos = (*gstring->control.line_break) (mt, gstring->from + i,
-					    gstring->from, gstring->to, 0, 0);
+					    gstring->from, gstring->from + i,
+					    0, 0);
       if (pos <= gstring->from)
-	pos = gstring->from + 1;
+	{
+	  g = find_glyph_in_gstring (gstring, gstring->from, 1);
+	  pos = g->to;
+	}
       else if (pos >= gstring->to)
 	pos = gstring->to;
     }
   else if (i == 0)
-    pos++;
-  compose_glyph_string (frame, mt, gstring->from, pos, gstring);
-  layout_glyph_string (frame, gstring);
+    {
+      g = find_glyph_in_gstring (gstring, gstring->from, 1);
+      pos = g->to;
+    }
+  if (pos < gstring->to)
+    {
+      compose_glyph_string (frame, mt, gstring->from, pos, gstring);
+      layout_glyph_string (frame, gstring);
+    }
 }
 
 
@@ -1808,6 +1858,7 @@ mdraw__init ()
   MLIST_INIT1 (&scratch_gstring, glyphs, 3);
 
   Minherited = msymbol ("inherited");
+  Mcommon = msymbol ("common");
 
   McatCc = msymbol ("Cc");
   McatCf = msymbol ("Cf");
@@ -1822,6 +1873,12 @@ mdraw__init ()
   fribidi_set_mirroring (TRUE);
 #endif
 
+  M_break_at_space = msymbol ("bs");
+  M_break_at_word = msymbol ("bw");
+  M_break_at_any = msymbol ("ba");
+  M_kinsoku_bol = msymbol ("kb");
+  M_kinsoku_eol = msymbol ("ke");
+
   return 0;
 }
 
@@ -1829,6 +1886,8 @@ void
 mdraw__fini ()
 {
   MLIST_FREE1 (&scratch_gstring, glyphs);
+  M17N_OBJECT_UNREF (linebreak_table);
+  linebreak_table = NULL;
 }
 
 /*** @} */
@@ -1884,7 +1943,7 @@ mdraw__fini ()
 	 of the script and language.  If no entry is found, proceed to
 	 the next step.  
 
-        If an entry is found, use one of the fonts in the entry that
+         If an entry is found, use one of the fonts in the entry that
 	 has a glyph for the character and that matches best with the
 	 face properties.  If no such font exists, proceed to the next
 	 step.
@@ -1914,67 +1973,57 @@ mdraw__fini ()
     @brief ɥ M-text 褹.
 
     ؿ mdraw_text () ϡե졼 $FRAME Υɥ $WIN κɸ 
-    ($X, $Y) ˡM-text $MT  $FROM  $TO ޤǤΥƥȤ
-    褹롣
+    ($X, $Y) ˡM-text $MT  $FROM  $TO ޤǤΥƥȤ褹롣
 
     ƥȤθɤʥեȡ롢ʤɡˤϡ @c Mface 
-    ǤƥȥץѥƥͤˤäƷޤ롣M-text ΰ뤤
-    ˤΤ褦ʥƥȥץѥƥդƤʤˤϡ$FRAME 
+    ǤƥȥץѥƥͤˤäƷޤ롣M-text 
+    ΰ뤤ˤΤ褦ʥƥȥץѥƥդƤʤˤϡ$FRAME 
     ΥǥեȥեѤ롣
 
-    M-text γʸɽեȤϡե fontset ץѥƥ
-    ͤʲΥ르ꥺФ롣
+    M-text γʸɽեȤϡե fontset 
+    ץѥƥͤΤ顢ʲΥ르ꥺФ롣
 
     <ol>
 
-    <li> ʸΥƥȥץѥƥΤ @c Mcharset Ǥ
-         ΤͤĴ٤롣ͤʸåȤɽ魯ܥ뤫 #Mnil 
-         Τɤ餫Ǥ롣#Mnil ʤСΥƥåפ˿ʤࡣǤ
-         СΥեȥåȤΥޥåԥ󥰥ơ֥뤫餽ʸ
-         ѤΤΤõեȤߤĤʤСΥƥåפ
-         ʤࡣ
+    <li> ʸΥƥȥץѥƥΤ @c Mcharset
+         ǤΤͤĴ٤롣ͤʸåȤɽ魯ܥ뤫 #Mnil 
+         Τɤ餫Ǥ롣#Mnil ʤСΥƥåפ˿ʤࡣ
+	 ǤʤСfontset 
+	 Υޥåԥ󥰥ơ֥ˤʸåѤΥեȤ뤫ɤĴ٤롣
+	 ̵СΥƥåפ˿ʤࡣ
          
-         ʸåѤΥեȤߤĤСΤߤʸ
-	 ѤΥդեγƥץѥƥ˺Ǥ褯פ
-	 ΤȤΤ褦ʥեȤ̵мΥƥåפ˿
-	 ࡣ
+         ʸåѤΥեȤߤĤСΤߤʸѤΥդեγƥץѥƥ˺Ǥ褯פΤȤ
+	 Τ褦ʥեȤ̵мΥƥåפ˿ʤࡣ
 
     <li> ʸʸץѥƥ "script" ʥץȡˤĴ٤롣
-         ƤʤФʸʸץѥƥ "script" 
-         Ĵ٤롣ʸʤäꡢʸץѥƥäƤ
-         äˤϡΥƥåפ˿ʤࡣ
+         ΥץѥƥѾƤʤФʸʸץѥƥ "script" 
+	 Ĵ٤롣ʸʤäꡢʸץѥƥäƤʤäˤϡΥƥåפ˿ʤࡣ
 
-         ʸΥƥȥץѥƥΤ @c Mlanguage Ǥ
-         ΤͤĴ٤롣ͤϸɽ魯ܥ뤫 @c Mnil 
-         줫Ǥ롣
+         ʸΥƥȥץѥƥΤ @c Mlanguage ǤΤͤĴ٤롣
+	 ͤϸɽ魯ܥ뤫 @c Mnil Τ줫Ǥ롣
 
-	 ǤʤСθȥץȤȤ߹碌ѤΥե
-	 åȤޥåԥ󥰥ơ֥뤫õĤʤмΥƥ
-	 פ˿ʤࡣ
+	 θȥץȤȤ߹碌 fontset
+	 Υޥåԥ󥰥ơ֥ˤ뤫ɤĴ٤롣ĤʤмΥƥåפ˿ʤࡣ
 
-	 Τ褦ʸåѤΥեȤߤĤСΤ
-	 ߤʸѤΥդեγƥץѥƥ˺Ǥ褯
-	 פƤΤȤΤ褦ʥեȤ̵мΥƥ
-	 פ˿ʤࡣ
+	 ĤäФˤϡΥեȤΤߤʸѤΥդեγƥץѥƥ˺Ǥ褯פƤΤȤ
+	 Τ褦ʥեȤ̵мΥƥåפ˿ʤࡣ
 
-    <li> ʸΥդĥեȤ򡢥եȥåȤfall-backơ
-         ֥뤫õեȤĤФȤ
+    <li> ʸΥդĥեȤ򡢥եȥåȤ fall-back 
+         ơ֥뤫õեȤĤФȤ
 
     </ol>
 
-    ʾΥ르ꥺǥեȤĤʤСδؿϤʸ
-    Ȥƶλͳѷɽ롣
+    ʾΥ르ꥺǥեȤĤʤСδؿϤʸȤƶλͳѷɽ롣
 
-    δؿ褹ΤϥդʤǤ롣طʿꤹˤϡ
-    ؿ mdraw_image_text () ؿ mdraw_text_with_control () Ȥ
-    ȡ
+    δؿ褹ΤϥդʤǤ롣طʿꤹˤϡؿ
+    mdraw_image_text () ؿ mdraw_text_with_control () Ȥȡ
 
     δؿϡX ɥˤؿ <tt>XDrawString ()</tt>,
     <tt>XmbDrawString ()</tt>, <tt>XwcDrawString ()</tt> 롣
 
     @return
     硢mdraw_text ()  0 ֤顼Ф줿
-     -1 ֤ѿ #merror_code ˥顼ɤꤹ롣
+    -1 ֤ѿ #merror_code ˥顼ɤꤹ롣
 
     @latexonly \IPAlabel{mdraw_text} @endlatexonly  */
 
@@ -2024,20 +2073,19 @@ mdraw_text (MFrame *frame, MDrawWindow win, int x, int y,
     @brief ǥץ쥤M-text Ȥ.
   
     ؿ mdraw_image_text () ϡե졼 $FRAME Υɥ $WIN 
-    ɸ ($X, $Y) ˡM-text $MT  $FROM  $TO ޤǤΥƥȤ
-    Ȥ
+    κɸ ($X, $Y) ˡM-text $MT  $FROM  $TO 
+    ޤǤΥƥȤȤ
 
-    ƥȤˡ mdraw_text () ȤۤƱǤ뤬δؿ
-    ϥեǻꤵ줿طʤۤʤäƤ롣
+    ƥȤˡ mdraw_text ()
+    ȤۤƱǤ뤬δؿǤϥեǻꤵ줿طʤۤʤäƤ롣
 
     δؿϡX ɥˤ <tt>XDrawImageString ()</tt>,
     <tt>XmbDrawImageString ()</tt>, <tt>XwcDrawImageString ()</tt> 
     롣
 
     @return
-    硢mdraw_image_text ()  0 ֤顼
-    줿 -1 ֤ѿ #m_errro ˥顼ɤꤹ
-    롣
+    硢mdraw_image_text ()  0 
+    ֤顼Ф줿 -1 ֤ѿ #m_errro ˥顼ɤꤹ롣
 
     @latexonly \IPAlabel{mdraw_image_text} @endlatexonly   */
 
@@ -2080,19 +2128,18 @@ mdraw_image_text (MFrame *frame, MDrawWindow win, int x, int y,
     more detail.  */
 
 /***ja
-    @brief ǥץ쥤M-text ʾܺ٤Ĥ.
+    @brief ǥץ쥤M-text ܺ٤Ĥ.
 
     ؿ mdraw_text_with_control () ϡե졼 $FRAME Υɥ 
     $WIN κɸ ($X, $Y) ˡM-text $MT  $FROM  $TO ޤǤΥƥ
     Ȥ
 
-    ƥȤˡ mdraw_text () ȤۤƱǤ뤬δؿ
-    ѤΥ֥ $CONTROL Ǥλؼˤ⽾ۤʤäƤ롣
+    ƥȤˡ mdraw_text () ȤۤƱǤ뤬δؿѤΥ֥
+    $CONTROL λؼˤ⽾ۤʤäƤ롣
 
-    Ȥ $CONTROL  <two_dimensional> ǤʤСδؿ
-    M-text 2Ūʤ M-text βԤǹԤᡢ³
-    ʸϼιԤܺ٤Ϲ¤ @ MDrawControl 򻲾Ȥ뤳
-    ȡ*/
+    Ȥ $CONTROL  <two_dimensional> ǤʤСδؿ 
+    M-text 2Ūʤ M-text βԤǹԤᡢ³ʸϼιԤܺ٤Ϲ¤
+    @ MDrawControl 򻲾Ȥ뤳ȡ*/
 
 int
 mdraw_text_with_control (MFrame *frame, MDrawWindow win, int x, int y,
@@ -2119,13 +2166,13 @@ mdraw_text_with_control (MFrame *frame, MDrawWindow win, int x, int y,
     surrounding box, the box is included in the bounding box.
 
     If $OVERALL_LOGICAL_RETURN is not @c NULL, this function also
-    computes the bounding box that provides mininum spacing to other
+    computes the bounding box that provides minimum spacing to other
     graphical features (such as surrounding box) for the M-text, and
     stores the results in the members of the structure pointed to by
     $OVERALL_LOGICAL_RETURN.
 
     If $OVERALL_LINE_RETURN is not @c NULL, this function also
-    computes the bounding box that provides mininum spacing to the
+    computes the bounding box that provides minimum spacing to the
     other M-text drawn, and stores the results in the members of the
     structure pointed to by $OVERALL_LINE_RETURN.  This is a union of
     $OVERALL_INK_RETURN and $OVERALL_LOGICAL_RETURN if the members
@@ -2147,27 +2194,27 @@ mdraw_text_with_control (MFrame *frame, MDrawWindow win, int x, int y,
     楪֥ $CONTROL Ѥ M-text $MT  $FROM  $TO 
     ޤǤե졼 $FRAME ɽݤɬפȤʤ֤
 
-    $OVERALL_INK_RETURN  @c NULL ǤʤСδؿ M-text ʸ
-    Υ󥯤ΥХǥ󥰥ܥå׻$OVERALL_INK_RETURN 
-    ؤ¤ΤΥФˤη̤ꤹ롣M-text ˰Ϥ(surrounding box)
+    $OVERALL_INK_RETURN  @c NULL ǤʤСδؿ M-text 
+    ʸΥ󥯤ΥХǥ󥰥ܥå׻$OVERALL_INK_RETURN 
+    ؤ¤ΤΥФˤη̤ꤹ롣M-text ˰Ϥ (surrounding box)
     ꤹեСХǥ󥰥ܥå˴ޤࡣ
 
     $OVERALL_LOGICAL_RETURN  @c NULL ǤʤСδؿ M-text 
-    ¾ graphical feature ʰϤȤʤɡˤȤδ֤κǾΥڡ
-    Хǥ󥰥ܥå׻$OVERALL_LOGICAL_RETURN ؤ
-    ¤ΤΥФˤη̤ꤹ롣
+    ¾ graphical feature ʰϤȤʤɡ
+    Ȥδ֤κǾΥڡ򼨤Хǥ󥰥ܥå׻$OVERALL_LOGICAL_RETURN
+    ؤ¤ΤΥФˤη̤ꤹ롣
 
     $OVERALL_LINE_RETURN  @c NULL ǤʤСδؿ¾ M-text 
     Ȥδ֤κǾΥڡ򼨤Хǥ󥰥ܥå׻
-    $OVERALL_LINE_RETURN ؤ¤ΤΥФˤη̤ꤹ롣
-    ֥ $CONTROL Υ min_line_ascent, min_line_descent,
+    $OVERALL_LINE_RETURN ؤ¤ΤΥФˤη̤ꤹ롣֥
+    $CONTROL Υ min_line_ascent, min_line_descent,
     max_line_ascent, max_line_descent ٤0λˤϡͤ 
     $OVERALL_INK_RETURN $OVERALL_LOGICAL_RETURN ¤Ȥʤ롣
 
-    @return δؿɽɬפʥƥȤԥñ̤֤
-    $CONTROL->two_dimensional 0ǤʤƥȤʣιԤϤä
-    ˤϡ֤顼 -1 ֤
-    ѿ #merror_code ˥顼ɤꤹ롣
+    @return 
+    δؿɽɬפʥƥȤԥñ̤֤$CONTROL->two_dimensional
+    0ǤʤƥȤʣιԤϤäˤϡ֤顼
+    -1 ֤ѿ #merror_code ˥顼ɤꤹ롣
 
     @latexonly \IPAlabel{mdraw_text_extents} @endlatexonly  */
 
@@ -2283,15 +2330,14 @@ mdraw_text_extents (MFrame *frame,
 /***ja
     @brief  M-text γʸɽϰϤ׻.
 
-    ؿ mdraw_text_per_char_extents () ϡؿ 
-    mdraw_text_with_control () 楪֥ $CONTROL Ѥ
-     M-text $MT  $FROM  $TO ޤǤե졼 $FRAME ɽ
-    γʸΥ׻롣
+    ؿ mdraw_text_per_char_extents () ϡؿ mdraw_text_with_control ()
+    楪֥ $CONTROL Ѥ M-text $MT  $FROM  $TO 
+    ޤǤե졼 $FRAME ɽݤγʸΥ׻롣
 
     $ARRAY_SIZE ˤä $INK_ARRAY_RETURN $LOGICAL_ARRAY_RETURN 
-    ꤹ롣$INK_ARRAY_RETURN $LOGICAL_ARRAY_RETURN γ
-    Ǥϡ줾ʸ襤󥯤M-textɽ
-    а͡ˤˤäƽ롣ꤵ줿 $INK_ARRAY_RETURN  
+    Υꤹ롣$INK_ARRAY_RETURN $LOGICAL_ARRAY_RETURN 
+    γǤϡ줾ʸ襤󥯤M-text 
+    ɽа͡ˤˤäƽ롣ꤵ줿 $INK_ARRAY_RETURN  
     $LOGICAL_ARRAY_RETURN Ǥοϡ$NUM_CHARS_RETURN ᤵ롣
    
     $ARRAY_SIZE ٤Ƥˡ᤻ʤۤɾˤϡؿ -1 
@@ -2300,11 +2346,9 @@ mdraw_text_extents (MFrame *frame,
 
     ݥ $OVERALL_INK_RETURN  $OVERALL_LOGICAL_RETURN @c NULL 
     ǤʤСδؿϥƥΤΥ׻̤
-    $OVERALL_INK_RETURN  $OVERALL_LOGICAL_RETURN ǻؤ빽¤Υ
-    Ф¸롣
+    $OVERALL_INK_RETURN  $OVERALL_LOGICAL_RETURN ǻؤ빽¤ΥФ¸롣
 
-    $CONTROL->two_dimensional 0ǤʤСδؿϺǽιԤʸ
-    Υ׻롣 */
+    $CONTROL->two_dimensional 0ǤʤСδؿϺǽιԤʸΥ׻롣 */
 
 int
 mdraw_text_per_char_extents (MFrame *frame,
@@ -2438,12 +2482,12 @@ mdraw_text_per_char_extents (MFrame *frame,
     mdraw_coordinates_position () returns the character position of
     that character.\n\n
     If $Y is less than the minimum Y-coordinate of the drawn area, it
-    returns $FROM.\n\n\n
+    returns $FROM.\n\n
     If $Y is greater than the maximum Y-coordinate of the drawn area,
-    it returns $TO.\n\n\n
+    it returns $TO.\n\n
     If $Y fits in with the drawn area but $X is less than the minimum
     X-coordinate, it returns the character position of the first
-    character drawn on the line $Y.\n\n\n
+    character drawn on the line $Y.\n\n
     If $Y fits in with the drawn area but $X is greater than the
     maximum X-coordinate, it returns the character position of the
     last character drawn on the line $Y.  */
@@ -2452,25 +2496,21 @@ mdraw_text_per_char_extents (MFrame *frame,
     @brief ꤷɸ˺Ǥᤤʸʸ֤.
 
     ؿ mdraw_coordinates_position () ϡؿ 
-    mdraw_text_with_control () 楪֥ $CONTROL Ѥ
-    M-text $MT  $FROM  $TO ޤǤɸ (0, 0) Ȥ
-    ˡɸ ($X, $Y) 褵ʸʸ֤֤
-    ʸ֤Ȥϡ M-text ˤƤʸǽ餫鲿ܤ
-    Ǥ롣ǽʸʸ֤0Ȥ롣
+    mdraw_text_with_control () 楪֥ $CONTROL Ѥơ
+    M-text $MT  $FROM  $TO ޤǤɸ (0, 0) 
+    Ȥ褹ݤˡɸ ($X, $Y) 
+    褵ʸʸ֤֤ʸ֤Ȥϡ
+    M-text ˤƤʸǽ餫鲿ܤ򼨤Ǥ롣ǽʸʸ֤0Ȥ롣
 
     $FRAME ϥǥեȤΥեξ뤿Ѥ롣
 
     @return
     ɸ ($X, $Y) ʸΥդʤ硢 ؿ 
-    mdraw_coordinates_position () Ϥʸʸ֤֤
-
-    ⤷ $Y ΰκǾYɸ⾮ʤ $FROM ֤
-
-    ⤷ $Y ΰκYɸ礭ʤ $TO ֤
-
+    mdraw_coordinates_position () Ϥʸʸ֤֤\n\n
+    ⤷ $Y ΰκǾYɸ⾮ʤ $FROM ֤\n\n
+    ⤷ $Y ΰκYɸ礭ʤ $TO ֤\n\n
     ⤷ $Y ΰ˾äƤƤ $X ΰκǾXɸ
-    ϡľ y = $Y 褵ǽʸʸ֤֤
-
+    ϡľ y = $Y 褵ǽʸʸ֤֤\n\n
     ⤷ $Y ΰ˾äƤƤ $X ΰκXɸ
     礭ϡľ y = $Y 褵Ǹʸʸ֤֤ */
 
@@ -2552,9 +2592,9 @@ mdraw_coordinates_position (MFrame *frame, MText *mt, int from, int to,
     @brief դ˴ؤ׻.
 
     ؿ mdraw_glyph_info () ϡؿ mdraw_text_with_control () 
-    楪֥ $CONTROL ѤM-text $MT  $FROM  $TO 
-    ޤǤե졼 $FRAME 褷硢M-text ʸ $POS ʸ
-    ʤդ˴ؤ׻롣
+     楪֥ $CONTROL ѤM-text $MT  $FROM  $TO 
+    ޤǤե졼 $FRAME 褷硢M-text ʸ $POS 
+    ʸʤդ˴ؤ׻롣
 
     $INFO ΥФݻ롣  */
 
@@ -2611,7 +2651,7 @@ mdraw_glyph_info (MFrame *frame, MText *mt, int from, int pos,
   info->metrics.height = gstring->height;
   info->metrics.width = - g->lbearing + g->width;
   if (g->rface->rfont)
-    info->font = &g->rface->rfont->font;
+    info->font = (MFont *) g->rface->rfont;
   else
     info->font = NULL;
   /* info->logical_width is calculated later.  */
@@ -2623,7 +2663,8 @@ mdraw_glyph_info (MFrame *frame, MText *mt, int from, int pos,
 
       info->prev_from = g_tmp->pos;
     }
-  else if (info->line_from > 0)
+  else if (info->line_from > 0
+	   && gstring->from > 0)
     {
       /* The logically previous glyph is on the previous line.  */
       MGlyphString *gst = get_gstring (frame, mt, gstring->from - 1,
@@ -2752,13 +2793,17 @@ mdraw_glyph_info (MFrame *frame, MText *mt, int from, int pos,
     @brief ˴ؤ׻.
 
     ؿ mdraw_glyph_list () ϡؿ mdraw_text_with_control () 
-    楪֥ $CONTROL ѤM-text $MT  $FROM  $TO ޤ
-    ե졼 $FRAME 褷Ρƥդξ $GLYPHS ؤ
-    ˳Ǽ롣 $ARRAY_SIZE ϤΥǤ롣
+    楪֥ $CONTROL ѤM-text $MT  $FROM  $TO
+    ޤǤե졼 $FRAME 褷Ρƥդξ $GLYPHS 
+    ؤ˳Ǽ롣 $ARRAY_SIZE ϤΥǤ롣
 
-    ⤷ $ARRAY_SIZE ٤ƤΥդˤĤƤξǼΤ˽ʬ
-    С $NUM_GLYPHS_RETURN ؤ˼ºݤ᤿Ǥο
-     0 ֤  */
+    ⤷ $ARRAY_SIZE ٤ƤΥդˤĤƤξǼΤ˽ʬǤС
+    $NUM_GLYPHS_RETURN ؤ˼ºݤ᤿Ǥοꤷ 0 ֤
+
+    
+    ǤʤС$NUM_GLYPHS_RETURN ؤɬפΥꤷ
+    -1 ֤
+    */
 
 /***
     @seealso
@@ -2812,8 +2857,11 @@ mdraw_glyph_list (MFrame *frame, MText *mt, int from, int to,
 	  glyphs->y_advance = 0;
 	  if (g->rface->rfont)
 	    {
-	      glyphs->font = &g->rface->rfont->font;
-	      glyphs->font_type = g->rface->rfont->type;
+	      glyphs->font = (MFont *) g->rface->rfont;
+	      glyphs->font_type
+		= (glyphs->font->source == MFONT_SOURCE_X ? Mx
+		   : g->rface->rfont->driver == &mfont__ft_driver ? Mfreetype
+		   : Mxft);
 	      glyphs->fontp = g->rface->rfont->fontp;
 	    }
 	  else
@@ -2839,17 +2887,16 @@ mdraw_glyph_list (MFrame *frame, MText *mt, int from, int to,
     @brief Draw one or more textitems.
 
     The mdraw_text_items () function draws one or more M-texts on
-    window $WIN of $FRAME at coordinate ($X, $Y).  $ITEMS is an array
+    window $WIN of frame $FRAME at coordinate ($X, $Y).  $ITEMS is an array
     of the textitems to be drawn and $NITEMS is the number of
-    textimtems in the array.  */
+    textitems in the array.  */
 
 /***ja
     @brief textitem ɽ.
 
     ؿ mdraw_text_items () ϡİʾΥƥȥƥ򡢥ե졼
-     $FRAME Υɥ $WIN κɸ ($X, $Y) ɽ롣$ITEMS 
-    ɽ٤ƥȥƥؤΥݥ󥿤Ǥꡢ$NITEMS Ϥ
-    ĿǤ롣
+    $FRAME Υɥ $WIN κɸ ($X, $Y) ɽ롣$ITEMS 
+    ɽ٤ƥȥƥǤꡢ$NITEMS ϤθĿǤ롣
 
     @latexonly \IPAlabel{mdraw_text_items} @endlatexonly  */
 
@@ -2879,70 +2926,63 @@ mdraw_text_items (MFrame *frame, MDrawWindow win, int x, int y,
     }
 }
 
+/*=*/
+/***en
+    @brief Option of line breaking for drawing text.
+
+    The variable #mdraw_line_break_option specifies line breaking
+    options by logical-or of the members of #MTextLineBreakOption.  It
+    controls the line breaking algorithm of the function
+    mdraw_default_line_break ().  */
+    
+int mdraw_line_break_option;
+
 /*=*/
 /***en 
-      @brief   calculate a line breaking position.
+    @brief Calculate a line breaking position.
 
-      The function mdraw_default_line_break () calculates a line
-      breaking position based on the line number $LINE and the
-      coordinate $Y, when a line is too long to fit within the width
-      limit.  $POS is the position of the character next to the last
-      one that fits within the limit.  $FROM is the position of the
-      first character of the line, and TO is the position of the last
-      character displayed on the line if there were not width limit.
-      LINE and Y are reset to 0 when a line is broken by a newline
-      character, and incremented each time when a long line is broken
-      because of the width limit.  
-
-      @return 
-      This function returns a character position to break the
-      line.
+    The function mdraw_default_line_break () calculates a line
+    breaking position based on the line number $LINE and the
+    coordinate $Y, when a line is too long to fit within the width
+    limit.  $POS is the position of the character next to the last one
+    that fits within the limit.  $FROM is the position of the first
+    character of the line, and $TO is the position of the last
+    character displayed on the line if there were not width limit.
+    $LINE and $Y are reset to 0 when a line is broken by a newline
+    character, and incremented each time when a long line is broken
+    because of the width limit.
 
+    @return This function returns a character position to break the
+    line.
 */
+
 /***ja 
       @brief ԰֤׻.
 
-      ؿ mdraw_default_line_break () ϡԤ˼ޤʤ
-      ˹Ԥ֤򡢹ֹ LINE Ⱥɸ Y ˴ŤƷ׻롣
-      $POS Ϻ˼ޤǸʸμʸΰ֤Ǥ롣$FROM 
-      Ԥκǽʸΰ֡$TO ϺꤵƤʤФιԤ
-      ɽǸʸΰ֤Ǥ롣 LINE  Y ϲʸˤä
-      Ԥޤäݤˤ 0 ˥ꥻåȤ졢ˤäƹԤޤä
-      ˤ 1 Ť䤵롣
+      ؿ mdraw_default_line_break () ϡԤ˼ޤʤβ԰֤򡢹ֹ
+      $LINE Ⱥɸ $Y ˴ŤƷ׻롣
+      $POS Ϻ˼ޤǸʸμʸΰ֤Ǥ롣
+      $FROM ϤιԤκǽʸΰ֡$TO 
+      ϺꤵƤʤФιԤɽǸʸΰ֤Ǥ롣
+      $LINE  $Y ϲʸˤäƹԤޤäݤˤ 0
+      ˥ꥻåȤ졢ˤäƹԤޤäˤ 1 Ť䤵롣
 
       @return 
-      δؿϹԤʸ֤֤
+      δؿϲԤʸ֤֤
 */
 
 int
 mdraw_default_line_break (MText *mt, int pos,
 			  int from, int to, int line, int y)
 {
-  int c = mtext_ref_char (mt, pos);
-  int orig_pos = pos;
+  int p, after;
 
-  if (c == ' ' || c == '\t')
-    {
-      pos++;
-      while (pos < to
-	     && ((c = mtext_ref_char (mt, pos)) == ' ' || c == '\t'))
-	pos++;
-    }
-  else
-    {
-      while (pos > from)
-	{
-	  if (c == ' ' || c == '\t')
-	    break;
-	  pos--;
-	  c = mtext_ref_char (mt, pos);
-	}
-      if (pos == from)
-	pos = orig_pos;
-      else
-	pos++;
-    }
-  return pos;
+  p = mtext_line_break (mt, pos, mdraw_line_break_option, &after);
+  if (p < from)
+    p = from;
+  else if (p >= to)
+    p = to;
+  return p;
 }
 
 /*=*/
@@ -2967,17 +3007,17 @@ mdraw_default_line_break (MText *mt, int pos,
 /***ja
     @brief M-text ʸɽϰϾ.
 
-    ؿ mdraw_per_char_extents () ϡM-text $MT γʸɽϰ
-    ׻롣η׻ѤեȤϡ$MT Υƥȥץѥƥ
-    ꤵ줿եaե졼 $FRAME Υǥեȥեˤä
-    ޤ롣$ARRAY_RETURN γǤϡ M-text γʸɽϰ
-    ˤäƽ롣ɽϰϾϡM-text ɽ
-    а֤Ǥ롣$ARRAY_RETURN ǿϡM-text ΰʾǤʤ
-    Фʤʤ
+    ؿ mdraw_per_char_extents () ϡM-text $MT 
+    γʸɽϰϤ׻롣η׻ѤեȤϡ
+    $MT Υƥȥץѥƥǻꤵ줿եȡե졼 $FRAME
+    ΥǥեȥեˤäƷޤ롣$ARRAY_RETURN γǤϡ$MT
+    γʸɽϰϾˤäƽ롣ɽϰϾȤϡ
+    ɽа֤ȳʸĹǤ롣$ARRAY_RETURN 
+    ǿϡM-text ʸʾǤʤФʤʤ
 
-    ݥ $OVERALL_RETURN  @c NULL ǤʤϡƥΤɽ
-    ϰϾ׻η̤ $OVERALL_RETURN λؤ¤Τ˳Ǽ
-    롣
+    ݥ $OVERALL_RETURN  @c NULL 
+    ǤʤϡƥΤɽϰϾ׻η̤ 
+    $OVERALL_RETURN λؤ¤Τ˳Ǽ롣
 
     @latexonly \IPAlabel{mdraw_per_char_extents} @endlatexonly  */
 
@@ -2997,7 +3037,7 @@ mdraw_per_char_extents (MFrame *frame, MText *mt,
 
     The mdraw_clear_cache () function clear cached information
     on M-text $MT that was attached by any of the drawing functions.
-    When the behaviour of `format' or `line_break'
+    When the behavior of `format' or `line_break'
     member functions of MDrawControl is changed, the cache must be cleared.
 
     @seealso
@@ -3005,10 +3045,11 @@ mdraw_per_char_extents (MFrame *frame, MText *mt,
 /***ja 
     @brief åä.
 
-    ؿ mdraw_clear_cache () ؿˤä M-text $MT ղ
-    줿å򤹤٤ƾõ롣MDrawControl  `format' 
-    뤤 `line_break' дؿο񤤤Ѥäˤϥå
-    õʤƤϤʤʤ
+    ؿ mdraw_clear_cache () ؿˤä M-text $MT 
+    ղä줿å򤹤٤ƾõ롣MDrawControl  `format' 
+    뤤 `line_break' 
+    дؿο񤤤ѤäˤϥåõʤƤϤʤʤ
+
     @seealso
     MDrawControl */
 
