@@ -52,28 +52,25 @@
     @addtogroup m17nFace
     @brief եȤϡM-text θɤ椹륪֥ȤǤ.
 
-    @e ե  #MFace Υ֥ȤǤꡢM-text ɽˡ
-    椹롣եϸĿ @e եץѥƥ ġ
-    եץѥƥϥͤʤ롣ϰʲΥܥΤ줫Ǥ롣
+    @e ե  #MFace Υ֥ȤǤꡢM-text 
+    ɽˡ椹롣եϸĤ @e եץѥƥ ġ
+    ¾ΥץѥƥƱͥեץѥƥϥͤʤꡢϰʲΥܥΤ줫Ǥ롣
 
     #Mforeground, #Mbackground, #Mvideomode, #Mhline, #Mbox, 
     #Mfoundry, #Mfamily, #Mweight, #Mstyle, #Mstretch, #Madstyle, 
     #Msize, #Mfontset, #Mratio, #Mhook_func, #Mhook_arg
 
-    ֥ե F ΥեץѥƥΤ @c Mxxx ǤΡ
-    ΤȤñˡF  xxx ץѥƥפȸƤ֤Ȥ롣
+    ֥ե F ΥեץѥƥΤ @c Mxxx 
+    ǤΡפΤȤñˡF  xxx ץѥƥפȸƤ֤Ȥ롣
 
     M-text ɽؿϡޤǽˤ M-text 饭ܥ 
     #Mface Ǥ褦ʥƥȥץѥƥõˤͤ˽ä 
-    M-text ɽ롣ͤϥե֥ȤؤΥݥ󥿤Ǥʤ
-    Фʤʤ
+    M-text ɽ롣ͤϥե֥ȤؤΥݥ󥿤ǤʤФʤʤ
 
-     M-text #Mface 򥭡ȤƥȥץѥƥʣäƤꡢ
-    Ĥͤδ֤˾ͤʤʤСեȤ߹蘆
-    Ѥ롣
+    M-text #Mface 
+    򥭡ȤƥȥץѥƥʣäƤꡢĤͤͤʤʤСեȤ߹蘆Ѥ롣
 
-    ƥ°ɤΥեˤäƤꤵƤʤϡ
-    եȥեͤѤ롣  */
+    ƥ°ɤΥեˤäƤꤵƤʤϡǥեȥեͤѤ롣  */
 
 /*=*/
 
@@ -167,7 +164,7 @@ get_box_create (MFaceBoxProp *prop)
     FACE.  */
 
 static MRealizedFace *
-find_realized_face (MFrame *frame, MFace *face)
+find_realized_face (MFrame *frame, MFace *face, MFont *font)
 {
   MPlist *plist;
 
@@ -175,8 +172,11 @@ find_realized_face (MFrame *frame, MFace *face)
     {
       MRealizedFace *rface = MPLIST_VAL (plist);
 
-      if (memcmp (rface->face.property, face->property, sizeof face->property)
-	  == 0)
+      if (memcmp (rface->face.property, face->property,
+		  sizeof face->property) == 0
+	  && (rface->font
+	      ? (font && ! memcmp (rface->font, font, sizeof (MFont)))
+	      : ! font))
 	return rface;
     }
   return NULL;
@@ -417,7 +417,7 @@ mface__init ()
   MFaceHLineProp *hline;
   MFaceBoxProp *box;
 
-  face_table.count = 0;
+  M17N_OBJECT_ADD_ARRAY (face_table, "Face");
   Mface = msymbol_as_managing_key ("face");
   msymbol_put (Mface, Mtext_prop_serializer, (void *) serialize_face);
   msymbol_put (Mface, Mtext_prop_deserializer, (void *) deserialize_face);
@@ -592,27 +592,31 @@ mface__fini ()
   M17N_OBJECT_UNREF (box_prop_list);
 
   free (work_gstring.glyphs);
-
-  mdebug__report_object ("Face", &face_table);
 }
 
-/** Return a realized face for ASCII characters from NUM number of
-    base faces pointed by FACES on the frame FRAME.  */
+/** Return a face realized from NUM number of base faces pointed by
+    FACES on the frame FRAME.  If SIZE is nonzero, it specifies the
+    maximum font size.  */
 
 MRealizedFace *
-mface__realize (MFrame *frame, MFace **faces, int num, int size)
+mface__realize (MFrame *frame, MFace **faces, int num, int size, MFont *font)
 {
   MRealizedFace *rface;
   MRealizedFont *rfont;
   MFace merged_face = *(frame->face);
-  void **props;
   int i, j;
-  MGlyph g;
   MFaceHookFunc func;
+  MFont spec;
 
-  if (num == 0 && frame->rface)
+  if (num == 0 && frame->rface && ! font)
     return frame->rface;
 
+  if (! mplist_find_by_value (frame->face->frame_list, frame))
+    mplist_push (frame->face->frame_list, Mt, frame);
+  for (i = 0; i < num; i++)
+    if (! mplist_find_by_value (faces[i]->frame_list, frame))
+      mplist_push (faces[i]->frame_list, Mt, frame);
+
   for (i = 0; i < MFACE_PROPERTY_MAX; i++)
     for (j = num - 1; j >= 0; j--)
       if (faces[j]->property[i])
@@ -621,52 +625,136 @@ mface__realize (MFrame *frame, MFace **faces, int num, int size)
 	  break;
 	}
 
-  if (! mplist_find_by_value (frame->face->frame_list, frame))
-    mplist_push (frame->face->frame_list, Mt, frame);
-  for (i = 0; i < num; i++)
-    if (! mplist_find_by_value (faces[i]->frame_list, frame))
-      mplist_push (faces[i]->frame_list, Mt, frame);
-
-  if (merged_face.property[MFACE_RATIO])
+  if (font)
     {
-      int font_size = (int) merged_face.property[MFACE_SIZE];
-
-      font_size *= (int) merged_face.property[MFACE_RATIO];
-      font_size /= 100;
-      merged_face.property[MFACE_SIZE] = (void *) font_size;
-      merged_face.property[MFACE_RATIO] = 0;
+      if (font->type != MFONT_TYPE_REALIZED)
+	font = mfont_copy (font);
+      for (i = 0; i <= MFACE_ADSTYLE; i++)
+	if (font->property[i])
+	  merged_face.property[i] = FONT_PROPERTY (font, i);
+      if (font->size)
+	{
+	  if (font->size < 0)
+	    font->size = ((double) (- font->size)) * frame->dpi / 72.27 + 0.5;
+	  merged_face.property[MFACE_SIZE] = (void *) font->size;
+	  merged_face.property[MFACE_RATIO] = (void *) 0;
+	}
     }
 
-  rface = find_realized_face (frame, &merged_face);
+  if (! font || ! font->size)
+    {
+      double font_size = (int) merged_face.property[MFACE_SIZE];
+      int ifont_size;
+
+      if (font_size < 0)
+	font_size = - font_size * frame->dpi / 72.27;
+      if (merged_face.property[MFACE_RATIO]
+	  && (int) merged_face.property[MFACE_RATIO] != 100)
+	{
+	  font_size *= (int) merged_face.property[MFACE_RATIO];
+	  font_size /= 100;
+	}
+      ifont_size = font_size + 0.5;
+      merged_face.property[MFACE_SIZE] = (void *) ifont_size;
+      merged_face.property[MFACE_RATIO] = (void *) 0;
+    }
+
+  merged_face.property[MFACE_FOUNDRY] = Mnil;
+  rface = find_realized_face (frame, &merged_face, font);
   if (rface)
-    return rface;
+    {
+      if (font && font->type != MFONT_TYPE_REALIZED)
+	free (font);
+      return rface;
+    }
 
   MSTRUCT_CALLOC (rface, MERROR_FACE);
   mplist_push (frame->realized_face_list, Mt, rface);
   rface->frame = frame;
   rface->face = merged_face;
-  props = rface->face.property;
-  rface->rfontset = mfont__realize_fontset (frame,
-					    (MFontset *) props[MFACE_FONTSET],
-					    &merged_face);
-  g.type = GLYPH_SPACE;
-  g.c = ' ';
-  num = 1;
-  rfont = mfont__lookup_fontset (rface->rfontset, &g, &num,
-				 Mlatin, Mnil, Mnil, size);
+  rface->font = font;
+
+  if (font)
+    {
+      if (font->type == MFONT_TYPE_SPEC)
+	rfont = (MRealizedFont *) mfont_find (frame, font, NULL, 0);
+      else if (font->type == MFONT_TYPE_OBJECT)
+	{
+	  MFONT_INIT (&spec);
+	  spec.size = (int) merged_face.property[MFONT_SIZE];
+	  if (font->property[MFONT_REGISTRY])
+	    spec.property[MFONT_REGISTRY] = font->property[MFONT_REGISTRY];
+	  else
+	    mfont_put_prop (&spec, Mregistry,
+			    (font->source == MFONT_SOURCE_X
+			     ? Miso8859_1 : Municode_bmp));
+	  rfont = mfont__open (frame, font, &spec);
+	}
+      else
+	rfont = (MRealizedFont *) font;
+    }
+  else
+    {
+      MFontset *fontset = (MFontset *) merged_face.property[MFACE_FONTSET];
+
+      rface->rfontset = mfont__realize_fontset (frame, fontset, &merged_face,
+						font);
+      rfont = NULL;
+      mfont__set_spec_from_face (&spec, &merged_face);
+      mfont_put_prop (&spec, Mregistry, Miso8859_1);
+      spec.source = MFONT_SOURCE_X;
+      font = mfont__select (frame, &spec, 0);
+      if (font)
+	rfont = mfont__open (frame, font, &spec);
+      if (! rfont)
+	{
+	  mfont_put_prop (&spec, Mregistry, Municode_bmp);
+	  spec.source = MFONT_SOURCE_FT;
+	  font = mfont__select (frame, &spec, 0);
+	  if (font)
+	    rfont = mfont__open (frame, font, &spec);
+	}
+      if (! rfont)
+	{
+	  num = 0;
+	  rfont = mfont__lookup_fontset (rface->rfontset, NULL, &num,
+					 Mlatin, Mnil, Mnil, size, 0);
+	}
+    }
+
   if (rfont)
     {
       rface->rfont = rfont;
-      g.otf_encoded = 0;
-      work_gstring.frame = frame;
-      work_gstring.glyphs[0] = g;
+      rface->layouter = rfont->layouter;
+      rfont->layouter = Mnil;
       work_gstring.glyphs[0].rface = rface;
-      work_gstring.glyphs[1].code = MCHAR_INVALID_CODE;
-      work_gstring.glyphs[1].rface = rface;
-      mfont__get_metric (&work_gstring, 0, 2);
-      rface->space_width = work_gstring.glyphs[0].width;
-      rface->ascent = work_gstring.glyphs[1].ascent;
-      rface->descent = work_gstring.glyphs[1].descent;
+      work_gstring.glyphs[0].code = MCHAR_INVALID_CODE;
+      mfont__get_metric (&work_gstring, 0, 1);
+      rface->ascent = work_gstring.glyphs[0].ascent;
+      rface->descent = work_gstring.glyphs[0].descent;
+      work_gstring.glyphs[0].code
+	= mfont__encode_char (frame, (MFont *) rfont, NULL, ' ');
+      if (work_gstring.glyphs[0].code != MCHAR_INVALID_CODE)
+	{
+	  mfont__get_metric (&work_gstring, 0, 1);
+	  rface->space_width = work_gstring.glyphs[0].width;
+	}
+      else
+	rface->space_width = rfont->spec.size / 10;
+      if (rfont->average_width)
+	rface->average_width = rfont->average_width;
+      else
+	{
+	  work_gstring.glyphs[0].code
+	    = mfont__encode_char (frame, (MFont *) rfont, NULL, 'x');
+	  if (work_gstring.glyphs[0].code != MCHAR_INVALID_CODE)
+	    {
+	      mfont__get_metric (&work_gstring, 0, 1);
+	      rface->average_width = work_gstring.glyphs[0].width;
+	    }
+	  else
+	    rface->average_width = rface->space_width;
+	}
     }
   else
     {
@@ -674,10 +762,10 @@ mface__realize (MFrame *frame, MFace **faces, int num, int size)
       rface->space_width = frame->space_width;
     }
 
-  rface->hline = (MFaceHLineProp *) props[MFACE_HLINE];
+  rface->hline = (MFaceHLineProp *) merged_face.property[MFACE_HLINE];
   if (rface->hline && rface->hline->width == 0)
     rface->hline = NULL;
-  rface->box = (MFaceBoxProp *) props[MFACE_BOX];
+  rface->box = (MFaceBoxProp *) merged_face.property[MFACE_BOX];
   if (rface->box && rface->box->width == 0)
     rface->box = NULL;
   rface->ascii_rface = rface;
@@ -707,13 +795,98 @@ MGlyph *
 mface__for_chars (MSymbol script, MSymbol language, MSymbol charset,
 		  MGlyph *from_g, MGlyph *to_g, int size)
 {
-  MRealizedFont *rfont;
+  MRealizedFont *rfont = from_g->rface->rfont;
+  MSymbol layouter;
   int num = to_g - from_g;
+  int i;
+
+  if (from_g->rface->font)
+    {
+      MRealizedFace *rface = from_g->rface, *new;
+
+      if (! rfont)
+	rfont = mfontset__get_font (rface->frame,
+				    rface->face.property[MFACE_FONTSET], 
+				    script, language,
+				    rface->font, NULL);
+      else if (script != Mlatin)
+	rfont = mfontset__get_font (rface->frame,
+				    rface->face.property[MFACE_FONTSET],
+				    script, language,
+				    (MFont *) rfont, NULL);
+      if (! rfont)
+	{
+	  for (; from_g < to_g && from_g->rface->font; from_g++)
+	    from_g->code = MCHAR_INVALID_CODE;
+	}
+      else
+	{
+	  if (rface->rfont == rfont && rfont->layouter == Mnil)
+	    new = rface;
+	  else
+	    {
+	      MSTRUCT_MALLOC (new, MERROR_FACE);
+	      mplist_push (rface->non_ascii_list, Mt, new);
+	      *new = *rface;
+	      new->rfont = rfont;
+	      new->layouter = rfont->layouter;
+	      rfont->layouter = Mnil;
+	      new->non_ascii_list = NULL;
+	      new->ascent = rfont->ascent;
+	      new->descent = rfont->descent;
+	    } 
+	  for (; from_g < to_g && from_g->rface->font; from_g++)
+	    {
+	      from_g->rface = new;
+	      if (new->layouter)
+		{
+		  from_g->code = mfont__flt_encode_char (new->layouter, 
+							 from_g->c);
+		  if (from_g->code == MCHAR_INVALID_CODE)
+		    {
+		      from_g->rface = rface;
+		      from_g->code = mfont__encode_char (rfont->frame, 
+							 (MFont *) rfont,
+							 NULL, from_g->c);
+		    }
+		}
+	      else
+		from_g->code = mfont__encode_char (rfont->frame, 
+						   (MFont *) rfont,
+						   NULL, from_g->c);
+	    }
+	}
+      return from_g;
+    }
+
+  if (rfont && script == Mlatin)
+    {
+      for (i = 0; i < num; i++)
+	{
+	  unsigned code = mfont__encode_char (rfont->frame, (MFont *) rfont,
+					      NULL, from_g[i].c);
+	  if (code == MCHAR_INVALID_CODE)
+	    break;
+	  from_g[i].code = code;
+	}
+      if (i == num || from_g[i].rface->font)
+	return from_g + i;
+    }
 
   rfont = mfont__lookup_fontset (from_g->rface->rfontset, from_g, &num,
-				 script, language, charset, size);
-  if (! rfont)
-    num = 1;
+				 script, language, charset, size, 0);
+  if (rfont)
+    {
+      layouter = rfont->layouter;
+      rfont->layouter = Mnil;
+    }
+  else
+    {
+      from_g->code = MCHAR_INVALID_CODE;
+      num = 1;
+      rfont = NULL;
+      layouter = Mnil;
+    }
   
   to_g = from_g + num;
   while (from_g < to_g)
@@ -722,19 +895,26 @@ mface__for_chars (MSymbol script, MSymbol language, MSymbol charset,
       MRealizedFace *rface = from_g++->rface;
 
       while (from_g < to_g && rface == from_g->rface) from_g++;
-      if (rface->rfont != rfont)
+      if (rface->rfont != rfont
+	  || rface->layouter != layouter)
 	{
 	  MPlist *plist = mplist_find_by_value (rface->non_ascii_list, rfont);
-	  MRealizedFace *new;
+	  MRealizedFace *new = NULL;
 
-	  if (plist)
-	    new = MPLIST_VAL (plist);
-	  else
+	  while (plist)
+	    {
+	      new = MPLIST_VAL (plist);
+	      if (new->layouter == layouter)
+		break;
+	      plist = mplist_find_by_value (MPLIST_NEXT (plist), rfont);
+	    }
+	  if (! plist)
 	    {
 	      MSTRUCT_MALLOC (new, MERROR_FACE);
 	      mplist_push (rface->non_ascii_list, Mt, new);
 	      *new = *rface;
 	      new->rfont = rfont;
+	      new->layouter = layouter;
 	      new->non_ascii_list = NULL;
 	      if (rfont)
 		{
@@ -758,6 +938,8 @@ mface__free_realized (MRealizedFace *rface)
   MPLIST_DO (plist, rface->non_ascii_list)
     free (MPLIST_VAL (plist));
   M17N_OBJECT_UNREF (rface->non_ascii_list);
+  if (rface->font && rface->font->type != MFONT_TYPE_REALIZED)
+    free (rface->font);
   free (rface);
 }
 
@@ -765,8 +947,9 @@ void
 mface__update_frame_face (MFrame *frame)
 {
   frame->rface = NULL;
-  frame->rface = mface__realize (frame, NULL, 0, 0);
+  frame->rface = mface__realize (frame, NULL, 0, 0, NULL);
   frame->space_width = frame->rface->space_width;
+  frame->average_width = frame->rface->average_width;
   frame->ascent = frame->rface->ascent;
   frame->descent = frame->rface->descent;
 }
@@ -795,15 +978,14 @@ mface__update_frame_face (MFrame *frame)
     #Mnil means that the face does not specify a foreground color.
     Otherwise, the foreground of an M-text is drawn by the specified
     color.  */
-
 /***ja
     @brief ʿꤹեץѥƥΥ.
 
     ѿ #Mforeground ϥեץѥƥΥȤѤ롣
     ץѥƥͤϡ̾̾Ȥƻĥܥ뤫 #Mnil Ǥ롣
 
-    #Mnil ξ硢ʿϻꤵʤǤʤ M-text ʤ
-    ꤵ줿ɽ롣  */
+    #Mnil ξ硢ʿϻꤵʤǤʤ M-text 
+    ʤϻꤵ줿ɽ롣  */
 
 MSymbol Mforeground;
 
@@ -817,15 +999,14 @@ MSymbol Mforeground;
     #Mnil means that the face does not specify a background color.
     Otherwise, the background of an M-text is drawn by the specified
     color.  */
-
 /***ja
     @brief طʿꤹ뤿ΥեץѥƥΥ.
 
     ѿ #Mbackground ϥեץѥƥΥȤѤ롣
     ץѥƥͤϡ̾̾Ȥƻĥܥ뤫 #Mnil Ǥ롣
 
-    #Mnil ξ硢طʿϻꤵʤǤʤ M-text طʤ
-    ꤵ줿ɽ롣  */
+    #Mnil ξ硢طʿϻꤵʤǤʤ M-text 
+    طʤϻꤵ줿ɽ롣  */
 
 MSymbol Mbackground;
 
@@ -844,18 +1025,16 @@ MSymbol Mbackground;
     is drawn by foreground color).
 
     #Mnil means that the face does not specify a video mode.  */
-
 /***ja
     @brief ӥǥ⡼ɤꤹ뤿ΥեץѥƥΥ.
 
-    ѿ #Mvideomode ϥեץѥƥΥȤѤ롣
-    ץѥƥͤϡ#Mnormal, #Mreverse, #Mnil Τ줫ǤʤƤϤʤʤ
+    ѿ #Mvideomode ϥեץѥƥΥȤѤ롣ץѥƥͤϡ
+    #Mnormal, #Mreverse, #Mnil Τ줫ǤʤƤϤʤʤ
 
-    #Mnormal ξϡM-text ɸΥӥǥ⡼ɡʤʿǡ
-    ʤطʿǡˤɽ롣
+    #Mnormal ξϡM-text 
+    ɸΥӥǥ⡼ɡʤʿǡطʤطʿǡˤɽ롣
 
-    #Mreverse ξϥСӥǥ⡼ɤǡʤطʿǡطʤ
-    ʿǡɽ롣
+    #Mreverse ξϥСӥǥ⡼ɤǡʤطʿǡطʤʿǡɽ롣
 
     #Mnil ξϥӥǥ⡼ɤϻꤵʤ
     */
@@ -880,8 +1059,7 @@ MSymbol Mvideomode;
 
     ͤ0ʤСեȥϻꤵʤǤʤСM-text 
     (FONTSIZE * RATIO / 100) ȤΥեȤɽ롣
-     FONTSIZE ϥեץѥƥ #Msize ǻꤵ줿Ǥ
-    롣 */
+    FONTSIZE ϥեץѥƥ#Msize ǻꤵ줿Ǥ롣 */
 
 MSymbol Mratio;
 
@@ -895,17 +1073,15 @@ MSymbol Mratio;
     The value @c NULL means that the face does not specify this
     property.  Otherwise, an M-text is drawn with a horizontal line by
     a way specified by the object that the value points to.  */
-
 /***ja
     @brief ʿꤹ뤿ΥեץѥƥΥ.
 
     ѿ #Mhline ϥեץѥƥΥȤѤ롣ͤ 
-    #MFaceHLineProp ֥ȤؤΥݥ󥿤 @c NULL ǤʤƤϤ
-    ʤ
+    #MFaceHLineProp ֥ȤؤΥݥ󥿤 @c NULL ǤʤƤϤʤʤ
 
-    ͤ @c NULL ʤСΥץѥƥϻꤵʤǤʤ
-   ؤ֥Ȥ˻ꤵ줿褦˿ʿղä M-text ɽ
-   롣*/
+    ͤ @c NULL ʤСΥץѥƥϻꤵʤ
+    Ǥʤͤؤ֥Ȥ˻ꤵ줿褦˿ʿղä M-text 
+    ɽ롣*/
 
 MSymbol Mhline;
 
@@ -918,17 +1094,15 @@ MSymbol Mhline;
     The value @c NULL means that the face does not specify a box.
     Otherwise, an M-text is drawn with a surrounding box by a way
     specified by the object that the value points to.  */
-
 /***ja
     @brief ϤȤꤹ뤿ΥեץѥƥΥ.
 
     ѿ #Mbox ϥեץѥƥΥȤѤ롣ͤ 
-    #MFaceBoxProp ֥ȤؤΥݥ󥿤 @c NULL ǤʤƤϤʤ
-    ʤ
+    #MFaceBoxProp ֥ȤؤΥݥ󥿤 @c NULL ǤʤƤϤʤʤ
 
-    ͤ @c NULL ʤСΥץѥƥϻꤵʤǤʤ
-    ؤ֥Ȥ˻ꤵ줿褦˰ϤȤղä M-text ɽ
-    롣*/
+    ͤ @c NULL ʤСΥեϰϤȤꤷƤʤ
+    Ǥʤͤؤ֥Ȥ˻ꤵ줿褦˰ϤȤղä 
+    M-text ɽ롣*/
 
 MSymbol Mbox;
 
@@ -942,15 +1116,14 @@ MSymbol Mbox;
     The value @c NULL means that the face does not specify a fontset.
     Otherwise, an M-text is drawn with a font selected from what
     specified in the fontset.  */
-
 /***ja
     @brief եȥåȤꤹ뤿ΥեץѥƥΥ.
 
     ѿ #Mfontset ϥեץѥƥΥȤѤ롣ͤ 
     #Mfontset ֥ȤؤΥݥ󥿤 @c NULL ǤʤƤϤʤʤ
 
-    ͤ @c NULL ʤСեȥåȤϻꤵʤǤʤ
-    ؤ֥Ȥ˻ꤵ줿եȥåȤեȤ 
+    ͤ @c NULL ʤСեȥåȤϻꤵƤʤ
+    Ǥʤͤؤ֥Ȥ˻ꤵ줿եȥåȤեȤ 
     M-text ɽ롣*/
     
 MSymbol Mfontset;
@@ -970,8 +1143,8 @@ MSymbol Mfontset;
     ѿ #Mhook_func ϥեץѥƥΥȤѤ롣ͤ 
     #MFaceHookFunc δؿ @c NULL ǤʤƤϤʤʤ
 
-    ͤ @c NULL ʤСեåϻꤵʤǤʤХե
-    ¸˻ꤷؿƤФ롣      */
+    ͤ @c NULL ʤСեåϻꤵƤʤ
+    ǤʤХե¸˻ꤵ줿ؿƤФ롣      */
 MSymbol Mhook_func;
 
 /***en
@@ -983,9 +1156,8 @@ MSymbol Mhook_func;
 /***ja
     @brief եåΰꤹ뤿ΥեץѥƥΥ.
 
-    ѿ #Mhook_arg ϥեץѥƥΥȤѤ롣ͤ 
-    Ǥ褯եץѥƥ #Mhook_func ǻꤵؿϤ
-    롣 */
+    ѿ #Mhook_arg ϥեץѥƥΥȤѤ롣
+    ͤϲǤ褯եץѥƥ #Mhook_func ǻꤵؿϤ롣 */
 MSymbol Mhook_arg;
 
 /*** @} */
@@ -1025,8 +1197,8 @@ MSymbol Mreverse;
 
     ѿ #mface_normal_video  #Mvideomode ץѥƥͤ #Mnormal 
     Ǥեؤݥ󥿤Ǥ롣¾Υץѥƥϻꤵʤ
-    ΥեɽM-text ɸο (ʤʤʿ
-    طʤطʿˤ롣  */
+    ΥեɽM-text 
+    ɸο (ʤʤʿطʤطʿˤ롣  */
 
 MFace *mface_normal_video;
 
@@ -1042,9 +1214,9 @@ MFace *mface_normal_video;
     @brief Сӥǥե.
 
     ѿ #mface_reverse_video  #Mvideomode ץѥƥͤ 
-    #Mreverse Ǥեؤݥ󥿤Ǥ롣¾Υץѥƥϻ
-    ʤΥեɽM-text ʿطʿ
-    ä (ʤʤطʿطʤʿ롣  */
+    #Mreverse Ǥեؤݥ󥿤Ǥ롣¾Υץѥƥϻꤵʤ
+    ΥեɽM-text 
+    ʿطʿؤä (ʤʤطʿطʤʿ롣  */
 
 MFace *mface_reverse_video;
 
@@ -1069,8 +1241,7 @@ MFace *mface_reverse_video;
     @brief ե.
 
     ѿ #mface_underline  #Mhline ץѥƥͤ #MFaceHLineProp 
-    ֥ȤؤΥݥ󥿤Ǥեؤݥ󥿤Ǥ롣
-    ȤΥФϰʲ̤ꡣ
+    ֥ȤؤΥݥ󥿤Ǥեؤݥ󥿤Ǥ롣֥ȤΥФϰʲ̤ꡣ
 
 @verbatim
       
@@ -1080,8 +1251,7 @@ MFace *mface_reverse_video;
     color   Mnil
 @endverbatim
 
-    ¾ΥץѥƥϻꤵʤΥե M-text ϲդ
-    ɽ롣*/ 
+    ¾ΥץѥƥϻꤵʤΥե M-text ϲդɽ롣*/ 
 
 MFace *mface_underline;
 
@@ -1095,10 +1265,10 @@ MFace *mface_underline;
 /***ja
     @brief ߥǥե.
 
-    ѿ #mface_medium  #Mweight ץѥƥͤ "medium" Ȥ̾
-    ĥܥǤ褦ʥեؤݥ󥿤Ǥ롣¾Υץ
-    ѥƥϻꤵʤΥե M-text ϡߥǥ०
-    ȤΥեȤɽ롣  */
+    ѿ #mface_medium  #Mweight ץѥƥͤ "medium" 
+    Ȥ̾ĥܥǤ褦ʥեؤݥ󥿤Ǥ롣
+    ¾ΥץѥƥϻꤵʤΥե M-text
+    ϡߥǥ०ȤΥեȤɽ롣  */
 MFace *mface_medium;
 
 /***en
@@ -1108,15 +1278,13 @@ MFace *mface_medium;
     property with value a symbol of name "bold".  The other properties
     are not specified.  An M-text that has this face is drawn with a
     font of bold weight.  */
-
 /***ja
     @brief ܡɥե.
 
-    ѿ #mface_bold  #Mweight ץѥƥͤ "bold" Ȥ̾
-    ĥܥǤ褦ʥեؤݥ󥿤Ǥ롣¾Υץѥƥ
-    ϻꤵʤΥե M-text ϡܡɤΥեȤ
-    ɽ롣
-     */
+    ѿ #mface_bold  #Mweight ץѥƥͤ "bold" 
+    Ȥ̾ĥܥǤ褦ʥեؤݥ󥿤Ǥ롣
+    ¾ΥץѥƥϻꤵʤΥե M-text 
+    ϡܡɥեȤɽ롣    */
 
 MFace *mface_bold;
 
@@ -1127,15 +1295,13 @@ MFace *mface_bold;
     property with value a symbol of name "italic".  The other
     properties are not specified.  An M-text that has this face is
     drawn with a font of italic style.  */
-
 /***ja
     @brief åե.
 
-    ѿ #mface_italic  #Mstyle ץѥƥͤ "italic" Ȥ̾
-    ĥܥǤ褦ʥեؤݥ󥿤Ǥ롣¾Υץ
-    ƥϻꤵʤΥե M-text ϡåΤɽ
-    롣
-     */
+    ѿ #mface_italic  #Mstyle ץѥƥͤ "italic" 
+    Ȥ̾ĥܥǤ褦ʥեؤݥ󥿤Ǥ롣
+    ¾ΥץѥƥϻꤵʤΥե M-text
+    ϡåΤɽ롣    */
 
 MFace *mface_italic;
 
@@ -1147,16 +1313,14 @@ MFace *mface_italic;
     property with value a symbol of name "italic".  The other
     properties are not specified.  An M-text that has this face is
     drawn with a font of bold weight and italic style.  */
-
 /***ja
     @brief ܡɥåե.
 
-    ѿ #mface_bold_italic ϡ#Mweight ץѥƥͤ "bold" Ȥ
-    ̾ĥܥǤꡢ #Mstyle ץѥƥͤ "italic" 
+    ѿ #mface_bold_italic ϡ#Mweight ץѥƥͤ "bold" 
+    Ȥ̾ĥܥǤꡢ #Mstyle ץѥƥͤ "italic" 
     Ȥ̾ĥܥǤ褦ʥեؤݥ󥿤Ǥ롣
-    ¾ΥץѥƥϻꤵʤΥե M-text ϡܡ
-    ɥåΤɽ롣
-    */
+    ¾ΥץѥƥϻꤵʤΥե M-text 
+    ϡܡɥåΤɽ롣    */
 
 MFace *mface_bold_italic;
 
@@ -1167,13 +1331,12 @@ MFace *mface_bold_italic;
     property with value 50.  The other properties are not specified.
     An M-text that has this face is drawn with a font whose size is
     50% of a normal font.  */
-
 /***ja
     @brief ǾΥե.
 
-    ѿ #mface_xx_small ϡ#Mratio ץѥƥͤ 50 Ǥե
-    ؤݥ󥿤Ǥ롣¾ΥץѥƥϻꤵʤΥե
-     M-text ɸ 50% 礭ΥեȤѤɽ롣
+    ѿ #mface_xx_small ϡ#Mratio ץѥƥͤ 50 
+    Ǥեؤݥ󥿤Ǥ롣¾Υץѥƥϻꤵʤ
+    Υե M-text ɸեȤ 50% 礭ΥեȤѤɽ롣
      */
 
 MFace *mface_xx_small;
@@ -1185,13 +1348,12 @@ MFace *mface_xx_small;
     property with value 66.  The other properties are not specified.
     An M-text that has this face is drawn with a font whose size is
     66% of a normal font.  */
-
 /***ja
-    @brief äȾե.
+    @brief 꾮ե.
 
-    ѿ #mface_x_small ϡ#Mratio ץѥƥͤ 66 Ǥե
-    ؤݥ󥿤Ǥ롣¾ΥץѥƥϻꤵʤΥե
-     M-text ɸ 66% 礭ΥեȤѤɽ롣
+    ѿ #mface_x_small ϡ#Mratio ץѥƥͤ 66 
+    Ǥեؤݥ󥿤Ǥ롣¾Υץѥƥϻꤵʤ
+    Υե M-text ɸեȤ 66% 礭ΥեȤѤɽ롣
      */
 
 MFace *mface_x_small;
@@ -1203,13 +1365,12 @@ MFace *mface_x_small;
     property with value 75.  The other properties are not specified.
     An M-text that has this face is drawn with a font whose size is
     75% of a normal font.  */
-
 /***ja
     @brief ե.
 
-    ѿ #mface_small ϡ#Mratio ץѥƥͤ 75 Ǥե
-    ؤݥ󥿤Ǥ롣¾ΥץѥƥϻꤵʤΥե
-     M-text ɸ 75% 礭ΥեȤѤɽ롣
+    ѿ #mface_small ϡ#Mratio ץѥƥͤ 75 
+    Ǥեؤݥ󥿤Ǥ롣¾Υץѥƥϻꤵʤ
+    Υե M-text ɸեȤ 75% 礭ΥեȤѤɽ롣
      */
 
 MFace *mface_small;
@@ -1221,13 +1382,12 @@ MFace *mface_small;
     #Mratio property with value 100.  The other properties are not
     specified.  An M-text that has this face is drawn with a font
     whose size is the same as a normal font.  */
-
 /***ja
     @brief ɸ礭Υե.
 
-    ѿ #mface_normalsize ϡ#Mratio ץѥƥͤ 100 Ǥե
-    ؤݥ󥿤Ǥ롣¾ΥץѥƥϻꤵʤΥե
-     M-text ɸƱ礭ΥեȤѤɽ롣
+    ѿ #mface_normalsize ϡ#Mratio ץѥƥͤ 100 
+    Ǥեؤݥ󥿤Ǥ롣¾Υץѥƥϻꤵʤ
+    Υե M-text ɸեȤƱ礭ΥեȤѤɽ롣
      */
 
 MFace *mface_normalsize;
@@ -1239,13 +1399,12 @@ MFace *mface_normalsize;
     property with value 120.  The other properties are not specified.
     An M-text that has this face is drawn with a font whose size is
     120% of a normal font.  */
-
 /***ja
     @brief 礭ե.
 
-    ѿ #mface_large ϡ#Mratio ץѥƥͤ 120 Ǥե
-    ؤݥ󥿤Ǥ롣¾ΥץѥƥϻꤵʤΥե
-     M-text ɸ 120% 礭ΥեȤѤɽ롣
+    ѿ #mface_large ϡ#Mratio ץѥƥͤ 120 
+    Ǥեؤݥ󥿤Ǥ롣¾Υץѥƥϻꤵʤ
+    Υե M-text ɸեȤ 120% 礭ΥեȤѤɽ롣
      */
 
 MFace *mface_large;
@@ -1257,13 +1416,12 @@ MFace *mface_large;
     property with value 150.  The other properties are not specified.
     An M-text that has this face is drawn with a font whose size is
     150% of a normal font.  */
-
 /***ja
     @brief ä礭ե.
 
-    ѿ #mface_x_large ϡ#Mratio ץѥƥͤ 150 Ǥե
-    ؤݥ󥿤Ǥ롣¾ΥץѥƥϻꤵʤΥե
-     M-text ɸ 150% 礭ΥեȤѤɽ롣
+    ѿ #mface_x_large ϡ#Mratio ץѥƥͤ 150 
+    Ǥեؤݥ󥿤Ǥ롣¾Υץѥƥϻꤵʤ
+    Υե M-text ɸեȤ 150% 礭ΥեȤѤɽ롣
      */
 
 MFace *mface_x_large;
@@ -1275,13 +1433,12 @@ MFace *mface_x_large;
     property with value 200.  The other properties are not specified.
     An M-text that has this face is drawn with a font whose size is
     200% of a normal font.  */
-
 /***ja
     @brief Υե.
 
-    ѿ #mface_xx_large ϡ#Mratio ץѥƥͤ 200 Ǥե
-    ؤݥ󥿤Ǥ롣¾ΥץѥƥϻꤵʤΥե
-     M-text ɸ 200% 礭ΥեȤѤɽ롣
+    ѿ #mface_xx_large ϡ#Mratio ץѥƥͤ 200 
+    Ǥեؤݥ󥿤Ǥ롣¾Υץѥƥϻꤵʤ
+    Υե M-text ɸեȤ 200% 礭ΥեȤѤɽ롣
      */
 
 MFace *mface_xx_large;
@@ -1293,14 +1450,13 @@ MFace *mface_xx_large;
     #Mforeground property with value a symbol of name "black".  The
     other properties are not specified.  An M-text that has this face
     is drawn with black foreground.  */
-
 /***ja 
     @brief ե.
 
     ѿ #mface_black ϡ#Mforeground ץѥƥͤȤ "black" 
-    ̾ΥܥĤ褦ʥեؤݥ󥿤Ǥ롣¾Υ
-    ѥƥϻꤵʤΥե M-text ʿȤ
-    ɽ롣     */
+    Ȥ̾ΥܥĤ褦ʥեؤݥ󥿤Ǥ롣
+    ¾ΥץѥƥϻꤵʤΥե M-text 
+    ʿȤƹѤɽ롣     */
 
 MFace *mface_black;
 
@@ -1311,14 +1467,13 @@ MFace *mface_black;
     #Mforeground property with value a symbol of name "white".  The
     other properties are not specified.  An M-text that has this face
     is drawn with white foreground.  */
-
 /***ja
     @brief ե.
 
     ѿ #mface_white ϡ#Mforeground ץѥƥͤȤ "white" 
-    ̾ΥܥĤ褦ʥեؤݥ󥿤Ǥ롣¾Υ
-    ѥƥϻꤵʤΥե M-text ʿȤ
-    ɽ롣  */
+    Ȥ̾ΥܥĤ褦ʥեؤݥ󥿤Ǥ롣
+    ¾ΥץѥƥϻꤵʤΥե M-text 
+    ʿȤѤɽ롣     */
 
 MFace *mface_white;
 
@@ -1329,14 +1484,13 @@ MFace *mface_white;
     #Mforeground property with value a symbol of name "red".  The
     other properties are not specified.  An M-text that has this face
     is drawn with red foreground.  */
-
 /***ja
     @brief ֥ե.
 
-    ѿ #mface_red ϡ#Mforeground ץѥƥͤȤ "red" Ȥ
-    ̾ΥܥĤ褦ʥեؤݥ󥿤Ǥ롣¾Υץ
-    ƥϻꤵʤΥե M-text ʿ֤Ȥɽ
-    롣  */
+    ѿ #mface_red ϡ#Mforeground ץѥƥͤȤ "red" 
+    Ȥ̾ΥܥĤ褦ʥեؤݥ󥿤Ǥ롣
+    ¾ΥץѥƥϻꤵʤΥե M-text 
+    ʿȤ֤Ѥɽ롣  */
 
 MFace *mface_red;
 
@@ -1347,14 +1501,13 @@ MFace *mface_red;
     #Mforeground property with value a symbol of name "green".  The
     other properties are not specified.  An M-text that has this face
     is drawn with green foreground.  */
-
 /***ja
     @brief Хե.
 
     ѿ #mface_green ϡ#Mforeground ץѥƥͤȤ "green" 
-    ̾ΥܥĤ褦ʥեؤݥ󥿤Ǥ롣¾Υ
-    ѥƥϻꤵʤΥե M-text ʿФȤ
-    ɽ롣  */
+    Ȥ̾ΥܥĤ褦ʥեؤݥ󥿤Ǥ롣
+    ¾ΥץѥƥϻꤵʤΥե M-text 
+    ʿȤФѤɽ롣  */
 
 MFace *mface_green;
 
@@ -1365,14 +1518,13 @@ MFace *mface_green;
     #Mforeground property with value a symbol of name "blue".  The
     other properties are not specified.  An M-text that has this face
     is drawn with blue foreground.  */
-
 /***ja
     @brief ĥե.
 
-    ѿ #mface_blue ϡ#Mforeground ץѥƥͤȤ "blue" Ȥ
-    ̾ΥܥĤ褦ʥեؤݥ󥿤Ǥ롣¾Υץ
-    ѥƥϻꤵʤΥե M-text ʿĤȤ
-    ɽ롣  */
+    ѿ #mface_blue ϡ#Mforeground ץѥƥͤȤ "blue" 
+    Ȥ̾ΥܥĤ褦ʥեؤݥ󥿤Ǥ롣
+    ¾ΥץѥƥϻꤵʤΥե M-text 
+    ʿȤĤѤɽ롣  */
 
 MFace *mface_blue;
 
@@ -1383,14 +1535,13 @@ MFace *mface_blue;
     #Mforeground property with value a symbol of name "cyan".  The
     other properties are not specified.  An M-text that has this face
     is drawn with cyan foreground.  */
-
 /***ja
     @brief ե.
 
-    ѿ #mface_cyan ϡ#Mforeground ץѥƥͤȤ "cyan" Ȥ
-    ̾ΥܥĤ褦ʥեؤݥ󥿤Ǥ롣¾Υץ
-    ѥƥϻꤵʤΥե M-text ʿ򥷥
-    ɽ롣  */
+    ѿ #mface_cyan ϡ#Mforeground ץѥƥͤȤ "cyan"
+    Ȥ̾ΥܥĤ褦ʥեؤݥ󥿤Ǥ롣
+    ¾ΥץѥƥϻꤵʤΥե M-text 
+    ʿȤƥѤɽ롣  */
 
 MFace *mface_cyan;
 
@@ -1406,9 +1557,9 @@ MFace *mface_cyan;
     @brief ե.
 
     ѿ #mface_yellow ϡ#Mforeground ץѥƥͤȤ "yellow" 
-    Ȥ̾ΥܥĤ褦ʥեؤݥ󥿤Ǥ롣¾
-    ץѥƥϻꤵʤΥե M-text ʿ򲫿
-    Ȥɽ롣  */
+    Ȥ̾ΥܥĤ褦ʥեؤݥ󥿤Ǥ롣
+    ¾ΥץѥƥϻꤵʤΥե M-text 
+    ʿȤƲѤɽ롣  */
 
 MFace *mface_yellow;
 
@@ -1424,9 +1575,9 @@ MFace *mface_yellow;
     @brief ޥ󥿥ե.
 
     ѿ #mface_magenta ϡ#Mforeground ץѥƥͤȤ 
-    "magenta" Ȥ̾ΥܥĤ褦ʥեؤݥ󥿤
-    롣¾ΥץѥƥϻꤵʤΥե M-text 
-    ʿޥ󥿤Ȥɽ롣  */
+    "magenta" Ȥ̾ΥܥĤ褦ʥեؤݥ󥿤Ǥ롣
+    ¾ΥץѥƥϻꤵʤΥե M-text 
+    ʿȤƥޥ󥿤Ѥɽ롣  */
 
 MFace *mface_magenta;
 
@@ -1442,17 +1593,16 @@ MFace *mface_magenta;
     @brief Key of a text property specifying a face.
 
     The variable #Mface is a symbol of name <tt>"face"</tt>.  A text
-    property whose key is
-    this symbol must have a pointer to an object
-    of type #MFace
-.  This is a managing key.  */
+    property whose key is this symbol must have a pointer to an object
+    of type #MFace.  This is a managing key.  */
 
 /***ja
     @brief եꤹƥȥץѥƥΥ.
 
-    ѿ #Mface  <tt>"face"</tt> Ȥ̾ĥܥǤ롣
-    Υܥ򥭡Ȥƥȥץѥƥϡ#MFace Υ֥
-    ȤؤΥݥ󥿤ʤФʤʤϴǤ롣  */
+    ѿ #Mface  <tt>"face"</tt> 
+    Ȥ̾ĥܥǤ롣Υܥ򥭡Ȥƥȥץѥƥϡ
+    #MFace Υ֥ȤؤΥݥ󥿤ʤФʤʤ
+    ϴǤ롣  */
 
 MSymbol Mface;
 /*=*/
@@ -1471,8 +1621,8 @@ MSymbol Mface;
 /***ja
     @brief եĤ.
 
-    ؿ mface () ϥץѥƥڻʤե֥
-    롣
+    ؿ mface () 
+    ϥץѥƥڻʤե֥Ȥ롣
 
     @return
     δؿϺäեؤΥݥ󥿤֤  */
@@ -1499,8 +1649,8 @@ mface ()
 /***ja
     @brief եΥԡ.
 
-    ؿ mface_copy () ϥե $FACE ΥԡꡢΥԡؤ
-    ݥ󥿤֤  */
+    ؿ mface_copy () ϥե $FACE 
+    ΥԡꡢΥԡؤΥݥ󥿤֤  */
 
 MFace *
 mface_copy (MFace *face)
@@ -1517,6 +1667,54 @@ mface_copy (MFace *face)
   return copy;
 }
 
+/*=*/
+/***en
+    @brief Compare faces.
+
+    The mface_equal () function compares faces $FACE1 and $FACE2.
+
+    @return If two faces have the same property values, return 1.
+    Otherwise return 0.  */
+
+int
+mface_equal (MFace *face1, MFace *face2)
+{
+  MFaceHLineProp *hline1, *hline2;
+  MFaceBoxProp *box1, *box2;
+  int i;
+
+  if (face1 == face2)
+    return 1;
+  if (memcmp (face1->property, face2->property, sizeof face1->property) == 0)
+    return 1;
+  for (i = MFACE_FOUNDRY; i <= MFACE_BACKGROUND; i++)
+    if (face1->property[i] != face2->property[i])
+      return 0;
+  for (i = MFACE_VIDEOMODE; i <= MFACE_RATIO; i++)
+    if (face1->property[i] != face2->property[i])
+      return 0;
+  hline1 = (MFaceHLineProp *) face1->property[MFACE_HLINE];
+  hline2 = (MFaceHLineProp *) face2->property[MFACE_HLINE];
+  if (hline1 != hline2)
+    {
+      if (! hline1 || ! hline2)
+	return 0;
+      if (memcmp (hline1, hline2, sizeof (MFaceHLineProp)) != 0)
+	return 0;
+    }
+  box1 = (MFaceBoxProp *) face1->property[MFACE_BOX];
+  box2 = (MFaceBoxProp *) face2->property[MFACE_BOX];
+  if (box1 != box2)
+    {
+      if (! box1 || ! box2)
+	return 0;
+      if (memcmp (box1, box2, sizeof (MFaceBoxProp)) != 0)
+	return 0;
+    }
+  return 1;
+}
+
+
 /*=*/
 /***en
     @brief Merge faces.
@@ -1576,8 +1774,8 @@ mface_merge (MFace *dst, MFace *src)
 /***ja
     @brief եȤե.
 
-    ؿ mface_from_font () ϥե $FONT Υץѥƥץѥƥ
-    ȤƻĿեꡢ֤  */
+    ؿ mface_from_font () ϥե $FONT 
+    ΥץѥƥץѥƥȤƻĿեꡢ֤  */
 
 MFace *
 mface_from_font (MFont *font)
@@ -1616,17 +1814,17 @@ mface_from_font (MFont *font)
 /***ja
     @brief եΥץѥƥͤ.
 
-    ؿ mface_get_prop () ϡե $FACE ĥեץѥƥ
-    ⡢ $KEY ǤΤ֤ͤ$KEY ϲΤ줫Ǥ
-    Фʤʤ
+    ؿ mface_get_prop () ϡե $FACE 
+    ĥեץѥƥ⡢ $KEY ǤΤ֤ͤ
+    $KEY ϲΤ줫ǤʤФʤʤ
 
         #Mforeground, #Mbackground, #Mvideomode, #Mhline, #Mbox,
         #Mfoundry, #Mfamily, #Mweight, #Mstyle, #Mstretch, #Madstyle,
         #Msize, #Mfontset, #Mratio, #Mhook_func, #Mhook_arg
 
     @return 
-    ͤη $KEY ˰¸롣嵭Υ򻲾Ȥ뤳
-    ȡ顼Ф줿 @c NULL ֤ѿ #merror_code 
+    ͤη $KEY ˰¸롣嵭Υ򻲾Ȥ뤳ȡ
+    顼Ф줿 @c NULL ֤ѿ #merror_code 
     ˥顼ɤꤹ롣  */
 
 /***
@@ -1673,17 +1871,16 @@ mface_get_prop (MFace *face, MSymbol key)
 /***ja
     @brief եץѥƥͤꤹ.
 
-    ؿ mface_put_prop () ϡե $FACE ǥ $KEY Ǥ
-    ѥƥͤ $VAL ꤹ롣$KEY ϰʲΤ줫ǤʤƤϤ
-    ʤ
+    ؿ mface_put_prop () ϡե $FACE ǥ $KEY 
+    Ǥץѥƥͤ $VAL ꤹ롣$KEY 
+    ϰʲΤ줫ǤʤƤϤʤʤ
 
         #Mforeground, #Mbackground, #Mvideomode, #Mhline, #Mbox,
         #Mfoundry, #Mfamily, #Mweight, #Mstyle, #Mstretch, #Madstyle,
         #Msize, #Mfontset, #Mratio, #Mhook_func, #Mhook_arg.
 
     ΤΡեȴϢΥץѥƥ (#Mfamily  #Msize 
-    ޤ) ϡեΥեȥåΥեȤ˴ؤǥե
-    ȤʤꡢġΥեȤͤꤷʤäѤ롣
+    ޤ) ϡեΥեȥåΥեȤ˴ؤǥեͤȤʤꡢġΥեȤͤꤷʤäѤ롣
 
     ͤη $KEY ˰¸롣嵭Υ򻲾Ȥ뤳ȡ
 
@@ -1787,8 +1984,8 @@ mface_update (MFrame *frame, MFace *face)
 /***ja
     @brief եפ.
 
-    ؿ mdebug_dump_face () ϥե $FACE  stderr ˿ʹ֤˲ɤ
-    ǰ롣 $INDENT ϣܰʹߤΥǥȤꤹ롣
+    ؿ mdebug_dump_face () ϥե $FACE  stderr 
+    ˿ʹ֤˲ɤʷǰ롣 $INDENT ϣܰʹߤΥǥȤꤹ롣
 
     @return
     δؿ $FACE ֤  */
