@@ -25,13 +25,19 @@
 
 extern MRealizedFontset *mfont__realize_fontset (MFrame *frame,
 						 MFontset *fontset,
-						 MFace *face);
+						 MFace *face, MFont *spec);
 
 void mfont__free_realized_fontset (MRealizedFontset *realized);
 
 extern MRealizedFont *mfont__lookup_fontset (MRealizedFontset *realized,
 					     MGlyph *g, int *num,
 					     MSymbol script, MSymbol language,
-					     MSymbol charset, int size);
+					     MSymbol charset, int size,
+					     int ignore_fallback);
+
+extern MRealizedFont *mfontset__get_font (MFrame *frame, MFontset *fontset,
+					  MSymbol script, MSymbol language,
+					  MFont *font, int *best);
+
 
 #endif /* _M17N_FONTSET_H_ */
