@@ -39,17 +39,16 @@
     @addtogroup m17nInputMethodWin
     @brief ɥƥϥ᥽åɤΥݡ.
 
-    ϥɥ饤 @c minput_gui_driver ϡɥƥѤ
-    ϥ᥽åѤǤ롣Υɥ饤ФϥݥåȤ preedit 
-    ƥȤ status ƥȤɽ롣ܺ٤ˤĤƤ @c
-    minput_gui_driver 򻲾ȤΤȡ
+    ϥɥ饤 @c minput_gui_driver ϡ
+    ɥƥѤϥ᥽åѤΥɥ饤ФǤ롣
+    Υɥ饤ФϥݥåȤ preedit ƥȤ status 
+    ƥȤɽ롣ܺ٤ˤĤƤ @c minput_gui_driver 򻲾ȤΤȡ
 
-    m17n-X 饤֥ϡ@c Mxim ȸ̾ĳϥ᥽åɤ
-    Ƥ롣 XIM (X Input Method) Хå饦ɤϥ
-    󥸥ȤѤ롣ܥ @c Mxim  @c Minput_driver Ȥ
-    ץѥƥäƤꡢͤϥɥ饤 @c minput_xim_driver 
-    ؤΥݥ󥿤Ǥ롣 ܺ٤ˤĤƤ @c minput_xim_driver 
-    ȤΤȡ  */
+    m17n-X 饤֥ϡ@c Mxim ȸ̾ĳϥ᥽åɤ󶡤Ƥ롣 
+    XIM (X Input Method) Хå饦ɤϥ󥸥ȤѤ롣ܥ
+    @c Mxim  @c Minput_driver ȤץѥƥäƤꡢͤϥɥ饤 
+    @c minput_xim_driver ؤΥݥ󥿤Ǥ롣 ܺ٤ˤĤƤ 
+    @c minput_xim_driver 򻲾ȤΤȡ  */
 
 /*=*/
 
@@ -159,10 +158,8 @@ win_filter (MInputContext *ic, MSymbol key, void *arg)
       || ! ic->active)
     return 0;
 
-  if (key == Mnil)
+  if (key == Mnil && arg)
     {
-      if (! arg)
-	return 0;
       key = minput_event_to_key (win_ic_info->frame, arg);
       if (key == Mnil)
 	return 1;
@@ -549,7 +546,10 @@ minput__win_init ()
     plist = mplist_add (plist, Minput_toggle, (void *) win_callback);
     plist = mplist_add (plist, Minput_reset, (void *) win_callback);
   }
+#if 0
+  /* This will make a caller of minput_method_open() pazzled.  */
   minput_driver = &minput_gui_driver;
+#endif
 
   face_box_prop.width = 1;
   face_box_prop.color_top = face_box_prop.color_left
@@ -607,9 +607,9 @@ minput__win_fini ()
     pointer to the structure @c MInputGUIArgIC.  See the documentation
     of @c MInputGUIArgIC for more details.
 
-    If the argument $KEY is @c Mnil, the argument $ARG of the
-    function minput_filter () must be a pointer to the object of type
-    @c XEvent.  In that case, $KEY is generated from $ARG.
+    If the argument $KEY of function minput_filter () is @c Mnil, the
+    argument $ARG must be a pointer to the object of type @c XEvent.
+    In that case, $KEY is generated from $ARG.
 
     The argument $ARG of the function minput_lookup () must be the
     same one as that of the function minput_filter (). */
@@ -617,25 +617,23 @@ minput__win_fini ()
 /***ja
     @brief ɥƥϥ᥽åϥɥ饤.
 
-    ϥɥ饤 @c minput_gui_driver ϡɥƥѤ
-    ϥ᥽åѤǤ롣
+    ϥɥ饤 @c minput_gui_driver
+    ϡɥƥѤϥ᥽åѥɥ饤ФǤ롣
 
-    Υɥ饤Фϡؿ minput_set_spot () ˤäꤵ줿ϥݥ
-    Ȥ preedit ƥѤΥ֥ɥ status ƥѤΥ
-    ɥꡢ줾ɽ롣
+    Υɥ饤Фϡؿ minput_set_spot () ˤäꤵ줿ϥݥåȤ
+    preedit ƥѤΥ֥ɥ status 
+    ƥѤΥ֥ɥꡢ줾ɽ롣
 
-    ޥ M17N_INIT () ѿ @c minput_driver 򤳤Υɥ饤
-    ؤΥݥ󥿤ꤷƤϥ᥽åɤΥɥ饤ФȤ
-    ˤ롣
+    ޥ M17N_INIT () ѿ @c minput_driver 
+    򤳤Υɥ饤ФؤΥݥ󥿤ꤷƤϥ᥽åɤΥɥ饤ФȤ褦ˤ롣
 
     äơ@c minput_driver ǥեͤΤޤޤǤСminput_ 
-    ǻϤޤ̾ĴؿΰΤɥ饤а¸ΤΤϰʲΤ褦
-    ʤ롣
+    ǻϤޤ̾ĴؿΰΤɥ饤а¸ΤΤϰʲΤ褦ˤʤ롣
 
     ؿ minput_open_im () ΰ $ARG ̵뤵롣
 
     ؿ minput_create_ic () ΰ $ARG Ϲ¤ @c MInputGUIArgIC 
-    Υݥ󥿤ǤʤƤϤʤʤܺ٤ˤĤƤ @c MInputGUIArgIC 
+    ؤΥݥ󥿤ǤʤƤϤʤʤܺ٤ˤĤƤ @c MInputGUIArgIC 
     򻲾ȤΤȡ
 
     ؿ minput_filter () ΰ $ARG  @c Mnil ξ硢 $ARG  @c
@@ -657,8 +655,8 @@ MInputDriver minput_gui_driver;
 /***ja
     @brief "xim"̾Ȥƻĥܥ .
 
-    ѿ Mxim "xim"̾ȤƻĥܥǤ롣"xim" ϥ᥽
-    ɥɥ饤 #minput_xim_driver ̾Ǥ롣  */ 
+    ѿ Mxim "xim"̾ȤƻĥܥǤ롣"xim" 
+    ϥ᥽åɥɥ饤 #minput_xim_driver ̾Ǥ롣  */ 
 
 MSymbol Mxim;
 
@@ -685,9 +683,9 @@ MSymbol Mxim;
     the byte is bitwise anded by 0x1F and the Control modifier is
     cleared.
 
-    If $EVENT still has Shift, Control, Meta, Alt, Super, and/or Hyper
-    modifiers, the name is preceded by "S-", "C-", "M-", "A-", "s-",
-    and/or "H-" respectively in this order.
+    If $EVENT still has modifiers, the name is preceded by "H-"
+    (Hyper), "s-" (Super), "A-" (Alt), "M-" (Meta), "C-" (Control),
+    and/or "S-" (Shift) in this order.
 
     For instance, if the keysym name is "a" and the event has Shift,
     Meta, and Hyper modifiers, the resulting name is "H-M-A".
@@ -697,29 +695,28 @@ MSymbol Mxim;
 /***ja
     @brief ٥ȤϥѴ.
 
-    ؿ minput_name_to_key () ϡ$FRAME Υ٥ $EVENT б
-    ϥ֤ǤΡбפϥɥƥ¸Ǥ롣
+    ؿ minput_event_to_key () ϡ$FRAME Υ٥ $EVENT 
+    бϥ֤ǤΡбפϥɥƥ˰¸롣
 
-    m17n-X 饤֥ξˤϡ$EVENT  ¤ @c XKeyEvent ؤΥ
-    󥿤ǤꡢΤ褦˽롣
+    m17n-X 饤֥ξˤϡ$EVENT  ¤ @c XKeyEvent 
+    ؤΥݥ󥿤ǤꡢΤ褦˽롣
 
-    ޤؿ @c XKeysymToString ˤäơ$EVENT  keysym ̾
-    ǰʲѹä롣
+    ޤؿ @c XKeysymToString ˤäơ$EVENT  keysym 
+    ̾ǰʲѹä롣
 
-    ̾ "a" .. "z" Τ줫Ǥä $EVENT  Shift ǥե
-    С̾Ϥ줾 "A" .. "Z" Ѵ졢Shift ǥե
-    ϼ롣
+    ̾ "a" .. "z" Τ줫Ǥä $EVENT  Shift 
+    ǥեС̾Ϥ줾 "A" .. "Z" Ѵ졢Shift 
+    ǥեϼ롣
 
     ̾ХĹ $EVENT  Control ǥեС̾
-     0x1F ӥåñ and 黻롣Control ǥեϼ
-    롣
+    0x1F Ȥӥåñ̤ and 黻Control ǥեϼ롣
 
-    $EVENT ˤޤ Shift, Control, Meta, Alt, Super, Hyper ʤɤΥǥ
-    եС̾ˤ줾"S-", "C-", "M-", "A-", "s-",
-    "H-" դ
+    Ǥ $EVENT ˤޤǥեС̾ˤ줾
+    "H-" (Hyper), "s-" (Super), "A-" (Alt), "M-" (Meta), "C-"
+    (Control), "S-" (Shift) ν֤դ
     
-    ȤСkeysym ̾ "a" ǥ٥Ȥ Shift, Meta, and Hyper ǥ
-    եƤС̾ "H-M-A" Ǥ롣
+    ȤСkeysym ̾ "a" ǥ٥Ȥ Shift, Meta, and Hyper 
+    ǥեƤС̾ "H-M-A" Ǥ롣
 
     Ǹˤ̾ĥܥ֤*/
 
