@@ -25,7 +25,9 @@
 
 typedef struct
 {
+  MInputMethod *im;
   MText *title;
+  MPlist *maps;
   MPlist *states;
   MPlist *macros;
   MPlist *externals;
@@ -40,6 +42,9 @@ typedef struct
   /** The current state.  */
   MIMState *state;
 
+  /** The previous state.  */
+  MIMState *prev_state;
+
   /** The current map.  */
   MIMMap *map;
 
@@ -65,6 +70,8 @@ typedef struct
   /* List of variables. */
   MPlist *vars;
 
+  MText *preceding_text, *following_text;
+
   int key_unhandled;
 
   /** Used by minput_win_driver (input-win.c).  */
