@@ -54,6 +54,8 @@ struct MFrame
       character of the default face.  */
   int space_width;
 
+  int average_width;
+
   /** The default ascent and descent of a line.  It is ascent and
       descent of ASCII font of the default face.  */
   int ascent, descent;
@@ -72,6 +74,9 @@ struct MFrame
   /** Logical OR of enum MDeviceType.  */
   int device_type;
 
+  /** Resolution (dots per inch) of the device.  */
+  int dpi;
+
   /** Correction of functions to manipulate the device.  */
   MDeviceDriver *driver;
 
@@ -223,8 +228,6 @@ struct MGlyphString
 
 typedef struct MGlyphString MGlyphString;
 
-typedef struct MFontDriver MFontDriver;
-
 typedef struct
 {
   short x, y;
