@@ -31,8 +31,7 @@
     @addtogroup m17nLocale
     @brief 륪֥ȤȤ˴ؤ API.
 
-    m17n 饤֥ϥϢ #MLocale Υ֥Ȥ
-    ɽ롣  */
+    m17n 饤֥ϥϢ #MLocale Υ֥Ȥɽ롣  */
 
 /*=*/
 
@@ -357,13 +356,11 @@ MSymbol Mcodeset;
 /***ja
     @brief ߤΥꤹ.
 
-    ؿ mlocale_set () ϸߤΥΰꤷ䤤碌
-    ꤹ롣ǰȤ $CATEGORY ǻꤵ졢<tt>setlocale ()</tt> 
-    ͭȤʤΤǤʤƤϤʤʤ
+    ؿ mlocale_set () ϸߤΥΰꤷ䤤碌ꤹ롣ǰȤ 
+    $CATEGORY ǻꤵ졢<tt>setlocale ()</tt> ͭȤʤΤǤʤƤϤʤʤ
 
-    $LOCALE  NULL ǤʤСꤷʬΥ뤬$LOCALE 
-    롣 $LOCALE ƥ˥ݡȤƤʤСϹԤ
-    줺ߤΥѤʤ
+    $LOCALE  NULL ǤʤСꤷʬΥ뤬$LOCALE ꤵ롣
+    $LOCALE ƥ˥ݡȤƤʤСϹԤ줺ߤΥѤʤ
 
     $LOCALE  NULL ʤСߤΥλꤷʬ䤤碌롣
 
@@ -434,9 +431,9 @@ mlocale_set (int category, const char *name)
 /***ja
     @brief ץѥƥͤ.
 
-    ؿ mlocale_get_prop () ϡ $LOCALE  $KEY ץѥƥ
-    ֤ͤ $KEY  #Mname, #Mlanguage, #Mterritory, #Mcodeset,
-    #Mmodifier, #Mcoding Τ줫Ǥ롣 */ 
+    ؿ mlocale_get_prop () ϡ $LOCALE  $KEY ץѥƥ֤ͤ
+    $KEY  #Mname, #Mlanguage, #Mterritory, #Mcodeset, #Mmodifier, 
+    #Mcoding Τ줫Ǥ롣 */ 
 
 MSymbol
 mlocale_get_prop (MLocale *locale, MSymbol key)
@@ -475,9 +472,8 @@ mlocale_get_prop (MLocale *locale, MSymbol key)
     @brief դȻ֤եޥåȤ.
 
     ؿ mtext_ftime () ϻǡ (broken-down time) $TM $FORMAT 
-    ǻꤵ줿񤷡̤M-text $MT ղä롣եޥ
-    Ȥ NULL Ǥʤ  $LOCALE ˡޤϸߤΥ 
-    (LC_TIME) ˽
+    ǻꤵ줿񤷡̤M-text $MT ղä롣եޥåȤ 
+    NULL Ǥʤ  $LOCALE ˡޤϸߤΥ(LC_TIME) ˽
 
      $TM  $FORMAT ΰ̣ strftime () ξƱ
 
@@ -554,8 +550,8 @@ mtext_ftime (MText *mt, const char *format, const struct tm *tm,
 /***ja
     @brief Ķѿ.
 
-    ؿ mtext_getenv ()  $NAME ǻؤʸȹפʸ
-    ĶѿΥꥹ椫õ
+    ؿ mtext_getenv ()  $NAME 
+    ǻؤʸȹפʸĶѿΥꥹ椫õ
 
     Ĥäˤϡͤ򸽺ߤΥ(LC_CTYPE) ˽ä 
     M-text ˥ǥɤM-text ֤
@@ -588,9 +584,9 @@ mtext_getenv (const char *name)
 /***ja
     @brief Ķѿѹɲä.
 
-    ؿ mtext_putenv ()  M-text $MT ˽äơĶѿͤѹ
-    ɲäꤹ롣δؿϡߤΥ(LC_CTYPE) ˽ä 
-    $MT 򥨥󥳡ɤȤƴؿ <tt>putenv</tt> Ƥ֡
+    ؿ mtext_putenv ()  M-text $MT 
+    ˽äơĶѿͤѹɲäꤹ롣δؿϡߤΥ
+    (LC_CTYPE) ˽ä$MT 򥨥󥳡ɤȤƴؿ <tt>putenv</tt> Ƥ֡
 
     @return
     δؿϡˤ 0 򡢥顼 -1 ֤
@@ -630,14 +626,13 @@ mtext_putenv (MText *mt)
 /***ja
     @brief ߤΥѤƣĤ M-text Ӥ.
 
-    ؿ mtext_coll () ϣĤ M-text $MT1  $MT2 Ӥ롣
-    , 0, ͤΤ줫Ǥꡢ줾$MT1  $MT2 
-    꾮Ʊ礭롣ӤϸߤΥ 
+    ؿ mtext_coll () ϣĤ M-text $MT1  $MT2 
+    Ӥ롣ͤ, 0, ͤΤ줫Ǥꡢ줾 
+    $MT1  $MT2 꾮Ʊ礭롣ӤϸߤΥ 
     (LC_COLLATE) ˴ŤƹԤ롣
 
-    δؿ M-text ΥƥȥץѥƥȤƼưŪ˥å夵
-    ѤΤǡܰʹߤƱӤϣܤ®¹Ԥ
-    롣  */
+    δؿ M-text 
+    ΥƥȥץѥƥȤƼưŪ˥å夵ѤΤǡܰʹߤƱӤϣܤ®¹Ԥ롣  */
 
 int
 mtext_coll (MText *mt1, MText *mt2)
