@@ -49,38 +49,36 @@ extern MSymbol Mxim;
     @brief ؿ minput_open_im () ΰ $ARG ˤäƻؤ빽¤.
 
     #MInputXIMArgIM ϡؿ minput_open_im () ̾ #Mxim 
-    ĳϥ᥽åɤݤ˰ $ARG ˤäƻؤ빽¤Τ
-    롣  */
+    ĳϥ᥽åɤݤ˰ $ARG ˤäƻؤ빽¤ΤǤ롣  */
 
 typedef struct
 {
   /***en The meaning of the following four members are the same as
       arguments to XOpenIM ().  */
-  /***ja ʲΣĤΥФΰ̣ϡXOpenIM () ΰΰ̣ƱǤ
-      롣  */
+  /***ja ʲΣĤΥФΰ̣ϡXOpenIM () ΰΰ̣ƱǤ.  */
 
   /***en Display of the client.  */
-  /***ja 饤ȤΥǥץ쥤  */
+  /***ja 饤ȤΥǥץ쥤.  */
   Display *display;
 
   /***en Pointer to the X resource database.  */
-  /***ja X ꥽ǡ١ؤΥݥ  */
+  /***ja X ꥽ǡ١ؤΥݥ.  */
   XrmDatabase db;
 
   /***en Full class name of the application.  */
-  /***ja ץꥱδʥ饹̾  */
+  /***ja ץꥱδʥ饹̾.  */
   char *res_class;
 
   /***en Full resource name of the application.  */
-  /***ja ץꥱδʥ꥽̾  */
+  /***ja ץꥱδʥ꥽̾.  */
   char *res_name;
 
   /***en Locale name under which an XIM is opened.  */
-  /***ja XIMץ󤵤줿̾  */
+  /***ja XIMץ󤵤줿̾.  */
   char *locale;
 
   /***en Arguments to XSetLocaleModifiers ().  */
-  /***ja XSetLocaleModifiers () ΰ  */
+  /***ja XSetLocaleModifiers () ΰ.  */
   char *modifier_list;
 } MInputXIMArgIM;
 
@@ -98,8 +96,7 @@ typedef struct
     @brief ؿ minput_create_ic () ΰ $ARG ˤäƻؤ빽¤.
 
     #MInputXIMArgIC ϡؿ minput_create_ic () ̾ #Mxim 
-    ĳϥ᥽åѤ˸ƤФݤˡ $ARG ˤäƻؤ빽
-    ¤ΤǤ롣 */
+    ĳϥ᥽åѤ˸ƤФݤˡ $ARG ˤäƻؤ빽¤ΤǤ롣 */
 
 typedef struct
 {
@@ -109,14 +106,13 @@ typedef struct
       <status_attrs> are set to @c NULL.  */
   /***ja @c XCreateIC  @c XNInputStyle ³ȤѤ롣
       ʤС ( @c XIMPreeditNothing | @c XIMStatusNothing) 
-      졢 <preedit_attrs>  <status_attrs>  @c NULL ꤵ
-      롣 */
+      졢 <preedit_attrs>  <status_attrs>  @c NULL 
+      ꤵ롣 */
 
   XIMStyle input_style;
   /***en Used as the argument of @c XCreateIC following @c XNClientWindow.  */
   /***ja @c XCreateIC  @c XNClientWindow ³ȤѤ롣  */
 
-
   Window client_win;
   /***en Used as the argument of @c XCreateIC following @c XNFocusWindow.  */
   /***ja @c XCreateIC  @c XNFocusWindow ³ȤѤ롣  */
@@ -124,13 +120,13 @@ typedef struct
   Window focus_win;
   /***en If non- @c NULL, used as the argument of @c XCreateIC following
       @c XNPreeditAttributes.  */
-  /***ja @c NULLǤʤС @c XCreateIC following @c
+  /***ja @c NULL ǤʤС @c XCreateIC following @c
       XNPreeditAttributes ³ȤѤ롣  */
 
   XVaNestedList preedit_attrs;
   /***en If non- @c NULL, used as the argument of @c XCreateIC following
       @c XNStatusAttributes.  */ 
-  /***ja @c NULLǤʤС @c XCreateIC following  @c
+  /***ja @c NULL ǤʤС @c XCreateIC following  @c
       XNStatusAttributes ³ȤѤ롣  */
 
   XVaNestedList status_attrs;
