@@ -55,7 +55,7 @@
     supported as a graphic device.
 
     On a frame of the null device, you can't draw text nor use an
-    input method (but, for instance, the funciton mdraw_glyph_list ()
+    input method (but, for instance, the function mdraw_glyph_list ()
     is available).
 
     On a frame of the X Window System, you can use the whole GUI API.
@@ -149,36 +149,31 @@
     <li>  API
 
     M-text 򰷤δŪʥ⥸塼󶡤롣ʬ API 
-    m17n ǡ١ɬפȤʤѤ뤿ˤϡץꥱ
-    ץ<m17n-core<EM></EM>.h>  include  -lm17n-core ǥ
-    󥯤ʤƤϤʤʤ
+    m17n ǡ١ɬפȤʤѤ뤿ˤϡץꥱץ
+    <m17n-core<EM></EM>.h>  include  -lm17n-core ǥ󥯤ʤƤϤʤʤ
 
     <li>  API
 
     m17n ǡ١Ѥ⥸塼ʥѴʸץѥƥ
-    ˤ󶡤롣⥸塼ϥǡ١ɬפ˱¿ͤʥǡ
-    ɤ롣Ѥ뤿ˤϡץꥱץ
-    <m17n<EM></EM>.h>  include  -lm17n-core -lm17n ǥ󥯤
-    ƤϤʤʤ
+    󶡤롣⥸塼ϥǡ١ɬפ˱¿ͤʥǡɤ롣
+    Ѥ뤿ˤϡץꥱץ <m17n<EM></EM>.h>  include
+     -lm17n-core -lm17n ǥ󥯤ʤƤϤʤʤ
 
      API Ѥݤˤϥ API ѤǤ롣
 
     <li> GUI API
 
-    M-texts 򥰥եåǥХɽϤꤹȤä
-    եå桼󥿥ե⥸塼󶡤롣API Τϥ
-    եåǥХȤΩǤ뤬¿δؿΥեå
-    ǥХ˺줿 MFrame ˼롣Ǥϡ̥ǥХ
-    X ɥƥࡢGD 饤֥Υ᡼gdImagePtrˡ
-    եåǥХȤƥݡȤƤ롣
+    M-text 򥰥եåǥХɽϤꤹȤäեå桼󥿥ե⥸塼󶡤롣
+    API ΤϥեåǥХȤΩǤ뤬¿δؿΥեåǥХ˺줿 
+    MFrame ˼롣Ǥϡ̥ǥХX ɥƥࡢ
+    GD 饤֥Υ᡼gdImagePtrˡեåǥХȤƥݡȤƤ롣
 
     ̥ǥХǤɽϤǤʤʤȤ
     mdraw_glyph_list () ʤɤδؿϻѲǽˡ
 
-    X ɥƥǤϤ٤Ƥ GUI API Ȥ롣
+    X ɥƥǤϤ٤Ƥ GUI API ѤǤ롣
 
-    GD 饤֥Υ᡼ǤϡѤ API Ϥ٤ƻȤ뤬Ϥ
-    Ǥʤ
+    GD 饤֥Υ᡼ǤϡѤ API Ϥ٤ƻѤǤ뤬ϤϤǤʤ
 
     ΥǥХѤ뤿ˤϡץꥱץ
     <m17n-gui<EM></EM>.h>  include -lm17n-core -lm17n -lm17n-gui
@@ -188,10 +183,9 @@
 
     <li> ¾ API
 
-    顼ǥХåѤΤ¾δؿ󶡤롣 API Ϥ
-    ǤϻѤǤ嵭¾ΤΤȶ˻ȤѤ뤿ˤϡ嵭
-    줫 include ե˲äơ <m17n-misc<EM></EM>.h> 
-    include ʤƤϤʤʤ
+    顼ǥХåѤΤ¾δؿ󶡤롣 API 
+    ϤǤϻѤǤ嵭¾ΤΤȶ˻ȤѤ뤿ˤϡ嵭Τ줫
+    include ե˲äơ <m17n-misc<EM></EM>.h>  include ʤƤϤʤʤ
 
     </ol>
 
@@ -204,22 +198,21 @@
     <ul>
     <li> @c M17NDIR
 
-    m17n ǡ١Υǡޤǥ쥯ȥ̾ܺ٤@ref
+    m17n ǡ١Υǡޤǥ쥯ȥ̾ܺ٤ @ref
     m17nDatabase ȡ
 
     <li> @c MDEBUG_XXXX
 
-    "MDEBUG_" ǻϤޤ̾ĴĶѿϥǥХå椹롣ܺ
-    @ref m17nDebug ȡ
+    "MDEBUG_" ǻϤޤ̾ĴĶѿϥǥХå椹롣ܺ٤ @ref m17nDebug ȡ
 
     </ul>
 
     @em API @em ̿̾§ ޥ
 
     饤֥ϡؿѿޥ export 롣'm' 
-     'M' ΤȤ˥֥̾("symbol"  "plist" ʤɡ 
+    ޤ 'M' ΤȤ˥֥̾("symbol"  "plist" ʤɡ 
     "mtext" ֥ȤϤϤ 'm' νʣ򤱤뤿 "text" 
-    롣) ޤϥ⥸塼̾(draw, input ʤ) ³ΤǤ롣
+    Ѥ롣) ޤϥ⥸塼̾(draw, input ʤ) ³ΤǤ롣
     
     <ul>
 
@@ -255,10 +248,9 @@
 /***en
     The #M17NLIB_MAJOR_VERSION macro gives the major version number
     of the m17n library.  */
-
 /***ja
-    ޥ #M17NLIB_MAJOR_VERSION  m17n 饤֥Υ᥸㡼С
-    ֹͿ롣  */
+    ޥ #M17NLIB_MAJOR_VERSION  m17n 
+    饤֥Υ᥸㡼СֹͿ롣  */
 
 #define M17NLIB_MAJOR_VERSION
 
@@ -269,8 +261,8 @@
     of the m17n library.  */
 
 /***ja
-    ޥ #M17NLIB_MINOR_VERSION  m17n 饤֥ΥޥʡС
-    ֹͿ롣  */
+    ޥ #M17NLIB_MINOR_VERSION  m17n 
+    饤֥ΥޥʡСֹͿ롣  */
 
 #define M17NLIB_MINOR_VERSION
 
@@ -281,8 +273,8 @@
     of the m17n library.  */
 
 /***ja
-    ޥ #M17NLIB_PATCH_LEVEL  m17n 饤֥Υѥå٥ֹ
-    Ϳ롣  */
+    ޥ #M17NLIB_PATCH_LEVEL  m17n 
+    饤֥Υѥå٥ֹͿ롣  */
 
 #define M17NLIB_PATCH_LEVEL
 
@@ -293,8 +285,8 @@
     m17n library as a string.  */
 
 /***ja
-    ޥ #M17NLIB_VERSION_NAME  m17n 饤֥ΥС̾
-    ʸȤͿ롣  */
+    ޥ #M17NLIB_VERSION_NAME  m17n 
+    饤֥ΥС̾ʸȤͿ롣  */
 
 #define M17NLIB_VERSION_NAME
 
@@ -319,11 +311,11 @@
 /***ja
     @brief m17n 饤֥.
 
-    ޥ M17N_INIT ()  m17n 饤֥롣m17n δؿ
-    ѤˡΥޥޤƤФʤƤϤʤʤ
+    ޥ M17N_INIT ()  m17n 饤֥롣m17n 
+    δؿѤˡΥޥޤƤФʤƤϤʤʤ
     
-    ΥޥʣƤǤǤ뤬ξ뤿
-    ˥ޥ M17N_FINI () ƱƤɬפ롣
+    ΥޥʣƤǤǤ뤬ξ뤿˥ޥ
+    M17N_FINI () ƱƤɬפ롣
 
     ѿ #merror_code ϡ 0 ˡǤʤ 
     -1 ꤵ롣  
@@ -351,9 +343,9 @@
 /***ja
     @brief m17n 饤֥λ. 
 
-    ޥ M17N_FINI ()  m17n 饤֥λ롣m17n 饤֥
-    ȤäƤΥΰϲ롣٤ΥޥƤФ줿顢
-    ޥ M17N_INIT () ٸƤФޤ m17n ؿϻȤ٤Ǥʤ
+    ޥ M17N_FINI ()  m17n 饤֥λ롣m17n 
+    饤֥꤬ȤäƤΥΰϲ롣٤ΥޥƤФ줿顢ޥ
+    M17N_INIT () ٸƤФޤ m17n ؿϻȤ٤Ǥʤ
 
     ޥ M17N_INIT ()  N ƤФƤˤϡΥޥ N 
     ƤФƽƥ꤬롣
@@ -381,6 +373,7 @@
 #include "m17n-core.h"
 #include "m17n-misc.h"
 #include "internal.h"
+#include "symbol.h"
 
 static void
 default_error_handler (enum MErrorCode err)
@@ -391,7 +384,29 @@ default_error_handler (enum MErrorCode err)
 static struct timeval time_stack[16];
 static int time_stack_index;
 
-static int report_header_printed;
+static M17NObjectArray *object_array_root;
+
+static void
+report_object_array ()
+{
+  fprintf (stderr, "%16s %7s %7s %7s\n",
+	   "object", "created", "freed", "alive");
+  fprintf (stderr, "%16s %7s %7s %7s\n",
+	   "------", "-------", "-----", "-----");
+  for (; object_array_root; object_array_root = object_array_root->next)
+    {
+      M17NObjectArray *array = object_array_root;
+
+      fprintf (stderr, "%16s %7d %7d %7d\n", array->name,
+	       array->used, array->used - array->count, array->count);
+      if (array->used > 0)
+	{
+	  free (array->objects);
+	  array->count = array->used = 0;
+	}
+    }
+}
+
 
 
 /* Internal API */
@@ -400,34 +415,12 @@ int m17n__core_initialized;
 int m17n__shell_initialized;
 int m17n__gui_initialized;
 
-void
-mdebug__report_object (char *name, M17NObjectArray *array)
-{
-  if (! (mdebug__flag & MDEBUG_FINI))
-    return;
-  if (! report_header_printed)
-    {
-      fprintf (stderr, "%16s %7s %7s %7s\n",
-	       "object", "created", "freed", "alive");
-      fprintf (stderr, "%16s %7s %7s %7s\n",
-	       "------", "-------", "-----", "-----");
-      report_header_printed = 1;
-    }
-  fprintf (stderr, "%16s %7d %7d %7d\n", name,
-	   array->used, array->used - array->count, array->count);
-  if (array->used > 0)
-    {
-      free (array->objects);
-      array->count = array->used = 0;
-    }
-}
-
-
 void *(*mdatabase__finder) (MSymbol tag1, MSymbol tag2,
 			   MSymbol tag3, MSymbol tag4);
 void *(*mdatabase__loader) (void *);
 
 int mdebug__flag;
+FILE *mdebug__output;
 
 void
 mdebug__push_time ()
@@ -461,11 +454,26 @@ mdebug__print_time ()
   do {						\
     char *env_value = getenv (env_name);	\
 						\
-    if (env_value && env_value[0] == '1')	\
-      mdebug__flag |= (mask);			\
+    if (env_value)				\
+      {						\
+	if (env_value[0] == '1')		\
+	  mdebug__flag |= (mask);		\
+	else if (env_value[0] == '0')		\
+	  mdebug__flag &= ~(mask);		\
+      }						\
   } while (0)
 
 
+void
+mdebug__add_object_array (M17NObjectArray *array, char *name)
+{
+  array->name = name;
+  array->count = 0;
+  array->next = object_array_root;
+  object_array_root = array;
+}
+
+
 void
 mdebug__register_object (M17NObjectArray *array, void *object)
 {
@@ -504,13 +512,14 @@ m17n_init_core (void)
 {
   int mdebug_mask = MDEBUG_INIT;
 
+  merror_code = MERROR_NONE;
   if (m17n__core_initialized++)
     return;
 
-  merror_code = MERROR_NONE;
   m17n_memory_full_handler = default_error_handler;
 
   mdebug__flag = 0;
+  SET_DEBUG_FLAG ("MDEBUG_ALL", MDEBUG_ALL);
   SET_DEBUG_FLAG ("MDEBUG_INIT", MDEBUG_INIT);
   SET_DEBUG_FLAG ("MDEBUG_FINI", MDEBUG_FINI);
   SET_DEBUG_FLAG ("MDEBUG_CHARSET", MDEBUG_CHARSET);
@@ -520,6 +529,15 @@ m17n_init_core (void)
   SET_DEBUG_FLAG ("MDEBUG_FONT_FLT", MDEBUG_FONT_FLT);
   SET_DEBUG_FLAG ("MDEBUG_FONT_OTF", MDEBUG_FONT_OTF);
   SET_DEBUG_FLAG ("MDEBUG_INPUT", MDEBUG_INPUT);
+  {
+    char *env_value = getenv ("MDEBUG_OUTPUT_FILE");
+
+    mdebug__output = NULL;
+    if (env_value)
+      mdebug__output = fopen (env_value, "a");
+    if (! mdebug__output)
+      mdebug__output = stderr;
+  }
 
   MDEBUG_PUSH_TIME ();
   MDEBUG_PUSH_TIME ();
@@ -529,14 +547,14 @@ m17n_init_core (void)
   if  (mplist__init () < 0)
     goto err;
   MDEBUG_PRINT_TIME ("INIT", (stderr, " to initialize plist module."));
+  if  (mchartable__init () < 0)
+    goto err;
+  MDEBUG_PRINT_TIME ("INIT", (stderr, " to initialize chartable module."));
   if (mtext__init () < 0)
     goto err;
   if (mtext__prop_init () < 0)
     goto err;
   MDEBUG_PRINT_TIME ("INIT", (stderr, " to initialize mtext module."));
-  if  (mchartable__init () < 0)
-    goto err;
-  MDEBUG_PRINT_TIME ("INIT", (stderr, " to initialize chartable module."));
 
   mdatabase__finder = NULL;
   mdatabase__loader = NULL;
@@ -558,20 +576,26 @@ m17n_fini_core (void)
 
   MDEBUG_PUSH_TIME ();
   MDEBUG_PUSH_TIME ();
-  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize chartable module."));
   mchartable__fini ();
-  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize textprop module."));
-  mtext__prop_fini ();
-  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize mtext module."));
+  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize chartable module."));
   mtext__fini ();
-  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize symbol module."));
+  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize mtext module."));
   msymbol__fini ();
-  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize plist module."));
+  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize symbol module."));
   mplist__fini ();
+  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize plist module."));
+  /* We must call this after the aboves because it frees interval
+     pools.  */
+  mtext__prop_fini ();
+  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize textprop module."));
   MDEBUG_POP_TIME ();
   MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize the core modules."));
   MDEBUG_POP_TIME ();
-  report_header_printed = 0;
+  if (mdebug__flag & MDEBUG_FINI)
+    report_object_array ();
+  msymbol__free_table ();
+  if (mdebug__output != stderr)
+    fclose (mdebug__output);
 }
 
 /*** @} */
@@ -595,8 +619,8 @@ m17n_fini_core (void)
 /***ja
     @brief m17n 饤֥Τɤʬ줿𤹤.
 
-    ؿ m17n_status () m17n 饤֥Τɤʬ줿
-    ơʲͤΤ줫֤
+    ؿ m17n_status ()  
+    m17n 饤֥Τɤʬ줿˱ơʲͤΤ줫֤
 
 	#M17N_NOT_INITIALIZED, #M17N_CORE_INITIALIZED,
 	#M17N_SHELL_INITIALIZED, #M17N_GUI_INITIALIZED  */
@@ -627,8 +651,8 @@ m17n_status (void)
     zero.
 
     A property whose key is a managing key can have only a managed
-    object as its value.  Such functions as msymbol_put () and
-    mplist_put () pay special attention to such a property.
+    object as its value.  Some functions, for instance msymbol_put ()
+    and mplist_put (), pay special attention to such a property.
 
     In addition to the predefined managed object types, users can
     define their own managed object types.  See the documentation of
@@ -638,19 +662,16 @@ m17n_status (void)
     @brief ֥ȤȤϻȿˤäƴƤ륪֥ȤǤ.
 
     m17n ֥ȤΤ뷿ΤΤϡȿˤäƴƤ롣
-    Υ֥Ȥ @e ֥ ȸƤФ롣
-    Ǥλȿ 0 ˽Ƥ롣ؿ m17n_object_ref () 
-    ϴ֥Ȥλȿ 1 䤷ؿm17n_object_unref () 
-     1 餹ȿ 0 ˤʤä֥ȤϼưŪ˲
-    롣
+    Υ֥Ȥ @e ֥ ȸƤФ롣줿Ǥλȿ
+    1 ˽Ƥ롣ؿ m17n_object_ref () ϴ֥Ȥλȿ
+    1 䤷ؿm17n_object_unref ()  1 餹ȿ
+    0 ˤʤä֥ȤϼưŪ˲롣
 
-    ǤץѥƥϡͤȤƴ֥Ȥ
-    롣ؿ msymbol_put ()  mplist_put () ʤɤϤΥץѥƥ
-    ̰롣
+    ǤץѥƥϡͤȤƴ֥Ȥ롣
+    ؿ msymbol_put ()  mplist_put () ʤɤϤΥץѥƥ̰롣
 
-    Ѥߴ֥ȥפ¾ˡ桼ɬפʴ֥
-    ȥפʬ뤳ȤǤ롣ܺ٤m17n_object () 
-    򻲾ȡ  */
+    Ѥߴ֥ȥפ¾ˡ桼ɬפʴ֥ȥפʬ뤳ȤǤ롣ܺ٤
+    m17n_object () 򻲾ȡ  */
 
 /*** @{  */
 /*=*/
@@ -676,14 +697,13 @@ m17n_status (void)
 /***ja
     @brief ֥ȤƤ.
 
-    ؿ m17n_object () $SIZE ХȤο֥Ȥ
-    ơλȿ 1 Ȥ롣 $FREER ϻȿ 0 ˤʤäݤˤ
-    ֥Ȥ뤿ѤؿǤ롣$FREER  NULL
-    С֥Ȥϴؿ free () ˤäƲ롣
+    ؿ m17n_object () $SIZE ХȤο֥Ȥơλȿ
+    1 Ȥ롣 $FREER ϻȿ 0 
+    ˤʤäݤˤΥ֥Ȥ뤿ѤؿǤ롣$FREER
+     NULLʤС֥Ȥϴؿ free () ˤäƲ롣
 
-    Ƥ줿֥ƬΥХȤϡ#M17NObjectHead 롣 
-    ΰ m17n 饤֥꤬ѤΤǡץꥱץ
-    ϿƤϤʤʤ
+    Ƥ줿֥ƬΥХȤϡ#M17NObjectHead 
+    롣ΰ m17n 饤֥꤬ѤΤǡץꥱץϿƤϤʤʤ
 
     @return
     δؿϿƤ줿֥Ȥ֤
@@ -726,6 +746,8 @@ m17n_object (int size, void (*freer) (void *))
   M17NObject *obj = malloc (size);
 
   obj->ref_count = 1;
+  obj->ref_count_extended = 0;
+  obj->flag = 0;
   obj->u.freer = freer;
   return obj;
 }
@@ -748,8 +770,8 @@ m17n_object (int size, void (*freer) (void *))
 /***ja
     @brief ֥Ȥλȿ 1 䤹.
 
-    ؿ m17n_object_ref ()  $OBJECT ǻؤ֥Ȥ
-    ȿ 1 䤹
+    ؿ m17n_object_ref ()  $OBJECT 
+    ǻؤ֥Ȥλȿ 1 䤹
 
     @return 
     δؿϡ䤷ȿ 16 ӥåȤ̵(ʤ 
@@ -808,14 +830,13 @@ m17n_object_ref (void *object)
 /***ja
     @brief ֥Ȥλȿ 1 餹.
 
-    ؿ m17n_object_unref ()  $OBJECT ǻؤ֥
-    λȿ 1 餹ȿ 0 ˤʤС֥Ȥϲؿ
-    äƲ롣
+    ؿ m17n_object_unref ()  $OBJECT ǻؤ֥Ȥλȿ
+    1 餹ȿ 0 ˤʤС֥ȤϲؿˤäƲ롣
 
     @return 
     δؿϡ餷ȿ 16 ӥåȤ̵(ʤ 
-    0x10000 ̤)ˤޤС֤Ǥʤ -1 ֤
-    Ĥޤꡢ0 ֤ä褿$OBJECT ϲƤ롣
+    0x10000 ̤)ˤޤС֤Ǥʤ -1 
+    ֤Ĥޤꡢ0 ֤ä褿$OBJECT ϲƤ롣
 
     @errors
     δؿϼԤʤ    */
@@ -884,13 +905,13 @@ m17n_object_unref (void *object)
 
     m17n 饤֥δؿǤϡĤμΥ顼롣
 
-    ĤϰΥ顼Ǥ롣饤֥δؿǤʤȤȤ
-    ƤФ줿硢δؿϥ顼̣֤ͤƱ˳ѿ 
+    ĤϰΥ顼Ǥ롣
+    饤֥δؿǤʤȤȤ˸ƤФ줿硢δؿϥ顼̣֤ͤƱ˳ѿ 
     #merror_code ˥Ǥʤ򥻥åȤ롣
 
-    ⤦Ĥμϥƥ顼Ǥ롣ƥबɬפ̤Υ
-    Ƥ뤳ȤǤʤ硢饤֥ؿϳѿ @c
-    m17n_memory_full_handler ؤؿƤ֡ǥեȤǤϡؿ 
+    ⤦Ĥμϥƥ顼Ǥ롣
+    ƥबɬפ̤ΥƤ뤳ȤǤʤ硢饤֥ؿϳѿ 
+    @c m17n_memory_full_handler ؤؿƤ֡ǥեȤǤϡؿ 
     default_error_handle () ؤƤꡢδؿñ <tt>exit
     ()</tt> Ƥ֡
 */
@@ -938,18 +959,17 @@ int merror_code;
 /***ja 
     @brief ƥ顼ϥɥ.
 
-    ѿ #m17n_memory_full_handler ϡ饤֥ؿ
-    ˼Ԥݤ˸Ƥ֤٤ؿؤΥݥ󥿤Ǥ롣$ERR  @c enum
-    #MErrorCode Τ줫ǤꡢɤΥ饤֥ؿǥ顼ä
-    򼨤
+    ѿ #m17n_memory_full_handler 
+    ϡ饤֥ؿƤ˼Ԥݤ˸Ƥ֤٤ؿؤΥݥ󥿤Ǥ롣
+    $ERR  @c enum #MErrorCode 
+    ΤΤ줫ǤꡢɤΥ饤֥ؿǥ顼ä򼨤
 
     @anchor test
 
-    Ǥϡѿñ <tt>exit ()</tt>  $ERR Ȥ
-    ƤִؿؤƤ롣
+    Ǥϡѿñ <tt>exit ()</tt>  $ERR 
+    ȤƸƤִؿؤƤ롣
 
-    Ȥϰۤʤ륨顼ɬפȤ륢ץꥱϡѿ
-    Ŭʴؿꤹ뤳ȤǡŪãǤ롣  */
+    Ȥϰۤʤ륨顼ɬפȤ륢ץꥱϡѿŬʴؿꤹ뤳ȤǡŪãǤ롣  */
 
 void (*m17n_memory_full_handler) (enum MErrorCode err);
 
@@ -1015,8 +1035,7 @@ void (*m17n_memory_full_handler) (enum MErrorCode err);
     @addtogroup m17nDebug
     @brief m17n 饤֥桼ΤΥץǥХåݡ.
 
-    m17n 饤֥ϡΥ桼ʬΥץǥХå뤿
-    ˡʲεǽ򥵥ݡȤƤ롣
+    m17n 饤֥ϡΥ桼ʬΥץǥХå뤿ˡʲεǽ򥵥ݡȤƤ롣
 
     <ul>
 
@@ -1024,40 +1043,38 @@ void (*m17n_memory_full_handler) (enum MErrorCode err);
 
     <ul>
 
-    <li> MDEBUG_INIT -- 1 ʤСM17N_INIT () ƤФ줿ǡ饤
-    ֥ν˴ؤץȤ롣
+    <li> MDEBUG_INIT -- 1 ʤСM17N_INIT () 
+    ƤФ줿ǡ饤֥ν˴ؤץȤ롣
 
-    <li> MDEBUG_FINI -- 1 ʤСM17N_FINI () ƤФ줿ǡޤ
-    Ƥʤ֥ȤλȿץȤ롣
+    <li> MDEBUG_FINI -- 1 ʤСM17N_FINI () 
+    ƤФ줿ǡޤƤʤ֥ȤλȿץȤ롣
 
-    <li> MDEBUG_CHARSET -- 1 ʤСm17n ǡ١ɤ줿
-    ʸåȤˤĤƤξץȤ롣
+    <li> MDEBUG_CHARSET -- 1 ʤСm17n
+    ǡ١ɤ줿ʸåȤˤĤƤξץȤ롣
 
-    <li> MDEBUG_CODING --  1 ʤСm17n ǡ١ɤ줿
-    ɷϤˤĤƤξץȤ롣
+    <li> MDEBUG_CODING --  1 ʤСm17n 
+    ǡ١ɤ줿ɷϤˤĤƤξץȤ롣
 
-    <li> MDEBUG_DATABASE -- 1 ʤСm17n ǡ١ɤ
-    ǡˤĤƤξץȤ롣
+    <li> MDEBUG_DATABASE -- 1 ʤСm17n
+    ǡ١ɤ줿ǡˤĤƤξץȤ롣
 
-    <li> MDEBUG_FONT -- 1 ʤС򤵤ƥץ󤵤줿եȤˤ
-    ƤξץȤ롣
+    <li> MDEBUG_FONT -- 1 ʤС򤵤ƥץ󤵤줿եȤˤĤƤξץȤ롣
 
-    <li> MDEBUG_FONT_FLT -- 1 ʤСFont Layout Table ΤɤΥޥ
-    ¹椫ˤĤƤΤץȤ롣
+    <li> MDEBUG_FONT_FLT -- 1 ʤСFont Layout Table 
+    ΤɤΥޥɤ¹椫ˤĤƤΤץȤ롣
 
-    <li> MDEBUG_FONT_OTF -- 1 ʤСOpenType Layout Table Τɤ°
-    ¹椫ˤĤƤΤץȤ롣
+    <li> MDEBUG_FONT_OTF -- 1 ʤСOpenType Layout Table 
+    Τɤ°¹椫ˤĤƤΤץȤ롣
 
-    <li> MDEBUG_INPUT -- 1 ʤС¹ϥ᥽åɤξ֤դ
-    ξץȤ롣
+    <li> MDEBUG_INPUT -- 1 ʤС¹ϥ᥽åɤξ֤դƤξץȤ롣
 
-    <li> MDEBUG_ALL -- 1 ʤС嵭٤Ƥѿ 1 ˤΤƱ
-    ̤ġ
+    <li> MDEBUG_ALL -- 1 ʤС嵭٤Ƥѿ 1 
+    ˤΤƱ̤ġ
 
     </ul>
 
-    <li> Υ֥Ȥʹ֤˲ɤʷǥץȤؿܺ٤
-    ؿ mdebug_dump_XXXX () ȡ
+    <li> Υ֥Ȥʹ֤˲ɤʷǥץȤؿܺ٤ϴؿ
+    mdebug_dump_XXXX () ȡ
 
     <li> 顼ȯ˸ƤФեåؿmdebug_hook () ȡ
 
@@ -1078,8 +1095,7 @@ void (*m17n_memory_full_handler) (enum MErrorCode err);
     @brief 顼κݤ˸ƤФեåؿ.
 
     ؿ mdebug_hook () ϥ顼äݤ˸ƤФ졢⤻-1 
-    ֤ǥХåǥ֥졼ݥȤꤹ뤿Ѥ뤳ȤǤ
-    롣
+    ֤ǥХåǥ֥졼ݥȤꤹ뤿Ѥ뤳ȤǤ롣
     */ 
 
 int
