@@ -1,5 +1,5 @@
 /* m17n-core.h -- header file for the CORE API of the m17n library.
-   Copyright (C) 2003, 2004
+   Copyright (C) 2003, 2004, 2005, 2006
      National Institute of Advanced Industrial Science and Technology (AIST)
      Registration Number H15PRO112
 
@@ -41,9 +41,9 @@ extern "C"
 #if !defined (FOR_DOXYGEN) || defined (DOXYGEN_INTERNAL_MODULE)
 
 #define M17NLIB_MAJOR_VERSION 1
-#define M17NLIB_MINOR_VERSION 2
-#define M17NLIB_PATCH_LEVEL 0
-#define M17NLIB_VERSION_NAME "1.2.0"
+#define M17NLIB_MINOR_VERSION 3
+#define M17NLIB_PATCH_LEVEL 1
+#define M17NLIB_VERSION_NAME "1.3.1"
 
 extern void m17n_init_core (void);
 #define M17N_INIT() m17n_init_core ()
@@ -101,11 +101,10 @@ extern enum M17NStatus m17n_status (void);
 /***ja
     @brief ֥ȤκǽΥ.
 
-    ץꥱץब¤Τ֥ȤȤ
-     ݤˤϡǽΥФ @c #M17NObjectHead ¤ηǤʤ
-     ƤϤʤʤ@c #M17NObjectHead Ƥ m17n 饤֥꤬Ѥ
-     ΤǡץꥱץϿƤϤʤʤ
-     */
+    ץꥱץब¤Τ֥ȤȤݤˤϡǽΥФ 
+    @c #M17NObjectHead ¤ηǤʤƤϤʤʤ
+    @c #M17NObjectHead Ƥ m17n 
+    饤֥꤬ѤΤǡץꥱץϿƤϤʤʤ    */
 
 typedef struct
 {
@@ -143,8 +142,8 @@ extern int m17n_object_unref (void *object);
 /***ja
     @brief ܥη.
 
-    #MSymbol ϥܥ륪֥ȤηǤ롣¤ϥץꥱ
-    ץफϸʤ  */
+    #MSymbol  @e ܥ (symbol) ֥ȤηǤ롣
+    ¤ϥץꥱץफϸʤ  */
 
 typedef struct MSymbolStruct *MSymbol;
 
@@ -163,6 +162,9 @@ extern MSymbol msymbol (const char *name);
 /* Return a managing key of name NAME.  */
 extern MSymbol msymbol_as_managing_key (const char *name);
 
+/* Check if SYMBOL is a managing key.  */
+extern int msymbol_is_managing_key (MSymbol symbol);
+
 /* Return a symbol of name NAME if it already exists.  */
 extern MSymbol msymbol_exist (const char *name);
 
@@ -299,6 +301,8 @@ extern int mchartable_map (MCharTable *table, void *ignore,
 
 extern void mchartable_range (MCharTable *table, int *from, int *to);
 
+extern MCharTable *mchar_get_prop_table (MSymbol key, MSymbol *type);
+
 /*
  *  (5) Handling M-text.
  *	"M" of M-text stands for:
@@ -347,8 +351,8 @@ typedef struct MText MText;
 /***ja
     @brief M-text ΥեޥåȤꤹ.
 
-     #MTextFormat 
-    ؿ mtext_from_data () ΰȤѤ졢
+     #MTextFormat ϴؿ
+    mtext_from_data () ΰȤѤ졢
     M-text 븵ȤʤǡΥեޥåȤꤹ롣  */
 
 enum MTextFormat
@@ -365,6 +369,9 @@ enum MTextFormat
 
 extern MText *mtext ();
 
+extern void *mtext_data (MText *mt, enum MTextFormat *fmt, int *nunits,
+			 int *pos_idx, int *unit_idx);
+
 /*=*/
 
 /***en @name Variables: Default Endian of UTF-16 and UTF-32 */
@@ -385,8 +392,8 @@ extern MText *mtext ();
 /***ja
     @brief ͤ MTEXT_FORMAT_UTF_16LE  MTEXT_FORMAT_UTF_16BE Ǥѿ
 
-    ѿ #MTEXT_FORMAT_UTF_16 ϥȥ롦ǥ󡦥ƥʥ
-    ɤ LSB (Least Significant Byte) ˤƳǼ˾Ǥ
+    ѿ #MTEXT_FORMAT_UTF_16 ϥȥ롦ǥ󡦥ƥ
+    ʥɤ LSB (Least Significant Byte) ˤƳǼ˾Ǥ
     #MTEXT_FORMAT_UTF_16LE ˽졢ӥåǥ󡦥ƥ
     ʥɤ MSB (Most Significant Byte) ˤƳǼ˾Ǥ
     #MTEXT_FORMAT_UTF_16BE ˽롣  */
@@ -395,7 +402,7 @@ extern MText *mtext ();
     @seealso
     mtext_from_data ()  */
 
-extern const int MTEXT_FORMAT_UTF_16;
+extern const enum MTextFormat MTEXT_FORMAT_UTF_16;
 /*=*/
 
 /*** @ingroup m17nMtext */
@@ -411,8 +418,8 @@ extern const int MTEXT_FORMAT_UTF_16;
 /***ja
     @brief ͤ MTEXT_FORMAT_UTF_32LE  MTEXT_FORMAT_UTF_32BE Ǥѿ
 
-    ѿ #MTEXT_FORMAT_UTF_32 ϥȥ롦ǥ󡦥ƥʥ
-    ɤ LSB (Least Significant Byte) ˤƳǼ˾Ǥ
+    ѿ #MTEXT_FORMAT_UTF_32 ϥȥ롦ǥ󡦥ƥ
+    ʥɤ LSB (Least Significant Byte) ˤƳǼ˾Ǥ
     #MTEXT_FORMAT_UTF_32LE ˽졢ӥåǥ󡦥ƥ
     ʥɤ MSB (Most Significant Byte) ˤƳǼ˾Ǥ
     #MTEXT_FORMAT_UTF_32BE ˽롣  */
@@ -459,8 +466,13 @@ extern int mtext_del (MText *mt, int from, int to);
 
 extern int mtext_ins (MText *mt1, int pos, MText *mt2);
 
+extern int mtext_insert (MText *mt1, int pos, MText *mt2, int from, int to);
+
 extern int mtext_ins_char (MText *mt, int pos, int c, int n);
 
+extern int mtext_replace (MText *mt1, int from1, int to1,
+			  MText *mt2, int from2, int to2);
+
 extern MText *mtext_cat_char (MText *mt, int c);
 
 extern MText *mtext_duplicate (MText *mt, int from, int to);
@@ -499,6 +511,35 @@ extern int mtext_casecmp (MText *mt1, MText *mt2);
 
 extern int mtext_ncasecmp (MText *mt1, MText *mt2, int n);
 
+extern int mtext_lowercase (MText *mt);
+
+extern int mtext_titlecase (MText *mt);
+
+extern int mtext_uppercase (MText *mt);
+
+/***en
+    @brief Enumeration for specifying a set of line breaking option.
+
+    The enum #MTextLineBreakOption is to control the line breaking
+    algorithm of the function mtext_line_break () by specifying
+    logical-or of the members in the arg @e option.  */
+
+enum MTextLineBreakOption
+  {
+    /***en Specify the legacy support for space character as base for
+       combining marks.  See the section 8.3 of UAX#14. */
+    MTEXT_LBO_SP_CM = 1,
+    /***en Specify to use space characters for line breaking Korean
+	text.  */
+    MTEXT_LBO_KOREAN_SP = 2,
+    /***en Specify to treat characters of ambiguous line-breaking
+	class as of ideographic line-breaking class.  */
+    MTEXT_LBO_AI_AS_ID = 4,
+    MTEXT_LBO_MAX
+  };
+
+extern int mtext_line_break (MText *mt, int pos, int option, int *after);
+
 /*** @ingroup m17nPlist */
 extern MPlist *mplist_deserialize (MText *mt);
 
@@ -522,45 +563,43 @@ extern MPlist *mplist_deserialize (MText *mt);
 /***ja
     @brief ƥȥץѥƥ椹ե饰ӥå.
 
-    ؿ mtext_property () ϰʲΥե饰ӥåȤ OR Ȥ
-    Ȥ뤳ȤǤ롣ե饰ӥåȤ줿ƥȥץѥƥο
-    椹롣ܺ٤ϳƥե饰ӥåȤ򻲾ȡ*/
+    ؿ mtext_property () ϰʲΥե饰ӥåȤ
+    OR ȤƤȤ뤳ȤǤ롣
+    ե饰ӥåȤ줿ƥȥץѥƥο񤤤椹롣
+    ܺ٤ϳƥե饰ӥåȤ򻲾ȡ*/
 
 enum MTextPropertyControl
   {
     /***en If this flag bit is on, an M-text inserted at the start
 	position or at the middle of the text property inherits the
 	text property.  */
-    /***ja ΥӥåȤ on ʤСΥƥȥץѥƥλϤޤ
-	뤤֤줿 M-text ϤΥƥȥץѥƥѾ롣
+    /***ja ΥӥåȤ on ʤСΥƥȥץѥƥλϤޤ뤤֤줿
+	M-text ϤΥƥȥץѥƥѾ롣
 	*/
     MTEXTPROP_FRONT_STICKY = 0x01,
 
     /***en If this flag bit is on, an M-text inserted at the end
 	position or at the middle of the text property inherits the
 	text property.  */
-    /***ja ΥӥåȤ on ʤСΥƥȥץѥƥν
-	뤤֤줿 M-text ϤΥƥȥץѥƥѾ롣
+    /***ja ΥӥåȤ on ʤСΥƥȥץѥƥν뤤֤줿
+	M-text ϤΥƥȥץѥƥѾ롣
 	*/
     MTEXTPROP_REAR_STICKY = 0x02,
 
     /***en If this flag bit is on, the text property is removed if a
 	text in its region is modified.  */
-    /***ja ΥӥåȤ on ʤСΥƥȥץѥƥϰ
-	ƥȤѹ줿ƥȥץѥƥϼ롣  */
+    /***ja ΥӥåȤ on ʤСΥƥȥץѥƥϰΥƥȤѹ줿ƥȥץѥƥϼ롣  */
     MTEXTPROP_VOLATILE_WEAK = 0x04,
 
     /***en If this flag bit is on, the text property is removed if a
 	text or the other text property in its region is modified.  */
-    /***ja ΥӥåȤ on ʤСΥƥȥץѥƥϰ
-	ƥȤ뤤̤Υƥȥץѥƥѹ줿礳Υƥ
+    /***ja ΥӥåȤ on ʤСΥƥȥץѥƥϰΥƥȤ뤤̤Υƥȥץѥƥѹ줿礳Υƥ
 	ȥץѥƥϼ롣*/
     MTEXTPROP_VOLATILE_STRONG = 0x08,
 
     /***en If this flag bit is on, the text property is not
 	automatically merged with the others.  */
-    /***ja ΥӥåȤ on ʤСΥƥȥץѥƥ¾Υץ
-	ƥȼưŪˤϥޡʤ */
+    /***ja ΥӥåȤ on ʤСΥƥȥץѥƥ¾ΥץѥƥȼưŪˤϥޡʤ */
     MTEXTPROP_NO_MERGE = 0x10,
 
     MTEXTPROP_CONTROL_MAX = 0x1F
