@@ -40,17 +40,16 @@
     @addtogroup m17nGUI
     @brief ɥƥ GUI ݡ.
 
-    ΥϥɥƥΤȤǤ M-text ɽϤ
-     m17n GUI API 롣
+    ΥϥɥƥΤȤǤ M-text ɽϤˤ
+    m17n GUI API 롣
 
     ǤΤ٤ƤϥɥƥȤΩǤ롣
-    ݤΥ饤֥եϸ̤Υɥƥ˰¸礬
-    롣ȤХ饤֥ե m17n-X.so ϡm17n GUI API  X 
+    ºݤΥ饤֥եϸ̤Υɥƥ˰¸礬롣
+    ȤХ饤֥ե m17n-X.so ϡm17n GUI API  X 
     ɥѤμǤ롣
 
     ¤ˤϡGUI API ϼ˥ġ륭åȥ饤֥Ǥ뤫ޤ 
-    XOM 뤿ѤƤꡢץꥱץफ
-    ľܤѤǰƬˤΤǤϤʤ
+    XOM 뤿ѤƤꡢץꥱץफľܤѤǰƬˤΤǤϤʤ
 */
 
 /*=*/
@@ -113,7 +112,6 @@ free_frame (void *object)
 
   (*frame->driver->close) (frame);
   M17N_OBJECT_UNREF (frame->face);
-  free (frame->font);
   M17N_OBJECT_UNREF (frame->font_driver_list);
   free (object);
 }
@@ -198,8 +196,8 @@ null_device_fini ()
     mface__free_realized ((MRealizedFace *) MPLIST_VAL (plist));
   M17N_OBJECT_UNREF (null_device.realized_face_list);
 
-  MPLIST_DO (plist, null_device.realized_font_list)
-    mfont__free_realized ((MRealizedFont *) MPLIST_VAL (plist));
+  if (MPLIST_VAL (null_device.realized_font_list))
+    mfont__free_realized (MPLIST_VAL (null_device.realized_font_list));
   M17N_OBJECT_UNREF (null_device.realized_font_list);
   return 0;
 }
@@ -211,6 +209,9 @@ null_device_open (MFrame *frame, MPlist *param)
 
   frame->device = NULL;
   frame->device_type = 0;
+  frame->dpi = (int) mplist_get (param, Mresolution);
+  if (frame->dpi == 0)
+    frame->dpi = 100;
   frame->driver = &null_driver;
   frame->font_driver_list = mplist ();
   mplist_add (frame->font_driver_list, Mfreetype, &mfont__ft_driver);
@@ -238,6 +239,7 @@ m17n_init_win (void)
 {
   int mdebug_mask = MDEBUG_INIT;
 
+  merror_code = MERROR_NONE;
   if (m17n__gui_initialized++)
     return;
   m17n_init ();
@@ -262,6 +264,7 @@ m17n_init_win (void)
   Mdrawable = msymbol ("drawable");
   Mdepth = msymbol ("depth");
   Mwidget = msymbol ("widget");
+  Mcolormap = msymbol ("colormap");
 
   MDEBUG_PUSH_TIME ();
   if (mfont__init () < 0)
@@ -314,22 +317,23 @@ m17n_fini_win (void)
 	  dlclose (interface->handle);
 	}
       free (interface->library);
+      free (interface);
     }
 #ifdef HAVE_FREETYPE
   if (null_interface.handle)
     (*null_interface.fini) ();
 #endif	/* not HAVE_FREETYPE */
   M17N_OBJECT_UNREF (device_library_list);
-  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize input-gui module."));
   minput__win_fini ();
-  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize draw module."));
+  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize input-gui module."));
   mdraw__fini ();
-  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize face module."));
+  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize draw module."));
   mface__fini ();
-  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize fontset module."));
+  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize face module."));
   mfont__fontset_fini ();
-  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize font module."));
+  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize fontset module."));
   mfont__fini ();
+  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize font module."));
   mframe_default = NULL;
   MDEBUG_POP_TIME ();
   MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize the gui modules."));
@@ -347,13 +351,12 @@ m17n_fini_win (void)
     A @e frame is an object of the type #MFrame to hold various
     information about each display/input device.  Almost all m17n GUI
     functions require a pointer to a frame as an argument.  */
-
 /***ja
     @brief @e ե졼 ȤϥեåǥХб륪֥ȤǤ.
 
-    @e ե졼 Ȥ #MFrame Υ֥ȤǤꡢġɽϥ
-    ХξǼ뤿Ѥ롣ۤȤɤ٤Ƥ m17n GUI
-    ؿϡȤƥե졼ؤΥݥ󥿤׵᤹롣  */
+    @e ե졼 Ȥ #MFrame 
+    Υ֥ȤǤꡢġɽϥǥХξǼ뤿Ѥ롣
+    ۤȤɤ٤Ƥ m17n GUIؿϡȤƥե졼ؤΥݥ󥿤׵᤹롣  */
 
 /*** @{ */
 /*=*/
@@ -361,7 +364,7 @@ m17n_fini_win (void)
 /***en
     @name Variables: Keys of frame parameter
 
-    These are the symbols to use in a parameter to creat a frame.  See
+    These are the symbols to use in a parameter to create a frame.  See
     the function mframe () for details.
 
     #Mdevice, #Mdisplay, #Mscreen, #Mdrawable, #Mdepth, and #Mcolormap
@@ -516,8 +519,7 @@ MSymbol Mfont_descent;
     ؿ mframe ()  $PLIST Υѥ᡼Ŀե졼롣
     $PLIST  @c NULL Ǥɤ
 
-    $PLIST ˸륭Τɤ줬ǧ뤫ϥɥƥ
-    ˰¸롣
+    $PLIST ˸륭Τɤ줬ǧ뤫ϥɥƥ˰¸롣
 
     ʲΥϾǧ롣
 
@@ -526,18 +528,18 @@ MSymbol Mfont_descent;
     <li> #Mdevice. ͤ #Mx, #Mgd, #Mnil Τ줫ǤʤƤϤʤʤ
 
     ͤ #Mx ʤСե졼 X ɥƥѤǤ롣
-    Υե졼ȶ˻ꤵ줿 #MDrawWindow ϡ @c Window Ǥ
-    ƤϤʤʤե졼ɤ߽񤭤Ȥ˲ǽǤꡢ٤ƤGUI 
-    ѤǤ롣
+    Υե졼ȶ˻ꤵ줿 #MDrawWindow ϡ @c Window
+    ǤʤƤϤʤʤե졼ɤ߽񤭤Ȥ˲ǽǤꡢ٤ƤGUI 
+    ؿѤǤ롣
 
-    ͤ #Mgd ʤСե졼 GD 饤֥Υ᡼֥
-    ѤǤ롣Υե졼ȶ˻ꤵ줿 #MDrawWindow ϡ 
-    @c gdImagePtr ǤʤƤϤʤʤե졼Ͻ񤭽ФѤǤꡢ
+    ͤ #Mgd ʤСե졼 GD 
+    饤֥Υ᡼֥ѤǤ롣Υե졼ȶ˻ꤵ줿
+    #MDrawWindow ϡ @c gdImagePtr ǤʤƤϤʤʤե졼Ͻ񤭽ФѤǤꡢ
     minput_ ǻϤޤ̾δؿϻѤǤʤ
 
-    ͤ #Mnil ʤСե졼, null ǥХѤǤ롣Υ
-    졼ɤ߽񤭤ǤʤΤǡ #MDrawWindow ɬפȤmdraw_ 
-    ǻϤޤ̾δؿ䡢minput_ ǻϤޤ̾δؿϻѤǤʤ
+    ͤ #Mnil ʤСե졼, null 
+    ǥХѤǤ롣Υե졼ɤ߽񤭤ǤʤΤǡ #MDrawWindow 
+    ɬפȤmdraw_ ǻϤޤ̾δؿ䡢minput_ ǻϤޤ̾δؿϻѤǤʤ
 
     <li> #Mface. ͤ #MFace ؤΥݥ󥿤ǤʤƤϤʤʤ
 
@@ -545,8 +547,8 @@ MSymbol Mfont_descent;
 
     </ul>
 
-    Υ˲ä#Mdevice Υ #Mx Ǥ˸¤ʲΥ
-    ǧ롣ʲΥϥ롼ȥɥȡե졼ѤǤ 
+    Υ˲ä#Mdevice Υ #Mx 
+    Ǥ˸¤ʲΥǧ롣ʲΥϥ롼ȥɥȡե졼ѤǤ 
     drawable οꤹ롣
 
     <ul>
@@ -554,16 +556,15 @@ MSymbol Mfont_descent;
     <li> #Mdrawable. ͤ <tt>Drawable</tt> ǤʤƤϤʤʤ
 
      #Mdisplay ĥѥ᡼ꤵƤɬפ롣
-    줿ե졼ϡꤵ줿ǥץ쥤λꤵ줿 drawable Ʊ
-    롼ȥɥȿ drawable Ѥ롣
+    줿ե졼ϡꤵ줿ǥץ쥤λꤵ줿 drawable 
+    Ʊ롼ȥɥȿ drawable Ѥ롣
 
-    Υѥ᡼ˤϡ#Mscreen 򥭡Ȥѥ᡼̵
-    뤵롣
+    Υѥ᡼ˤϡ#Mscreen 򥭡Ȥѥ᡼̵뤵롣
 
     <li> #Mwidget. ͤ <tt>Widget</tt> ǤʤƤϤʤʤ
 
-    줿ե졼ϡꤷåȤƱ롼ȥɥ
-     drawable Ѥ롣
+    줿ե졼ϡꤷåȤƱ롼ȥɥȿ
+    drawable Ѥ롣
 
      #Mface ĥѥ᡼ʤСǥեȤΥեϤ
     åȤΥ꥽롣
@@ -577,16 +578,13 @@ MSymbol Mfont_descent;
 
     <li> #Mscreen. ͤ <tt>(Screen *)</tt> ǤʤƤϤʤʤ
 
-    ե졼ϡꤷ꡼Ʊ롼ȥɥ
-    ꡼ΥǥեȤοƱ drawable Ѥ롣
+    ե졼ϡꤷ꡼Ʊ롼ȥɥ꡼ΥǥեȤοƱ drawable Ѥ롣
 
-    Υѥ᡼ˤϡ#Mdisplay 򥭡Ȥѥ᡼̵
-    뤵롣
+    Υѥ᡼ˤϡ#Mdisplay 򥭡Ȥѥ᡼̵뤵롣
 
     <li> #Mdisplay. ͤ <tt>(Display *)</tt> ǤʤƤϤʤʤ
 
-    줿ե졼ϡꤷǥץ쥤Υǥեȥ꡼
-    Ʊ롼ȥɥƱdrawables Ѥ롣
+    줿ե졼ϡꤷǥץ쥤Υǥեȥ꡼Ʊ롼ȥɥƱdrawables Ѥ롣
 
     <li> #Mcolormap. ͤ <tt>(Colormap)</tt> ǤʤƤϤʤʤ
 
@@ -595,21 +593,19 @@ MSymbol Mfont_descent;
     <li> #Mfont. ͤϡ#Mx, #Mfreetype, #Mxft Τ줫
 
     줿ե졼ϻꤷեȥХåɤѤ롣ͤ 
-    #Mx Ǥ X Υեȡ#Mfreetype Ǥ FreeType ǥݡ
-    ȤƤեȡ#Mxft Ǥ Xft 饤֥ͳ
-    ѤեȤѤ롣ʣΥեȥХåɤ
-    ˤϡΥѥ᡼ʣ󡢰ۤʤͤǻꤹ뤳Ȥ
-    롣ꤷХåɤݡȤƤʤǥХǤϡ
-    ѥ᡼̵뤵롣
+    #Mx Ǥ X Υեȡ#Mfreetype Ǥ FreeType 
+    ǥݡȤƤեȡ#Mxft Ǥ Xft 
+    饤֥ͳѤեȤѤ롣
+    ʣΥեȥХåɤѤˤϡΥѥ᡼ʣ󡢰ۤʤͤǻꤹ뤳ȤǤ롣
+    ꤷХåɤݡȤƤʤǥХǤϡΥѥ᡼̵뤵롣
 
-    Υѥ᡼̵ˤϡǥХǥݡȤƤ뤹٤Ƥ
-    եȥХåɤѤ롣
+    Υѥ᡼̵ˤϡǥХǥݡȤƤ뤹٤ƤΥեȥХåɤѤ롣
 
     </ul>
 
     @return
      mframe() Ͽե졼ؤΥݥ󥿤֤Ǥʤ
-     @c NULL ֤  */
+    @c NULL ֤  */
 
 MFrame *
 mframe (MPlist *plist)
@@ -656,9 +652,13 @@ mframe (MPlist *plist)
       if (! interface->handle)
 	{
 	  if (! (interface->handle = dlopen (interface->library, RTLD_NOW))
-	      || ! (interface->init = dlsym (interface->handle, "device_init"))
-	      || ! (interface->open = dlsym (interface->handle, "device_open"))
-	      || ! (interface->fini = dlsym (interface->handle, "device_fini"))
+	      || ! (interface->init
+		    = (int (*) ()) dlsym (interface->handle, "device_init"))
+	      || ! (interface->open
+		    = (int (*) (MFrame *, MPlist *)) dlsym (interface->handle,
+							    "device_open"))
+	      || ! (interface->fini
+		    = (int (*) ()) dlsym (interface->handle, "device_fini"))
 	      || (*interface->init) () < 0)
 	    {
 	      fprintf (stderr, "%s\n", (char *) dlerror ());
@@ -684,7 +684,8 @@ mframe (MPlist *plist)
     if (MPLIST_KEY (pl) == Mface)
       mface_merge (frame->face, (MFace *) MPLIST_VAL (pl));
   mface__update_frame_face (frame);
-
+  frame->font
+    = frame->rface->rfont ? (MFont *) frame->rface->rfont : NULL;
   if (plist_created)
     M17N_OBJECT_UNREF (plist);
   return frame;
@@ -735,8 +736,8 @@ mframe (MPlist *plist)
 /***ja
     @brief ե졼Υץѥƥ֤ͤ.
 
-    ؿ mframe_get_prop () ϥե졼 $FRAME Υ $KEY ĥץ
-    ƥ֤ͤͭʥȤͤϰʲ̤ꡣ
+    ؿ mframe_get_prop () ϥե졼 $FRAME Υ $KEY 
+    ĥץѥƥ֤ͤͭʥȤͤϰʲ̤ꡣ
 
 @verbatim
 
@@ -778,9 +779,9 @@ mframe_get_prop (MFrame *frame, MSymbol key)
   if (key == Mface)
     return frame->face;
   if (key == Mfont)
-    return &frame->rface->rfont->font;
+    return frame->font;
   if (key == Mfont_width)
-    return (void *) (frame->space_width);
+    return (void *) (frame->average_width);
   if (key == Mfont_ascent)
     return (void *) (frame->ascent);
   if (key == Mfont_descent)
@@ -799,9 +800,8 @@ mframe_get_prop (MFrame *frame, MSymbol key)
 /***ja
     @brief ǥեȤΥե졼.
 
-    ѿ #mframe_default ϡǥեȤΥե졼ؤΥݥ󥿤
-    ġǥեȤΥե졼ϡǽ mframe () ƤӽФ줿Ȥ
-    롣  */
+    ѿ #mframe_default ϡǥեȤΥե졼ؤΥݥ󥿤ġ
+    ǥեȤΥե졼ϡǽ mframe () ƤӽФ줿Ȥ˺롣  */
 
 MFrame *mframe_default;
 
