@@ -69,14 +69,13 @@ extern void m17n_fini_win (void);
 /***ja
     @brief ե졼η.
 
-    #MFrame ϡ@e ե졼 ֥ѤηǤ롣ġΥե졼ϡ
-    줬бʪŪɽϥǥХγƼݻ롣
+    #MFrame ϡ@e ե졼 ֥ѤηǤ롣
+    ġΥե졼ϡ줬бʪŪɽϥǥХγƼݻ롣
 
     #MFrame ¤ϡץꥱץफϸʤ
-    ޤƤϻѤ륦ɥƥ˰¸롣ޤm17n-X 
-    ֥ˤե졼ϡX ɥ @e display  @e screen 
-    ˴ؤġ
-      */
+    ޤƤϻѤ륦ɥƥ˰¸롣ޤ m17n-X 
+    饤֥ˤե졼ϡX ɥ @e display  @e screen 
+    ˴ؤġ      */
 
 typedef struct MFrame MFrame;
 
@@ -127,11 +126,10 @@ extern void *mframe_get_prop (MFrame *frame, MSymbol key);
     @brief եȤη.
 
     #MFont ϥեȻѤι¤ΤǤꡢեȤΥץѥƥǤ
-     foundry, family, weight, style, stretch, adstyle, registry,
+    foundry, family, weight, style, stretch, adstyle, registry,
     size, resolution ˴ؤޤࡣ
 
-    ι¤ΤϥեȥåΥեȤꤹݤȡѲǽʥ
-    ƥեȤξǼݤξѤ롣
+    ι¤ΤϥեȥåΥեȤꤹݤȡѲǽʥƥեȤξǼݤξѤ롣
 
     ¤ϥץꥱץफϸʤ  */
 
@@ -151,13 +149,13 @@ extern MFont *mfont ();
 
 extern MFont *mfont_copy (MFont *font);
 
-extern MFont *mfont_parse_name (char *name, MSymbol format);
+extern MFont *mfont_parse_name (const char *name, MSymbol format);
 
 extern char *mfont_unparse_name (MFont *font, MSymbol format);
 
 /* These two are obsolete (from 1.1.0).  */
 extern char *mfont_name (MFont *font);
-extern MFont *mfont_from_name (char *name);
+extern MFont *mfont_from_name (const char *name);
 
 extern MSymbol Mfoundry;
 extern MSymbol Mfamily;
@@ -165,9 +163,12 @@ extern MSymbol Mweight;
 extern MSymbol Mstyle;
 extern MSymbol Mstretch;
 extern MSymbol Madstyle;
+extern MSymbol Mspacing;
 extern MSymbol Mregistry;
 extern MSymbol Msize;
 extern MSymbol Mresolution;
+extern MSymbol Mmax_advance;
+extern MSymbol Mfontfile;
 
 extern MSymbol Mfontconfig;
 
@@ -197,15 +198,15 @@ extern int mfont_set_encoding (MFont *font,
 /***ja
     @brief եȤõ.
 
-    ؿ mfont_find () ϡե졼 $FRAME ǥե $SPEC ˤ
-    ȤפѲǽʥեȤؤΥݥ󥿤֤  
+    ؿ mfont_find () ϡե졼 $FRAME ǥե $SPEC 
+    ˤäȤפѲǽʥեȤؤΥݥ󥿤֤  
 
-    $SCORE  NULL Ǥ뤫ĤäեȤ $SPEC ˤɤۤɹ
-    Ƥ뤫򼨤¸ؤΥݥ󥿤Ǥ롣
+    $SCORE  NULL Ǥ뤫ĤäեȤ $SPEC 
+    ˤɤۤɹäƤ뤫򼨤¸ؤΥݥ󥿤Ǥ롣
     ۤɤäƤ뤳Ȥ̣롣
 
-    $LIMITED_SIZE  0 ǤʤС$SPEC Υץѥƥ #Msize 礭
-    ʤեȤõ롣
+    $LIMITED_SIZE  0 ǤʤС$SPEC Υץѥƥ #Msize 
+    礭ʤեȤõ롣
 */
 
 extern MFont *mfont_find (MFrame *frame, MFont *spec,
@@ -220,6 +221,10 @@ extern int mfont_resize_ratio (MFont *font);
 extern MPlist *mfont_list (MFrame *frame, MFont *font, MSymbol language,
 			   int maxnum);
 
+typedef struct MFontset MFontset;
+
+extern int mfont_check (MFrame *frame, MFontset *fontset,
+			MSymbol script, MSymbol language, MFont *font);
 
 /* end of font module */
 /*=*/
@@ -230,8 +235,6 @@ extern MPlist *mfont_list (MFrame *frame, MFont *font, MSymbol language,
 /*=*/
 /*** @addtogroup m17nFontset
      @{   */
-typedef struct MFontset MFontset;
-
 extern MFontset *mfontset (char *name);
 
 extern MSymbol mfontset_name (MFontset *fontset);
@@ -265,8 +268,8 @@ extern MPlist *mfontset_lookup (MFontset *fontset, MSymbol script,
 /***ja
     @brief եη.
 
-    #MFace ϥե֥ȤΤι¤ΤǤ롣¤
-    ץꥱץफϸʤ  */
+    #MFace ϥե֥ȤΤι¤ΤǤ롣
+    ¤ϥץꥱץफϸʤ  */
 
 typedef struct MFace MFace;
 /*=*/
@@ -312,6 +315,8 @@ extern MSymbol Mface;
 
 extern MFace *mface ();
 
+extern int mface_equal (MFace *face1, MFace *face2);
+
 extern MFace *mface_copy (MFace *face);
 
 extern MFace *mface_merge (MFace *dst, MFace *src);
@@ -330,8 +335,8 @@ extern MFace *mface_from_font (MFont *font);
 /***ja
     @brief եοʿѷ.
 
-    #MFaceHLineProp ϥե #Mhline ץѥƥξܺ٤ꤹ뷿
-    Ǥ롣ΥץѥƥͤϤηΥ֥ȤǤʤƤϤʤʤ
+    #MFaceHLineProp ϥե #Mhline 
+    ץѥƥξܺ٤ꤹ뷿Ǥ롣ΥץѥƥͤϤηΥ֥ȤǤʤƤϤʤʤ
       */
 
 typedef struct
@@ -369,8 +374,8 @@ typedef struct
 /***ja
     @brief եΰϤȻѷ.
 
-    #MFaceBoxProp ϥե #Mbox ץѥƥξܺ٤ꤹ뷿Ǥ
-    롣ΥץѥƥͤϤηΥ֥ȤǤʤƤϤʤʤ
+    #MFaceBoxProp ϥե #Mbox ץѥƥξܺ٤ꤹ뷿Ǥ롣
+    ΥץѥƥͤϤηΥ֥ȤؤΥݥ󥿤ǤʤƤϤʤʤ
       */
 
 typedef struct
@@ -435,12 +440,10 @@ extern void mface_update (MFrame *frame, MFace *face);
 /***ja 
     @brief ɥƥ˰¸롢ɥη.
 
-    #MDrawWindow ϥɥʤĤǥ꡼Υߥ˥
-    ȤƯΰѤηǤ롣
+    #MDrawWindow ϥɥʤĤǥ꡼Υߥ˥奢ȤƯΰѤηǤ롣
 
-    ºݤ˲ؤϥɥƥ˰¸롣 m17n X 饤֥
-    Ѥץ @c Drawable 򤳤ηѴʤƤϤʤ
-    ʤ */
+    ºݤ˲ؤϥɥƥ˰¸롣 m17n X 
+    饤֥Ѥץ @c Drawable 򤳤ηѴʤƤϤʤʤ */
 
 typedef void *MDrawWindow;
 /*=*/
@@ -458,12 +461,10 @@ typedef void *MDrawWindow;
 /***ja
     @brief ɥƥ˰¸롢ΰη.
 
-    #MDrawRegion ΰ衢ʤ꡼ǤդΥԥν
-   ŵŪˤ϶ΰѤηǤ롣
+    #MDrawRegion ΰ衢ʤ꡼ǤդΥԥνŵŪˤ϶ΰѤηǤ롣
 
-    ºݤ˲ؤϥɥƥ˰¸롣 m17n X 饤֥
-    Ѥץ @c Region 򤳤ηѴʤƤϤʤ
-    ʤ  */
+    ºݤ˲ؤϥɥƥ˰¸롣 m17n X 
+    饤֥Ѥץ @c Region 򤳤ηѴʤƤϤʤʤ  */
 
 typedef void *MDrawRegion;
 /*=*/
@@ -487,16 +488,14 @@ typedef struct
       filled with background colors of faces put on the M-text.
       Otherwise, the background is not changed.  */
   /***ja 0 ǤʤС M-text Ȥơʤطʤ M-text 
-      եǻꤵƤطʿɽ롣Ǥʤ
-      ʤѤʤ  */
+      ΥեǻꤵƤطʿɽ롣ǤʤطʤѤʤ  */
   unsigned as_image : 1;
 
   /***en If nonzero and the first glyph of each line has negative
       lbearing, shift glyphs horizontally to right so that no pixel is
       drawn to the left of the specified position.  */
-  /***ja 0 ǤʤƹԤκǽΥդlbearing ʤСդ
-      ʿ˱ˤ餷ơꤷ֤꺸˥ԥ뤬ʤ褦
-      롣  */
+  /***ja 0 ǤʤƹԤκǽΥդ lbearing 
+      ʤСդʿ˱ˤ餷ơꤷ֤꺸˥ԥ뤬ʤ褦ˤ롣  */
   unsigned align_head : 1;
 
   /***en If nonzero, draw an M-text two-dimensionally, i.e., newlines
@@ -504,10 +503,9 @@ typedef struct
       the next line.  If <format> is non-NULL, and the function
       returns nonzero line width, a line longer than that width is
       also broken.  */
-  /***ja 0 ǤʤСM-text 򣲼ŪˡʤM-text  
+  /***ja 0 ǤʤСM-text 򣲼Ūˡʤ M-text  
       newline ǲԤ³ʸϼιԤɽ롣⤷ <format>  
-      NULL Ǥʤδؿ 0 Ǥʤ֤СĹԤ
-      Ԥ롣  */
+      NULL Ǥʤδؿ 0 Ǥʤ֤СĹԤԤ롣  */
   unsigned two_dimensional : 1;
 
   /***en If nonzero, draw an M-text to the right of a specified
@@ -532,15 +530,13 @@ typedef struct
 
   /***en If nonzero, draw glyphs with anti-aliasing if a backend font
       driver supports it.  */
-  /***ja 0 ǤʤСꥢǥդɽ롣ʥХå
-      ɤΥեȥɥ饤Фꥢǽ򥵥ݡȤƤ
-      Τߡ */
+  /***ja 0 ǤʤСꥢǥդɽ롣
+      ʥХåɤΥեȥɥ饤ФꥢǽľΤߡ */
   unsigned anti_alias : 1;
 
   /***en If nonzero, disable the adjustment of glyph positions to
       avoid horizontal overlapping at font boundary.  */
-  /***ja 0 ǤʤСեȶǤοʿΥդνŤʤ
-      뤿Υհ֤Ĵ̵ˤ롣  */
+  /***ja 0 ǤʤСեȶǤοʿΥդνŤʤ򤱤뤿Υհ֤Ĵ̵ˤ롣  */
   unsigned disable_overlapping_adjustment : 1;
 
   /***en If nonzero, the values are minimum line ascent and descent
@@ -558,17 +554,16 @@ typedef struct
   /***en If nonzero, the value specifies how many pixels each line can
       occupy on the display.  The value zero means that there is no
       limit.  It is ignored if <format> is non-NULL.  */
-  /***ja 0 ǤʤСͤϤΥǥץ쥤ǳƹԤ뤳ȤΤǤ
-      ԥ򼨤 0 ϸꤵʤȤ̣롣<format> 
-      NULL Ǥʤ̵뤵롣   */
+  /***ja 0 ǤʤСͤϤΥǥץ쥤ǳƹԤ뤳ȤΤǤԥ򼨤
+      0 ϸꤵʤȤ̣롣<format>  NULL Ǥʤ̵뤵롣   */
   unsigned int max_line_width;
 
   /***en If nonzero, the value specifies the distance between tab
       stops in columns (the width of one column is the width of a
       space in the default font of the frame).  The value zero means
       8.  */
-  /***ja 0 ǤʤСͤϥ֥ȥå״֤εΥ򥳥ñ̡ʥ
-      ե졼ΥǥեȥեȤˤʸǤˤǼ 
+  /***ja 0 ǤʤСͤϥ֥ȥå״֤εΥ򥳥ñ
+      ʥϥե졼ΥǥեȥեȤˤʸǤˤǼ 
       0  8 ̣롣 */
   unsigned int tab_width;
 
@@ -596,15 +591,15 @@ typedef struct
       ˴ŤƳƹԤΥǥȤȺ׻줾INDENT 
       WIDTH ǻؤ¸롣
 
-      ǥȤϡƹԤκǽΥդʥ 
-      <orientation_reversed>  0 λˤ뤤Ϻʤʳλˤ˲
-      ԥ뤺餹ꤹ롣ͤʤеˤ餹
+      ǥȤϡƹԤκǽΥդ򱦡ʥ 
+      <orientation_reversed>  0 
+      λˤ뤤Ϻʤʳλˤ˲ԥ뤺餹ꤹ롣ͤʤеˤ餹
 
-      ϡƹԤǥץ쥤뤳ȤΤǤԥκ
-      ͤǤ롣ͤ 0 ξ¤ʤȤ̣롣
+      ϡƹԤǥץ쥤뤳ȤΤǤԥκͤǤ롣ͤ
+      0 ξ¤ʤȤ̣롣
 
-      LINE  Y ϲʸˤäƹԤޤäݤˤ 0 ˥ꥻåȤ졢
-      ˤäƹԤޤäˤ 1 Ť䤵롣
+      LINE  Y ϲʸˤäƹԤޤäݤˤ 0 
+      ˥ꥻåȤ졢ĹԤ¤ˤäƲԤ뤿Ӥ 1 䤵롣
 
        <two_dimensional>  0 ǤʤˤΤͭǤ롣  */
   void (*format) (int line, int y, int *indent, int *width);
@@ -624,17 +619,16 @@ typedef struct
 
       The mdraw_default_line_break () function is useful for such a
       script that uses SPACE as a word separator.  */
-  /***ja NULL ǤʤСͤϹԤ˼ޤʤ˹Ԥ
-      ֤׻ؿǤ롣POS Ϻ˼ޤǸʸμʸ
-      ΰ֤Ǥ롣FROM ϹԤκǽʸΰ֡TO Ϻꤵ
-      ƤʤФιԤɽǸʸΰ֤Ǥ롣LINE  
-      Y  <format> ΰƱǤ롣
+  /***ja NULL ǤʤСͤϹԤ˼ޤʤ˹Ԥ֤׻ؿǤ롣
+      POS Ϻ˼ޤǸʸμʸΰ֤Ǥ롣FROM
+      ϹԤκǽʸΰ֡TO 
+      ϺꤵƤʤФιԤɽǸʸΰ֤Ǥ롣LINE 
+       Y  <format> ΰƱͤǤ롣
 
-      δؿϹԤʸ֤֤ʤƤϤʤʤޤMT 
-      ѹƤϤʤʤ
+      δؿϹԤʸ֤֤ʤƤϤʤʤޤ MT ѹƤϤʤʤ
 
-      ؿ mdraw_default_line_break () ϡζڤȤѤ
-      륹ץѤȤѤ뤳ȤǤ롣  */
+      ؿ mdraw_default_line_break ()
+      ϡζڤȤѤ륹ץѤȤͭѤǤ롣  */
   int (*line_break) (MText *mt, int pos, int from, int to, int line, int y);
 
   int with_cursor;
@@ -644,8 +638,8 @@ typedef struct
       displayed next to the last character of an M-text.  If the value
       is negative, even if <cursor_width> is nonzero, cursor is not
       displayed.  */
-  /***ja ɽʸ֤򼨤ʸ֤礭
-      СM-text κǸʸ٤ɽ롣ʤС
+  /***ja ɽʸ֤򼨤ʸ֤礭С 
+      M-text κǸʸ٤ɽ롣ʤС
       <cursor_width>  0 ǤʤƤ⥫ɽʤ
         */
   int cursor_pos;
@@ -654,18 +648,18 @@ typedef struct
       <cursor_pos>.  If the value is positive, it is the pixel width
       of the cursor.  If the value is negative, the cursor width is
       the same as the underlining glyph(s).  */
-  /***ja 0 ǤʤС<cursor_pos> ˥ɽ롣ͤʤС
-      Ϥ͡ʥԥñ̡ˤǤ롣ʤС
-      륰դƱǤ롣  */
+  /***ja 0 ǤʤС<cursor_pos> ˥ɽ롣
+      ͤʤСϤ͡ʥԥñ̡ˤǤ롣
+      ʤСΤ륰դƱǤ롣  */
   int cursor_width;
 
   /***en If nonzero and <cursor_width> is also nonzero, display double
       bar cursors; at the character position <cursor_pos> and at the
       logically previous character.  Both cursors have one pixel width
       with horizontal fringes at upper or lower positions.  */
-  /***ja If 0 Ǥʤ <cursor_width>  0 ǤʤСС
-      ʸ<cursor_pos> ŪˤˤʸΣɽ
-      롣Ȥ⣱ԥǡ岼˿ʿξ꤬Ĥ*/
+  /***ja If 0 Ǥʤ <cursor_width>  0 ǤʤССʸ
+      <cursor_pos> ŪˤˤʸΣɽ롣
+      Ȥ⣱ԥǡ夫˿ʿξ꤬Ĥ*/
   int cursor_bidi;
 
   /***en If nonzero, on drawing partial text, pixels of surrounding
@@ -677,14 +671,13 @@ typedef struct
       right half of V is erased if this member is zero.  By setting
       this member to nonzero, even with such a drawing, we can keep
       this CVC sequence correctly displayed.  */
-  /***ja 0 ǤʤСƥȤΰʬɽݤˡΥƥ
-      Τɽΰ˿ʬɽ롣ȤСʸ
-      λҲ-첻-ҲȤ󥹤ΤĤϡ첻ĤλҲ
-      δ֤˾ˤΤ褦롣Τ褦ʥ󥹤Ǥ
-      ƤꡢǸλҲľʤȤС֤
-      ݤʤɡˤΥФ 0 ǤС첻αȾʬä
-      ޤ 0 ʳˤ뤳ȤˤäơΤ褦ʺݤˤҲ-
-      -Ҳ󥹤ɽ³뤳ȤǤ롣  */
+  /***ja 0 ǤʤСƥȤΰʬɽݤˡΥƥȤΤɽΰ˿ʬɽ롣
+      ȤСƥ Ҳ-첻-Ҳ 
+      Ȥ󥹤ΤĤϡ첻ĤλҲδ֤˾ˤΤ褦롣
+      Τ褦ʥ󥹤ǤƤꡢǸλҲľ
+      ʤȤС֤򹹿ݤʤɡˤΥФ 0 
+      ǤС첻αȾʬäƤޤ 0 ʳˤ뤳ȤˤäơΤ褦ʺݤˤ
+      Ҳ-첻-Ҳ Υ󥹤ɽ³뤳ȤǤ롣  */
   int partial_update;
 
   /***en If nonzero, don't cache the result of any drawing information
@@ -698,11 +691,13 @@ typedef struct
 
 } MDrawControl;
 
+extern int mdraw_line_break_option;
+
 /*=*/
 
 /*** @ingroup m17nDraw */
 /***en
-    @brief Type of metric for gylphs and texts.
+    @brief Type of metric for glyphs and texts.
 
     The type #MDrawMetric is for a metric of a glyph and a drawn text.
     It is also used to represent a rectangle area of a graphic
@@ -710,8 +705,8 @@ typedef struct
 /***ja
     @brief դȥƥȤˡη.
 
-    #MDrawMetric ϥդɽ줿ƥȤˡѤηǤ롣ޤ
-    ɽǥХζΰɽΤˤѤ롣 */
+    #MDrawMetric ϥդɽ줿ƥȤˡѤηǤ롣
+    ޤɽǥХζΰɽΤˤѤ롣 */
 
 typedef struct {
   int x, y;
@@ -752,8 +747,8 @@ typedef struct
 
   /***en Font used for the glyph.  Set to NULL if no font is found for
       the glyph.  */
-  /***ja դ˻ȤեȡĤʤ NULL
-      the glyph.  */
+  /***ja դ˻ȤեȡĤʤ NULL */
+      
   MFont *font;
 
   /***en Character ranges corresponding to logically previous and next
@@ -761,8 +756,8 @@ typedef struct
       next_from because they must be the same as the members from and
       to respectively.  */
   /***ja ŪΥդбʸϰϡ prev_to 
-      next_from ϡ줾 from to ƱǤʤƤϤʤʤ
-      פǤ뤳Ȥա  */
+      next_from ϡ줾 from  to ƱǤϤʤΤ
+      פǤ롣  */
   int prev_from, next_to;
 
   /***en Character ranges corresponding to visually left and right
@@ -789,8 +784,8 @@ typedef struct
 /***ja
     @brief դˡȥեȤ˴ؤη.
 
-    #MDrawGlyph ϥդˡȥեȤ˴ؤޤ๽¤ΤǤ
-    롣mdraw_glyph_list () ϤѤ롣  */
+    #MDrawGlyph ϥդˡȥեȤ˴ؤޤ๽¤ΤǤ롣
+    mdraw_glyph_list () ϤѤ롣  */
 
 typedef struct
 {
@@ -817,8 +812,7 @@ typedef struct
 
   /***en Font used for the glyph.  Set to NULL if no font is found for
       the glyph.  */
-  /***ja դ˻ȤեȡĤʤ NULL
-      the glyph.  */
+  /***ja դ˻ȤեȡĤʤ NULL  */
   MFont *font;
 
   /***en Type of the font.  One of Mx, Mfreetype, Mxft.  */
@@ -830,8 +824,8 @@ typedef struct
       <font_type> member is Mfreetype, and (XftFont *) if <font_type>
       member is Mxft.  */
   /***ja եȤι¤ΤؤΥݥ󥿡ºݤη <font_type> Ф
-      Mx ʤ (XFontStruct *) Mfreetype ʤ FT_FaceMxft ʤ
-      (XftFont *) */
+      Mx ʤ (XFontStruct *) Mfreetype ʤ FT_FaceMxft 
+      ʤ (XftFont *) */
   void *fontp;
 
 } MDrawGlyph;
@@ -839,7 +833,7 @@ typedef struct
 /*=*/
 
 /***en
-    @brief Type of text items.
+    @brief Type of textitems.
 
     The type #MDrawTextItem is for @e textitem objects.
     Each textitem contains an M-text and some other information to
@@ -849,8 +843,7 @@ typedef struct
     @brief textitem η.
 
     #MDrawTextItem  @e ƥȥƥ ֥ѤηǤ롣
-    ƥȥƥϡ1Ĥ M-text ȡɽ椹뤿γƼ
-    ޤǤ롣
+    ƥƥȥƥϡ 1 Ĥ M-text ȡɽ椹뤿ξޤǤ롣
 
     @latexonly \IPAlabel{MTextItem} @endlatexonly  */
 
@@ -869,8 +862,7 @@ typedef struct
       Mnil, overrides the same property of face(s) specified as a text
       property in <mt>.  */
   /***ja ե֥ȤؤΥݥ󥿡եγƥץѥƥ 
-      Mnil Ǥʤ <mt> ǻꤵ줿եƱץѥƥͥ
-      */
+      Mnil Ǥʤ <mt> ǻꤵ줿եƱץѥƥͥ褹*/
   MFace *face;
 
   /***en Pointer to a draw control object.  The M-text <mt> is drawn
@@ -955,8 +947,8 @@ extern MInputDriver minput_gui_driver;
 /***ja
     @brief ؿ minput_create_ic () ΰη.
 
-    #MInputGUIArgIC ϡؿ minput_create_ic () ϥ᥽åɤ
-    ϥƥȤݤΡ $ARG ѤηǤ롣  */
+    #MInputGUIArgIC ϡؿ minput_create_ic () 
+    ϥ᥽åɤϥƥȤݤΡ $ARG ѤηǤ롣  */
 
 typedef struct
 {
