@@ -50,11 +50,10 @@
     m17n 饤֥ꥨ顼ɤ
 
     饤֥δؿǤʤȤȤ˸ƤФ줿ˤϡѿ 
-    #merror_code 򤳤ͤΤɤ줫˥åȤ롣٤ƤΥ顼
-    Ǥ롣
+    #merror_code 򤳤ͤΤɤ줫˥åȤ롣٤ƤΥ顼ɤǤ롣
 
-    ƥ顼κݤˤϡѿ #m17n_memory_full_handler λ
-    ؿͤΤΤɤ줫ȤƸƤФ롣
+    ƥ顼κݤˤϡѿ #m17n_memory_full_handler 
+    λؤؿͤΤΤɤ줫ȤƸƤФ롣
     */
 
 enum MErrorCode
@@ -81,6 +80,7 @@ enum MErrorCode
     MERROR_FONT,
     MERROR_FONTSET,
     MERROR_FONT_OTF,
+    MERROR_FONT_X,
     MERROR_FONT_FT,
     MERROR_IM,
     MERROR_DB,
