@@ -73,8 +73,8 @@ extern char *mdatabase_dir;
 /***ja 
     @brief ǡ١η.
 
-    #MDatabase ϥǡ١֥Ѥι¤ΤǤ롣¤
-    ϥץꥱץफϸʤ
+    #MDatabase ϥǡ١֥Ѥι¤ΤǤ롣
+    ¤ϥץꥱץफϸʤ
     */
 
 typedef struct MDatabase MDatabase;
@@ -241,8 +241,7 @@ enum MConversionResult
 
     /***en On encoding, the source contains a character that cannot be
 	encoded by the specified coding system. */
-    /***ja 󥳡ɻΥɷϤǥ󥳡ɤǤʤʸ
-	˴ޤޤƤ. */
+    /***ja 󥳡ɻΥɷϤǥ󥳡ɤǤʤʸ˴ޤޤƤ. */
     MCONVERSION_RESULT_INVALID_CHAR,
 
     /***en On decoding, the source ends with an incomplete byte sequence. */
@@ -293,20 +292,18 @@ typedef struct
       with "<M+XXXXXX>" (otherwise).  */
 
   /***ja 
-      ̩ѴɬפǤʤͤ 0 ʳˤ롣ǥեȤǤϡ
-      Ѵϸ̩Ǥ롣
+      ̩ѴɬפǤʤͤ 0 ʳˤ롣
+      ǥեȤǤϡѴϸ̩Ǥ롣
 
-      Ѵ̩ȤϡǥɤκݤˤϺǽʥХȤǥС
-      ߤޤ뤳ȡ󥳡ɤκݤˤѴ륳ɷϤǥݡȤ
-      ʤǽʸǥСߤޤ뤳Ȥؤξ硢
-      @c MConverter-\>result Ϥ줾 @c
+      Ѵ̩ȤϡǥɤκݤˤϺǽʥХȤǥСߤޤ뤳ȡ
+      󥳡ɤκݤˤѴ륳ɷϤǥݡȤʤǽʸǥСߤޤ뤳Ȥؤ
+      ξ硢@c MConverter-\>result Ϥ줾 @c
       MCONVERSION_RESULT_INVALID_BYTE @c
       MCONVERSION_RESULT_INVALID_CHAR Ȥʤ롣
 
-      Ѵ̩ǤʤˤϡǥɤκݤʥХȤϤΥХ
-      Τޤ޻Ĥ롣ޤ󥳡ɤκݤˤϡʸ Unicode ʸ
-      ˤ "<U+XXXX>" ˡǤʤˤ "<M+XXXXXX>" ֤
-      롣  */
+      Ѵ̩ǤʤˤϡǥɤκݤʥХȤϤΥХȤΤޤ޻Ĥ롣
+      ޤ󥳡ɤκݤˤϡʸ Unicode ʸξˤ "<U+XXXX>"
+      ˡǤʤˤ "<M+XXXXXX>" ֤롣  */
 
   int lenient;
 
@@ -338,24 +335,20 @@ typedef struct
 
 /***ja 
 
-     ХνüΥ֥åǥɤݡޤʸνüΥ
-     å򥨥󥳡ɤݤϡͤ 0 ʳˤ롣ͤϰʲΤ褦
-     Ѵ˱ƶ롣
+     ХνüΥ֥åǥɤݡޤʸνüΥ֥å򥨥󥳡ɤݤϡͤ
+     0 ʳˤ롣ͤϰʲΤ褦Ѵ˱ƶ롣
 
-     ǥǥ󥰤κݤ˺ǸοХȤХȥ󥹤
-     ˤû硧
+     ǥǥ󥰤κݤ˺ǸοХȤХȥ󥹤ˤû硧
 
-       ͤ 0 ǤʤСѴϤΥ󥹤κǽΥХȤˤơ
-       顼 (MCONVERSION_RESULT_INVALID_BYTE) ǽ롣
+       ͤ 0 ǤʤСѴϤΥ󥹤κǽΥХȤˤơ顼
+       (MCONVERSION_RESULT_INVALID_BYTE) ǽ롣
 
-       ͤ 0 ʤСѴƽ롣οХȤϥ꡼
-       ФȤƥСݻ졢³ѴԤݤѴХ
-       դ롣
+       ͤ 0 ʤСѴƽ롣
+       οХȤϥ꡼ФȤƥСݻ졢Ѵ³ԤݤѴХդ롣
 
      󥳡ǥ󥰤κݤ˥ɷϤʸ̮¸ξ硢
 
-       ͤ 0 ǤʤСƥȤǽ᤹ΥХ󤬥
-       ʸȤʤѴη뤳Ȥ롣
+       ͤ 0 ǤʤСƥȤǽ᤹ΥХ󤬥ʸȤʤѴη뤳Ȥ롣
 
        ͤ 0 ʤСΤ褦ʥХʤ
 
@@ -376,7 +369,7 @@ typedef struct
       The following three members are to report the result of the
       conversion.  */
   /***ja
-      ʲΣĤΥФѴη̤𤹤롣  */
+      ʲΣĤΥФѴη̤ɽΤΤǤ롣  */
 
   /***en
       Number of characters most recently decoded or encoded. */
@@ -408,10 +401,8 @@ typedef struct
       and at least 256 bytes of memory space can be used.  */
 
   /***ja
-      Ѵξ˴ؤξƤϥɷϤΥפˤ
-      ưۤʤ롣@c status ϤɤΤ褦ʷؤΥ㥹ȤФƤʤ
-      ˥ꥢ饤󤵤Ƥꡢޤ256ХȤΥΰ褬
-      褦ˤʤäƤ롣  */
+      Ѵξ˴ؤξƤϥɷϤΥפˤäưۤʤ롣
+      @c status ϤɤΤ褦ʷؤΥ㥹ȤФƤʤ褦˥ꥢ饤󤵤Ƥꡢޤ256ХȤΥΰ褬Ȥ褦ˤʤäƤ롣  */
 
   union {
     void *ptr;
@@ -422,8 +413,7 @@ typedef struct
   /***en
       This member is for internally use only.  An application program
       should never touch it.  */
-  /***ja ΥФŪ˻Ѥ졢ץꥱץϿ
-      ƤϤʤʤ */
+  /***ja ΥФŪ˻Ѥ졢ץꥱץϿƤϤʤʤ */
   void *internal_info;
 } MConverter;
 /*=*/
@@ -446,12 +436,10 @@ enum MCodingType
 	routines of this type.  */
 
     /***ja
-	ΥפΥɷϤʸåȤľܥݡȤ롣ʸ
-	ȤμȤϡʸåȤǰʸɽ뤿ɬפʥХ
-	ȿǤꡢХʸΥɥݥȤľɽ魯
+	ΥפΥɷϤʸåȤľܥݡȤ롣
+	ʸåȤμȤϡʸåȤǰʸɽ뤿ɬפʥХȿǤꡢХʸΥɥݥȤľɽ
 
-	m17n 饤֥ϤΥѤΥǥեȤΥ󥳡ɡǥ
-	ɥ롼ƥ󶡤롣  */
+	m17n 饤֥ϤΥѤΥǥեȤΥ󥳡ɡǥɥ롼ƥ󶡤롣  */
 
     MCODING_TYPE_CHARSET,
 
@@ -463,11 +451,9 @@ enum MCodingType
 	routines of this type.  */
 
     /***ja
-	ΥפΥɷϤϡUTF  (UTF-8, UTF-16, UTF-32) ΥХ
-	򥵥ݡȤ롣
+	ΥפΥɷϤϡUTF  (UTF-8, UTF-16, UTF-32) ΥХ򥵥ݡȤ롣
 
-	m17n 饤֥ϤΥѤΥǥեȤΥ󥳡ɡǥ
-	ɥ롼ƥ󶡤롣  */
+	m17n 饤֥ϤΥѤΥǥեȤΥ󥳡ɡǥɥ롼ƥ󶡤롣  */
 
     MCODING_TYPE_UTF,
 
@@ -483,8 +469,7 @@ enum MCodingType
         ΥפΥɷϤϡISO-2022 ϤΥХ򥵥ݡȤ롣
 	ƥɷϤι¤ξܺ٤ @c MCodingInfoISO2022 ǻꤵ롣
 
-	m17n 饤֥ϤΥѤΥǥեȤΥ󥳡ɡǥ
-	ɥ롼ƥ󶡤롣  */
+	m17n 饤֥ϤΥѤΥǥեȤΥ󥳡ɡǥɥ롼ƥ󶡤롣  */
 
     MCODING_TYPE_ISO_2022,
 
@@ -497,12 +482,9 @@ enum MCodingType
 	application program.  */
 
     /***ja
-	ΥפΥɷϤϡ¾ι¤ΥХΤΤΤǤ
-	롣
+	ΥפΥɷϤϡ¾ι¤ΥХΤΤΤǤ롣
 
-	m17n 饤֥ϤΥѤΥ󥳡ɡǥɥ롼ƥ
-        󶡤ʤΤǡץꥱץ¦ǽ
-        ɬפ롣  */
+	m17n 饤֥ϤΥѤΥ󥳡ɡǥɥ롼ƥ󶡤ʤΤǡץꥱץ¦ǽɬפ롣  */
 
     MCODING_TYPE_MISC
   };
@@ -539,10 +521,10 @@ enum MCodingFlagISO2022
 
     /***en
 	Use the non-standard 4 bytes format for designation sequence
-	for charsets JISX0208.1978, GB2312, and JISX0208.1983.  */
+	for charsets JISX0208-1978, GB2312, and JISX0208-1983.  */
     /***ja
-	JISX0208.1978, GB2312, JISX0208.1983 ʸФؼ
-	󥹤Ȥơɸ4ХȷѤ롣 */
+	JISX0208-1978, GB2312, JISX0208-1983 
+	ʸФؼ󥹤Ȥơɸ4ХȷѤ롣 */
 
     MCODING_ISO_LONG_FORM =		0x8,
 
@@ -550,16 +532,16 @@ enum MCodingFlagISO2022
 	On encoding, unless explicitly specified, designate charsets
 	to G0.  */
     /***ja 
-        󥳡ɤκݡä˻ꤵʤ¤ꡢʸ G0 ˻ؼ
-	롣*/
+        󥳡ɤκݡä˻ꤵʤ¤ꡢʸ G0 
+	˻ؼ롣*/
     MCODING_ISO_DESIGNATION_G0 =		0x10,
 
     /***en
 	On encoding, unless explicitly specified, designate charsets
 	except for ASCII to G1.  */
     /***ja
-	󥳡ɤκݡä˻ꤵʤ¤ꡢASCII ʳʸ G1 
-	˻ؼ롣*/
+	󥳡ɤκݡä˻ꤵʤ¤ꡢASCII ʳʸ 
+	G1 ˻ؼ롣*/
     MCODING_ISO_DESIGNATION_G1 =		0x20,
 
     /***en
@@ -576,8 +558,8 @@ enum MCodingFlagISO2022
 	ESC % G ... ESC % @@ .  On decoding, handle those escape
 	sequences.  */
     /***ja
-	󥳡ɤκݡISO-2022 ˹פʤʸ ESC % / ... ǥ
-	󥳡ɤ롣ݡȤƤʤ Unicode ʸ ESC % G ...
+	󥳡ɤκݡISO-2022 ˹פʤʸ ESC % / ... 
+	ǥ󥳡ɤ롣ݡȤƤʤ Unicode ʸ ESC % G ...
 	ESC % @@ ǥ󥳡ɤ롣
 	ǥɤκݡΥס󥹤᤹롣  */
     MCODING_ISO_DESIGNATION_CTEXT_EXT =	0x80,
@@ -617,8 +599,8 @@ enum MCodingFlagISO2022
 	On encoding, if a charset has revision number, produce escape
 	sequences to specify the number.  */
     /***ja
-	󥳡ɤκݡʸåȤ revision number Ф
-	ɽ魯ץ󥹤롣        */
+	󥳡ɤκݡʸåȤ revision number 
+	Фɽ魯ץ󥹤롣        */
     MCODING_ISO_REVISION_NUMBER =	0x2000,
 
     /***en
@@ -641,8 +623,7 @@ enum MCodingFlagISO2022
 /***ja 
     @brief #MCODING_TYPE_ISO_2022 פΥɷϤɬפղþѹ¤.
 
-    MCODING_TYPE_ISO_2022 פΥɷϤɬפղþѤݻ
-    ι¤Ρ
+    MCODING_TYPE_ISO_2022 פΥɷϤɬפղþѤݻ뤿ι¤Ρ
 
     @latexonly \IPAlabel{MCodingInfoISO2022} @endlatexonly  */
 
@@ -654,9 +635,9 @@ typedef struct
       means no code extension element is invoked to that plane.  */
 
   /***ja
-      ƿ޷ʸΰ (Graphic Left  Graphic Right) ˸ƤӽФƤ
-      롢ISO2022 ĥǤֹΥơ֥롣-1 Ϥΰˤɤ
-      ĥǤƤӽФƤʤȤ򼨤   */
+      ƿ޷ʸΰ (Graphic Left  Graphic Right) ˸ƤӽФƤ롢
+      ISO2022 ĥǤֹΥơ֥롣-1 
+      ϤΰˤɤĥǤƤӽФƤʤȤ򼨤   */
 
   int initial_invocation[2];
 
@@ -676,15 +657,15 @@ typedef struct
 
   /***ja
       ĥǤΥơ֥롣NܤǤϡ$CHARSET_NAMES  N 
-      ʸåȤб롣$CHARSET_NAMES ϴؿ 
+      ܤʸåȤб롣$CHARSET_NAMES ϴؿ 
       mconv_define_coding () ΰȤʤ롣
 
-      ͤ 0..3 ä顢бʸåȤ G0..G3 Τ줾˻ؼ
-      뤳ȡ֤ǤǤ G0..G3 ˻ؼƤ뤳Ȥ̣
-      롣
+      ͤ 0..3 ä顢бʸåȤ G0..G3 
+      Τ줾˻ؼ뤳ȡޤ֤ǤǤ G0..G3
+      ˻ؼƤ뤳Ȥ̣롣
 
-      ͤ -4..-1 ä顢бʸåȤ G0..G3 Τ줾˻
-       뤬֤ǤϤɤˤؼƤʤȤ̣롣
+      ͤ -4..-1 ä顢бʸåȤ G0..G3 
+      Τ줾˻ؼ뤳ȡ֤ǤϤɤˤؼƤʤȤ̣롣
         */
 
   char designations[32];
@@ -717,7 +698,7 @@ typedef struct
   /***en
       Specify bits of a code unit.  The value must be 8, 16, or 32.  */
   /***ja
-      Ĺʥӥåȿˡͤ 8, 16, 32 Τ줫  */
+      Ĺʥӥåȿˤλꡣͤ 8, 16, 32 Τ줫  */
   int code_unit_bits;
 
   /***en
@@ -743,18 +724,17 @@ typedef struct
       Ƭ BOM (Хȥޡ) μ갷ꤹ롣ͤ 0,
       1, 2 Τ줫Ǥꡢ줾ΰ̣ϰʲΤ褦ˤʤ롣
 
-      0: ǥɤκݤ˺ǽ 2 ХȤĴ٤롣⤷줬 BOM Ǥ
-      Сǥ򤽤Ƚꤹ롣ǤʤС @c
+      0: ǥɤκݤ˺ǽ 2 ХȤĴ٤롣⤷줬 BOM 
+      ǤСǥ򤽤Ƚꤹ롣ǤʤС @c
       endian ˽äƥǥꤹ롣󥳡ɤκݤˤ @c
       endian ˽äХƬ BOM դ롣
 
       1: ǥɤκݡǽ 2 ХȤ BOM Ȥư鷺ǥ
-       @c endian Ƚꤹ롣󥳡ɤκݤˤϡBOM Ϥ
+      @c endian Ƚꤹ롣󥳡ɤκݤˤϡBOM Ϥ
       @c endian ˱Х롣
 
-      2: ǥɤκݤ˺ǽ2ХȤ BOMȤư˽äƥ
-      ǥȽꤹ롣󥳡ɤκݤˤ @c endian ˱Х
-      Ƭ BOM դ롣  */
+      2: ǥɤκݤ˺ǽ2ХȤ BOMȤư˽äƥǥȽꤹ롣
+      󥳡ɤκݤˤ @c endian ˱ХƬ BOM դ롣  */
   int bom;
 
   /***en
@@ -763,8 +743,8 @@ typedef struct
 
       If <code_unit_bits> is 8, the value has no meaning.  */
   /***ja
-      ǥΥפꤹ롣ͤ 0  1 Ǥꡢ0 ʤХ
-      륨ǥ1 ʤХӥåǥǤ롣
+      ǥΥפꤹ롣ͤ 0  1 Ǥꡢ0 
+      ʤХȥ륨ǥ1 ʤХӥåǥǤ롣
 
       <code_unit_bits>  8 ξˤϡͤϰ̣ʤ
   */
@@ -785,7 +765,8 @@ extern MSymbol mconv_resolve_coding (MSymbol symbol);
 
 extern int mconv_list_codings (MSymbol **symbols);
 
-extern MConverter *mconv_buffer_converter (MSymbol coding, unsigned char *buf,
+extern MConverter *mconv_buffer_converter (MSymbol coding,
+					   const unsigned char *buf,
 					   int n);
 
 extern MConverter *mconv_stream_converter (MSymbol coding, FILE *fp);
@@ -795,13 +776,13 @@ extern int mconv_reset_converter (MConverter *converter);
 extern void mconv_free_converter (MConverter *converter);
 
 extern MConverter *mconv_rebind_buffer (MConverter *converter,
-					unsigned char *buf, int n);
+					const unsigned char *buf, int n);
 
 extern MConverter *mconv_rebind_stream (MConverter *converter, FILE *fp);
 
 extern MText *mconv_decode (MConverter *converter, MText *mt);
 
-MText *mconv_decode_buffer (MSymbol name, unsigned char *buf, int n);
+MText *mconv_decode_buffer (MSymbol name, const unsigned char *buf, int n);
 
 MText *mconv_decode_stream (MSymbol name, FILE *fp);   
 
@@ -844,12 +825,10 @@ extern MText *mconv_gets (MConverter *converter, MText *mt);
 /***ja
     @brief @c MLocale ¤.
 
-    @c MLocale ¤Τϡ̾졢ϰ衢ǥե
-    ɥåȡб륳ɷϤ˴ؤݻ뤿
-    롣
+    @c MLocale ¤Τϡ̾졢ϰ衢ǥեɥåȡб륳ɷϤ˴ؤݻ뤿Ѥ롣
 
-    ι¤ΤƤϼ˰¸롣 ¤ϥץꥱץ
-    फϸʤ  */
+    ι¤ΤƤϼ˰¸롣 
+    ¤ϥץꥱץफϸʤ  */
 
 /***
     @seealso
@@ -860,6 +839,7 @@ typedef struct MLocale MLocale;
 /*=*/
 
 extern MSymbol Mlanguage;
+extern MSymbol Miso639_1, Miso639_2;
 extern MSymbol Mterritory;
 extern MSymbol Mmodifier;
 extern MSymbol Mcodeset;
@@ -912,9 +892,8 @@ typedef struct MInputContext MInputContext;
 /***ja
     @brief ϥ᥽åɥХåؿη.
 
-    ϥ᥽åɤƤФ륳ХåؿηǤ롣#IC ϥ
-    ƥȤؤΥݥ󥿡#COMMAND ϴؿƤФ륳Хå̾
-    Ǥ롣  */
+    ϥ᥽åɤƤФ륳ХåؿηǤ롣#IC 
+    ϥƥȤؤΥݥ󥿡#COMMAND ϴؿƤФ륳Хå̾Ǥ롣  */
 
 typedef void (*MInputCallbackFunc) (MInputContext *ic, MSymbol command);
 /*=*/
@@ -928,8 +907,7 @@ typedef void (*MInputCallbackFunc) (MInputContext *ic, MSymbol command);
 /***ja
     @brief ϥɥ饤ѹ¤.
 
-    @c MInputDriver ϡϥ᥽åɤ갷ؿޤϥ᥽åɥ
-    饤Фι¤ΤηǤ롣  */
+    @c MInputDriver ϡϥ᥽åɤ갷ؿޤϥ᥽åɥɥ饤Фι¤ΤηǤ롣  */
 
 typedef struct MInputDriver
 {
@@ -945,11 +923,11 @@ typedef struct MInputDriver
   /***ja
       @brief ϥ᥽åɤ򥪡ץ󤹤.
 
-      δؿϡϥ᥽å $IM 򥪡ץ󤹤롣$IM  <info> ʳ
-      СåȤ줿ǡؿ minput_open_im () Ƥ
-      롣$IM 򥪡ץǤ 0 򡢤Ǥʤ -1֤δ
-       $IM->info ꤷơ¾Υɥ饤дؿ黲Ȥ
-      ݻ뤳ȤǤ롣
+      δؿϡϥ᥽å $IM 򥪡ץ󤹤롣$IM  <info> 
+      ʳСåȤ줿ǡؿ minput_open_im () 
+      ƤФ롣$IM 򥪡ץǤ 0 򡢤Ǥʤ -1֤
+      δؿ $IM->info 
+      ꤷơ¾Υɥ饤дؿ黲Ȥݻ뤳ȤǤ롣
       */
 
   int (*open_im) (MInputMethod *im);
@@ -967,9 +945,10 @@ typedef struct MInputDriver
       @brief ϥ᥽åɤ򥯥.
 
       δؿϡϥ᥽å $IM 򥯥롣ؿ 
-      minput_close_im () ƤФ롣ϥ᥽åɤΥ٤
-      λǡ$IM->info ˳ƤƤ()
-      ٤Ƴ롣$IM ¾ΥФ˱ƶͿƤϤʤʤ
+      minput_close_im () ƤФ롣
+      ϥ᥽åɤΥ٤ƽλǡδؿ$IM->info 
+      ˳ƤƤ()٤Ƴ롣
+      $IM ¾ΥФ˱ƶͿƤϤʤʤ
         */
 
   void (*close_im) (MInputMethod *im);
@@ -987,11 +966,11 @@ typedef struct MInputDriver
   /***ja
       @brief ϥƥȤ.
 
-      δؿϥƥ $IC 롣$IC  <info> ʳ
-      СåȤ줿ǡؿ minput_create_ic () Ƥ
-      롣$IC Ǥ 0 򡢤Ǥʤ -1 ֤δؿ
-       $IC->info ꤷơ¾Υɥ饤дؿ黲Ȥ
-      뤳ȤǤ롣  */
+      δؿϥƥ $IC 롣
+      $IC  <info> ʳСåȤ줿ǡؿ
+      minput_create_ic () ƤФ롣
+      $IC Ǥ 0 򡢤Ǥʤ -1 ֤
+      δؿ $IC->info ꤷơ¾Υɥ饤дؿ黲Ȥݻ뤳ȤǤ롣  */
 
 
   int (*create_ic) (MInputContext *ic);
@@ -1009,7 +988,7 @@ typedef struct MInputDriver
       @brief ϥƥȤ˲.
 
       ؿ minput_destroy_ic () ƤФ졢ϥƥ $IC 
-      롣ϥƥȤ˲٤ƽλǡ$IC->info 
+      ˲롣ϥƥȤ˲٤ƽλǡ$IC->info 
       ˳ƤƤ()٤Ƴ롣$IC 
       ¾ΥФ˱ƶͿƤϤʤʤ  */
 
@@ -1041,16 +1020,16 @@ typedef struct MInputDriver
       $KEY, $ARG ϴؿ minput_filter () ΤΤƱ
 
       δؿ $KEY $IC ֤򹹿롣 $KEY 
-      ᥽åɤ˵ۼƥƥȤʤäˤϡ 1 
-      Ǥʤ 0 ֤
+      ϥ᥽åɤ˵ۼƥƥȤʤäˤϡ 
+      1 ֤Ǥʤ 0 ֤
 
        <callback> ɬפǤС$IC->status, $IC->preedit,
       $IC->cursor_pos, $IC->ncandidates, $IC->candidates,
       $IC->produced 򹹿Ǥ롣
 
       $ARG ΰ̣ϥ᥽åɥɥ饤Ф˰¸롣 @c
-      minput_default_driver ޤ @c minput_gui_driver 򻲾Ȥ
-      ȡ */
+      minput_default_driver ޤ @c minput_gui_driver 
+      򻲾ȤΤȡ */
 
   int (*filter) (MInputContext *ic, MSymbol key, void *arg);
 
@@ -1071,9 +1050,9 @@ typedef struct MInputDriver
       @brief ϥƥȤƥȤγ.
 
       ؿ minput_lookup () ƤФ졢ϥƥ $IC 
-      ƥȤ򸡺롣ϥ $KEY ˤäƥ
-      СM-text $MT ɲä롣 $KEY ϥ᥽å $IC ˤ
-       0 ֤Ǥʤ 1 ֤
+      ƥȤ򸡺롣ϥ $KEY 
+      ˤäƥȤСM-text $MT ɲä롣 $KEY
+      ϥ᥽å $IC ˤä 0 ֤Ǥʤ 1 ֤
 
       $ARG ΰ̣ϥ᥽åɥɥ饤Ф˰¸롣 @c
       minput_default_driver ޤ @c minput_gui_driver 򻲾Ȥ
@@ -1089,7 +1068,8 @@ typedef struct MInputDriver
       #Minput_preedit_done, #Minput_status_start, #Minput_status_draw,
       #Minput_status_done, #Minput_candidates_start,
       #Minput_candidates_draw, #Minput_candidates_done,
-      #Minput_set_spot, #Minput_toggle, #Minput_reset.
+      #Minput_set_spot, #Minput_toggle, #Minput_reset,
+      #Minput_get_surrounding_text, #Minput_delete_surrounding_text.
       Values are functions of type #MInputCallbackFunc.  */
   /***ja
       @brief ХåؿΥꥹ.
@@ -1099,7 +1079,8 @@ typedef struct MInputDriver
       #Minput_preedit_done, #Minput_status_start, #Minput_status_draw,
       #Minput_status_done, #Minput_candidates_start,
       #Minput_candidates_draw, #Minput_candidates_done,
-      #Minput_set_spot, #Minput_toggle, #Minput_reset 
+      #Minput_set_spot, #Minput_toggle, #Minput_reset,
+      #Minput_get_surrounding_text, #Minput_delete_surrounding_text
       ͤ#MInputCallbackFunc δؿ  */
   MPlist *callback_list;
 
@@ -1125,6 +1106,13 @@ extern MSymbol Minput_candidates_done;
 extern MSymbol Minput_set_spot;
 extern MSymbol Minput_toggle;
 extern MSymbol Minput_reset;
+extern MSymbol Minput_get_surrounding_text;
+extern MSymbol Minput_delete_surrounding_text;
+
+/** Symbols for special input key event.  */
+extern MSymbol Minput_focus_move;
+extern MSymbol Minput_focus_in;
+extern MSymbol Minput_focus_out;
 
 /***en
     @brief Structure of input method.
@@ -1140,16 +1128,15 @@ struct MInputMethod
 {
   /***en Which language this input method is for.  The value is @c
       Mnil if the input method is foreign.  */
-  /***ja ɤθѤϥ᥽åɤϥ᥽åɤΤΤǤ
-      ˤͤȤ @c Mnil ġ  */
+  /***ja ɤθѤϥ᥽åɤ
+      ϥ᥽åɤΤΤǤͤ @c Mnil   */
   MSymbol language;
 
   /***en Name of the input method.  If the input method is foreign, it
       must has a property of key @c Minput_driver and the value must be a
       pointer to a proper input method driver.  */
   /***ja ϥ᥽åɤ̾᥽åɤǤˤϡ@c
-      Minput_driver 򥭡ȤץѥƥͤŬڤϥ
-      åɥɥ饤ФؤΥݥ󥿤ǤʤƤϤʤʤ*/
+      Minput_driver 򥭡ȤץѥƥͤŬڤϥ᥽åɥɥ饤ФؤΥݥ󥿤ǤʤƤϤʤʤ*/
   MSymbol name;
 
   /***en Input method driver of the input method.  */
@@ -1166,6 +1153,23 @@ struct MInputMethod
   void *info;
 };
 
+/*=*/
+
+/***en
+    @brief Bit-masks to specify how candidates of input method is changed.  */
+
+/***ja
+    @brief ϥ᥽åɤϸ䤬ɤѹ줿򼨤ӥåȥޥ.  */
+
+enum MInputCandidatesChanged
+  {
+    MINPUT_CANDIDATES_LIST_CHANGED = 1,
+    MINPUT_CANDIDATES_INDEX_CHANGED = 2,
+    MINPUT_CANDIDATES_SHOW_CHANGED = 4,
+    MINPUT_CANDIDATES_CHANGED_MAX
+  };
+
+
 /*=*/
 
 /***en
@@ -1177,15 +1181,14 @@ struct MInputMethod
 /***ja
     @brief ϥƥѹ¤.
 
-    @c MInputContext ϡϥƥȥ֥Ѥι¤Τη
-    롣  */
+    @c MInputContext ϡϥƥȥ֥Ѥι¤ΤηǤ롣  */
 
 struct MInputContext
 {
   /***en Backward pointer to the input method.  It is set up be the
       function minput_create_ic ().  */
-  /***ja ϥ᥽åɤؤεեݥ󥿡ؿ minput_create_ic () ˤä
-      ꤵ롣  */ 
+  /***ja ϥ᥽åɤؤεեݥ󥿡ؿ minput_create_ic () 
+      ˤäꤵ롣  */ 
   MInputMethod *im;
 
   /***en M-text produced by the input method.  It is set up by the
@@ -1200,14 +1203,12 @@ struct MInputContext
 
   /***en Flag telling whether the input context is currently active or
       inactive.  The value is set to 1 (active) when the input context
-      is created.  It can be toggled by the function minput_toggle
-      ().  */
-  /***ja ϥƥȤƥ֤ɤ򼨤ե饰ϥ
-      Ȥ줿Ǥͤ 1 ʥƥ֡ˤǤꡢؿ 
+      is created.  It is toggled by the function minput_toggle ().  */
+  /***ja ϥƥȤƥ֤ɤ򼨤ե饰
+      ϥƥȤ줿Ǥͤ 1 ʥƥ֡ˤǤꡢؿ 
       minput_toggle () ˤäƥȥ뤵롣  */
   int active;
 
-
   /***en Spot location and size of the input context.  */
   /***ja ϥƥȤΥݥåȤΰ֤礭.  */
   struct {
@@ -1237,7 +1238,7 @@ struct MInputContext
       internal input method.  They are set by the function
       <im>->driver.filter ().  */
   /***ja ʲΥФλˡϥ᥽åɥɥ饤Фˤäưۤʤ롣
-      ϡϥ᥽åѤϥɥ饤ФФΤǤ롣
+      ʲϡϥ᥽åѤϥɥ饤ФФΤǤ롣
       ϴؿ <im>->driver.filter () ˤäꤵ롣  */
 
   /***en Pointer to extra information that <im>->driver.create_ic ()
@@ -1249,24 +1250,24 @@ struct MInputContext
 
   /***en M-text describing the current status of the input
       context.  */
-  /***ja ϥƥȤθߤξɽ M-text  */
+  /***ja ϥƥȤθߤξ֤ɽ M-text  */
   MText *status;
 
   /***en The function <im>->driver.filter () sets the value to 1 when
       it changes <status>.  */
-  /***ja ؿ <im>->driver.filter () ϡ<status> Ѥͤ 1 
+  /***ja ؿ <im>->driver.filter () ϡ<status> Ѥݤˤͤ 1 
       ꤹ롣  */
   int status_changed;
 
   /***en M-text containing the current preedit text.  The function
       <im>->driver.filter () sets the value.  */
   /***ja ߤ preedit ƥȤޤ M-textؿ 
-      <im>->driver.filter () ͤꤹ롣  */ 
+      <im>->driver.filter () ˤäꤵ롣  */ 
   MText *preedit;
 
   /***en The function <im>->driver.filter () sets the value to 1 when
       it changes <preedit>.  */
-  /***ja ؿ <im>->driver.filter () ϡ<preedit> Ѥͤ 
+  /***ja ؿ <im>->driver.filter () ϡ<preedit> Ѥݤˤͤ 
       1 ꤹ롣  */
   int preedit_changed;
 
@@ -1274,21 +1275,70 @@ struct MInputContext
   /***ja <preedit>Υ  */
   int cursor_pos;
 
+  /***en The function <im>->driver.filter () sets the value to 1 when
+      it changes <cursor_pos>.  */
+  /***ja ؿ <im>->driver.filter () ϡ<cursor_pos> Ѥݤˤͤ 
+      1 ꤹ롣  */
   int cursor_pos_changed;
 
-  /***en Array of the current candidate texts.  */
-  /***ja ߤΥƥȸ.  */
+  /***en Plist of the current candidate groups.  Each element is an
+      M-text or a plist.  If an element is an M-text (i.e. the key is Mtext),
+      candidates in that group are characters in the M-text.  If it is
+      a plist (i.e. the key is Mplist), each element is an M-text, and
+      candidates in that group are those M-texts.  */
+  /***ja ߤθ䥰롼פ Plist Ǥ M-text  plist Ǥ롣
+      Ǥ M-text ξʥ Mtext ǤˤˤϡΥ롼פθϤ 
+      M-text γʸǤ롣 Ǥ plist ξʥ Mplist
+      ǤˤˤϡΥꥹȤγǤ M-text Ǥꡢ餬Υ롼פθȤʤ롣  */
   MPlist *candidate_list;
+
+  /***en Index number of the currently selected candidate in all the
+      candidates. The index of the first candidate is 0.  If the
+      number is 8, and the first candidate group contains 7
+      candidates, the currently selected candidate is the second element of the
+      second candidate group.  */
+  /***ja 򤵤Ƥ䤬ǲܤ򼨤ǥå
+      ǽθΥǥå 0ǽθ䥰롼פ˼Ĥθ䤬ޤޤƤꡢͤ
+      8 ʤСߤθܤθ䥰롼פܤǤȤȤˤʤ롣
+      */
   int candidate_index;
+
+  /***en Start and the end positions of the preedit text where
+       <candidate_list> corresponds to.  */
+  /***ja preedit ƥǡ<candidate_list>бǽȺǸΰ֡
+       */
   int candidate_from, candidate_to;
+
+  /***en Flag telling whether the current candidate group must be
+      shown or not.  The function <im>->driver.filter () sets the
+      value to 1 when an input method required to show candidates, and
+      sets the value to 0 otherwise.  */
+  /***ja ߤθ䥰롼פɽ뤫ɤ򼨤ե饰
+      ؿ <im>->driver.filter () ϡϥ᥽åɤɽ׵ᤷͤ
+      1 ˡʳλ 0 ꤹ롣  */
   int candidate_show;
 
-  /***en The function <im>->driver.filter () sets the value to 1 when
-      it changes one of the above members.  */
-  /***ja ؿ <im>->driver.filter () ϡ嵭ΥФΣĤѤ
-      ͤ 1 ꤹ롣  */
+  /***en The function <im>->driver.filter () sets the value to bitwise
+      OR of @c enum @c MInputCandidatesChanged when it changed any of
+      the above members (<candidate_XXX>), and sets the value to 0
+      otherwise.  */
+  /***ja ؿ <im>->driver.filter () ϡ嵭Υ <candidate_XXX>
+      ΣĤǤѹݤˤϡͤ @c enum @c
+      MInputCandidatesChanged Υӥåñ̤Ǥ OR ꤹ롣
+      Ǥʤ 0 ꤹ롣 */
   int candidates_changed;
 
+  /***en Plist that can be freely used by <im>->driver functions.
+      The driver of internal input method uses it to exchange extra
+      arguments and result for callback functions.  The function
+      <im>->driver.create_ic () sets this to an empty plist, and the
+      function <im>->driver.destroy_ic () frees it by using
+      m17n_object_unref ().  */
+  /***ja <im>->driver δؿˤäƼͳ˻ѤǤ plist
+      ϥ᥽åѥɥ饤ФϤ򥳡ХåؿȤΰ
+      μϤ˻Ѥ롣ؿ <im>->driver.create_ic () Ϥ plist
+      ꤹ롣ؿ<im>->driver.destroy_ic () 
+      m17n_object_unref () ѤƤ plist 롣  */
   MPlist *plist;
 };
 
@@ -1317,6 +1367,22 @@ extern void minput_set_spot (MInputContext *ic, int x, int y,
 extern void minput_toggle (MInputContext *ic);
 
 extern void minput_reset_ic (MInputContext *ic);
+
+extern MText *minput_get_description (MSymbol language, MSymbol name);
+
+extern MPlist *minput_get_title_icon (MSymbol language, MSymbol name);
+
+extern MPlist *minput_get_commands (MSymbol language, MSymbol name);
+
+extern int minput_assign_command_keys (MSymbol language, MSymbol name,
+				       MSymbol command, MPlist *keys);
+
+extern MPlist *minput_get_variables (MSymbol language, MSymbol name);
+
+extern int minput_set_variable (MSymbol language, MSymbol name,
+				MSymbol variable, void *value);
+
+
 /*=*/
 /***     @} */
 
