@@ -32,20 +32,40 @@
 
     If the key of a property is a @e managing @e key, its @e value is
     a @e managed @e object.  A property list itself is a managed
-    objects.  */
+    objects.
+
+    If each key of a plist is one of #Msymbol, #Mtext, #Minteger, and
+    #Mplist, the plist is called as @e well-formed and represented by
+    the following notation in the documentation.
+
+@verbatim
+      PLIST ::= '(' ELEMENT * ')'
+
+      ELEMENT ::= INTEGER | SYMBOL | M-TEXT | PLIST
+
+      M-TEXT ::= '"' text data ... '"'
+@endverbatim
+
+    For instance, if a plist has four elements; integer -20, symbol of
+    name "sym", M-text of contents "abc", and plist of integer 10 and
+    symbol of name "another-symbol", it is represented as this:
+
+      (-20 sym "abc" (10 another-symbol))
+
+  */
 /***ja
     @addtogroup m17nPlist
 
     @brief ץѥƥꥹȥ֥ȤȤ˴ؤ API.
 
-    @e ץѥƥꥹ (ޤ @e plist)  0 İʾΥץѥƥΥ
-    ȤǤ롣ץѥƥ @e   @e  ʤ롣ϥܥ
-    Ǥꡢͤ <tt>(void *)</tt> ˥㥹ȤǤΤʤвǤ
-    
+    @e ץѥƥꥹ (ޤ @e plist)  0 
+    İʾΥץѥƥΥꥹȤǤ롣ץѥƥ @e   @e  
+    ʤ롣ϥܥǤꡢͤ <tt>(void *)</tt> 
+    ˥㥹ȤǤΤʤвǤɤ
 
-    ץѥƥΥ @e  ʤС @e  @e 
-    ֥ Ǥ롣ץѥƥꥹȼΤ֥ȤǤ
-    롣  */
+    ץѥƥΥ @e  ʤС @e   @e 
+    @e ֥ 
+    Ǥ롣ץѥƥꥹȼΤ֥ȤǤ롣  */
 
 /*=*/
 
@@ -151,12 +171,9 @@ get_byte (MStream *st)
   return st->buffer[0];
 }
 
-#define GETC(st)	\
-  ((st)->p < (st)->pend ? *(st)->p++ : get_byte (st))
+#define GETC(st) ((st)->p < (st)->pend ? *(st)->p++ : get_byte (st))
 
-
-#define UNGETC(c, st)	\
-  (*--(st)->p = (c))
+#define UNGETC(c, st) (--((st)->p))
 
 /** Mapping table for reading a number.  Hexadecimal chars
     (0..9,A..F,a..F) are mapped to the corresponding numbers.
@@ -209,35 +226,34 @@ read_hexadesimal (MStream *st)
 /** Read an M-text element from ST, and add it to LIST.  Return a list
     for the next element.  */
 
+#define READ_MTEXT_BUF_SIZE 256
+
 static MPlist *
-read_mtext_element (MPlist *plist, MStream *st)
+read_mtext_element (MPlist *plist, MStream *st, int skip)
 {
-  unsigned char buffer[1024];
-  int bufsize = 1024;
-  unsigned char *buf = buffer;
-  int c, i;
+  union {
+    int chars[READ_MTEXT_BUF_SIZE];
+    unsigned char bytes[sizeof (int) * READ_MTEXT_BUF_SIZE];
+  } buffer;
+  unsigned char *bytes = buffer.bytes;
+  int nbytes = sizeof (int) * READ_MTEXT_BUF_SIZE;
+  int *chars = NULL;
+  int nchars = 0;
+  int c, i, j;
 
   i = 0;
   while ((c = GETC (st)) != EOF && c != '"')
     {
-      if (i + MAX_UTF8_CHAR_BYTES >= bufsize)
-	{
-	  bufsize *= 2;
-	  if (buf == buffer)
-	    {
-	      MTABLE_MALLOC (buf, bufsize, MERROR_PLIST);
-	      memcpy (buf, buffer, i);
-	    }
-	  else
-	    MTABLE_REALLOC (buf, bufsize, MERROR_PLIST);
-	}
+      int is_char = 0;
 
       if (c == '\\')
 	{
 	  c = GETC (st);
 	  if (c == EOF)
 	    break;
-	  if (c == 'x')
+	  if (c == '\n')
+	    continue;
+	  if (c == 'x' || c == 'u')
 	    {
 	      int next_c;
 
@@ -245,18 +261,76 @@ read_mtext_element (MPlist *plist, MStream *st)
 	      next_c = GETC (st);
 	      if (next_c != ' ')
 		UNGETC (next_c, st);
+	      if (c >= 0x80)
+		is_char = 1;
 	    }
 	  else
 	    c = escape_mnemonic[c];
 	}
 
-      buf[i++] = c;
+      if (! skip)
+	{
+	  if (is_char && ! chars)
+	    {
+	      chars = buffer.chars;
+	      for (j = i - 1; j >= 0; j--)
+		chars[j] = bytes[j];
+	      nchars = READ_MTEXT_BUF_SIZE;
+	      if (bytes != buffer.bytes)
+		free (bytes);
+	    }
+
+	  if (chars)
+	    {
+	      if (i + 1 >= nchars)
+		{
+		  nchars *= 2;
+		  if (chars == buffer.chars)
+		    {
+		      MTABLE_MALLOC (chars, nchars, MERROR_PLIST);
+		      memcpy (chars, buffer.chars, sizeof (int) * i);
+		    }
+		  else
+		    MTABLE_REALLOC (chars, nchars, MERROR_PLIST);
+		}
+	      chars[i++] = c;
+	    }
+	  else
+	    {
+	      if (i + MAX_UTF8_CHAR_BYTES >= nbytes)
+		{
+		  nbytes *= 2;
+		  if (bytes == buffer.bytes)
+		    {
+		      MTABLE_MALLOC (bytes, nbytes, MERROR_PLIST);
+		      memcpy (bytes, buffer.bytes, i);
+		    }
+		  else
+		    MTABLE_REALLOC (bytes, nbytes, MERROR_PLIST);
+		}
+	      bytes[i++] = c;
+	    }
+	}
     }
 
-  MPLIST_SET_ADVANCE (plist, Mtext,
-		      mtext__from_data (buf, i, MTEXT_FORMAT_UTF_8, 1));
-  if (buf != buffer)
-    free (buf);
+  if (! skip)
+    {
+      MText *mt;
+
+      if (chars)
+	{
+	  mt = mtext__from_data (chars, i, MTEXT_FORMAT_UTF_32, 1);
+	  if (chars != buffer.chars)
+	    free (chars);
+	}	  
+      else
+	{
+	  mt = mtext__from_data (bytes, i, MTEXT_FORMAT_UTF_8, 1);
+	  if (bytes != buffer.bytes)
+	    free (bytes);
+	}
+      MPLIST_SET_ADVANCE (plist, Mtext, mt);
+    }
   return plist;
 }
 
@@ -284,22 +358,80 @@ read_character (MStream *st, int c)
 }
 
 
+/** Read a symbol element from ST, and add it to LIST.  Return a list
+    for the next element.  */
+
+static MPlist *
+read_symbol_element (MPlist *plist, MStream *st, int c, int skip)
+{
+  unsigned char buffer[1024];
+  int bufsize = 1024;
+  unsigned char *buf = buffer;
+  int i;
+
+  i = 0;
+  while (c != EOF
+	 && c > ' '
+	 && c != ')' && c != '(' && c != '"')
+    {
+      if (i >= bufsize)
+	{
+	  bufsize *= 2;
+	  if (buf == buffer)
+	    {
+	      MTABLE_MALLOC (buf, bufsize, MERROR_PLIST);
+	      memcpy (buf, buffer, i);
+	    }
+	  else
+	    MTABLE_REALLOC (buf, bufsize, MERROR_PLIST);
+	}
+      if (c == '\\')
+	{
+	  c = GETC (st);
+	  if (c == EOF)
+	    break;
+	  c = escape_mnemonic[c];
+	}
+      if (! skip)
+	buf[i++] = c;
+      c = GETC (st);
+    }
+
+  if (c > ' ')
+    UNGETC (c, st);
+  if (! skip)
+    {
+      buf[i] = 0;
+      MPLIST_SET_ADVANCE (plist, Msymbol, msymbol ((char *) buf));
+      if (buf != buffer)
+	free (buf);
+    }
+  return plist;
+}
+
 /** Read an integer element from ST, and add it to LIST.  Return a
     list for the next element.  It is assumed that we have already
     read the character C. */
 
 static MPlist *
-read_integer_element (MPlist *plist, MStream *st, int c)
+read_integer_element (MPlist *plist, MStream *st, int c, int skip)
 {
   int num;
 
-  if (c == '0' || c == '#')
+  if (c == '#')
     {
       c = GETC (st);
-      if (c == 'x')
-	num = read_hexadesimal (st);
-      else
-	num = read_decimal (st, c);
+      if (c != 'x')
+	{
+	  UNGETC (c, st);
+	  return read_symbol_element (plist, st, '#', skip);
+	}
+      num = read_hexadesimal (st);
+    }
+  else if (c == '0')
+    {
+      c = GETC (st);
+      num = (c == 'x' ? read_hexadesimal (st) : read_decimal (st, c));
     }
   else if (c == '?')
     {
@@ -325,57 +457,20 @@ read_integer_element (MPlist *plist, MStream *st, int c)
 	}
     }
   else if (c == '-')
-    num = - read_decimal (st, GETC (st));
+    {
+      c = GETC (st);
+      if (c < '0' || c > '9')
+	{
+	  UNGETC (c, st);
+	  return read_symbol_element (plist, st, '-', skip);
+	}
+      num = - read_decimal (st, c);
+    }
   else
     num = read_decimal (st, c);
 
-  MPLIST_SET_ADVANCE (plist, Minteger, (void *) num);
-  return plist;
-}
-
-/** Read a symbol element from ST, and add it to LIST.  Return a list
-    for the next element.  */
-
-static MPlist *
-read_symbol_element (MPlist *plist, MStream *st)
-{
-  unsigned char buffer[1024];
-  int bufsize = 1024;
-  unsigned char *buf = buffer;
-  int c, i;
-
-  i = 0;
-  while ((c = GETC (st)) != EOF
-	 && c > ' '
-	 && c != ')' && c != '(' && c != '"')
-    {
-      if (i >= bufsize)
-	{
-	  bufsize *= 2;
-	  if (buf == buffer)
-	    {
-	      MTABLE_MALLOC (buf, bufsize, MERROR_PLIST);
-	      memcpy (buf, buffer, i);
-	    }
-	  else
-	    MTABLE_REALLOC (buf, bufsize, MERROR_PLIST);
-	}
-      if (c == '\\')
-	{
-	  c = GETC (st);
-	  if (c == EOF)
-	    break;
-	  c = escape_mnemonic[c];
-	}
-      buf[i++] = c;
-    }
-
-  buf[i] = 0;
-  MPLIST_SET_ADVANCE (plist, Msymbol, msymbol ((char *) buf));
-  if (buf != buffer)
-    free (buf);
-  if (c > ' ')
-    UNGETC (c, st);
+  if (! skip)
+    MPLIST_SET_ADVANCE (plist, Minteger, (void *) num);
   return plist;
 }
 
@@ -387,10 +482,14 @@ read_symbol_element (MPlist *plist, MStream *st)
 	'0'..'9', '-': integer
 	'?': integer representing character code
 	the other ASCII letters: symbol
-*/
+
+   If KEYS is not NULL, it is a plist contains target keys and stop
+   keys.  In this caes, read only a plist whose key has value 1 in
+   KEYS, and return NULL when we encounter a plist whose key has value
+   0 in KEYS while skipping any other elements.  */
 
 static MPlist *
-read_element (MPlist *plist, MStream *st)
+read_element (MPlist *plist, MStream *st, MPlist *keys)
 {
   int c;
 
@@ -411,18 +510,50 @@ read_element (MPlist *plist, MStream *st)
 
       MPLIST_NEW (pl);
       p = pl;
-      while ((p = read_element (p, st)));
-      MPLIST_SET_ADVANCE (plist, Mplist, pl);
+      p = read_element (p, st, NULL);
+      if (keys && p && MPLIST_SYMBOL_P (pl))
+	{
+	  if (MPLIST_TAIL_P (keys))
+	    {
+	      while ((p = read_element (p, st, NULL)));
+	      MPLIST_SET_ADVANCE (plist, Mplist, pl);
+	      return NULL;
+	    }
+	  else
+	    {
+	      MPlist *p0 = keys;
+
+	      MPLIST_FIND (p0, MPLIST_SYMBOL (pl));
+	      if (! MPLIST_TAIL_P (p0) && ! MPLIST_VAL (p0))
+		{
+		  M17N_OBJECT_UNREF (pl);
+		  return NULL;
+		}
+	      while ((p = read_element (p, st, NULL)));
+	      if (! MPLIST_TAIL_P (p0))
+		{
+		  MPLIST_SET_ADVANCE (plist, Mplist, pl);
+		  return NULL;
+		}
+	      else
+		M17N_OBJECT_UNREF (pl);
+	    }
+	}
+      else
+	{
+	  if (p)
+	    while ((p = read_element (p, st, NULL)));
+	  MPLIST_SET_ADVANCE (plist, Mplist, pl);
+	}
       return plist;
     }
   if (c == '"')
-    return read_mtext_element (plist, st);
+    return (read_mtext_element (plist, st, keys ? 1 : 0));
   if ((c >= '0' && c <= '9') || c == '-' || c == '?' || c == '#')
-    return read_integer_element (plist, st, c);
+    return (read_integer_element (plist, st, c, keys ? 1 : 0));
   if (c == EOF || c == ')')
     return NULL;
-  UNGETC (c, st);
-  return read_symbol_element (plist, st);
+  return (read_symbol_element (plist, st, c, keys ? 1 : 0));
 }
 
 void
@@ -534,7 +665,7 @@ dump_plist_element (MPlist *plist, int indent)
     fprintf (stderr, "%x", MPLIST_INTEGER (plist));
   else if (key == Mstring) 
     fprintf (stderr, "\"%s\"", MPLIST_STRING (plist));
-  else if (key == Mplist)
+  else if (MPLIST_NESTED_P (plist))
     {
       fprintf (stderr, "\n%s", prefix);
       mdebug_dump_plist (MPLIST_PLIST (plist), indent);
@@ -551,7 +682,7 @@ mplist__init ()
 {
   int i;
 
-  plist_table.count = 0;
+  M17N_OBJECT_ADD_ARRAY (plist_table, "Plist");
 
   Minteger = msymbol ("integer");
   Mplist = msymbol_as_managing_key ("plist");
@@ -581,7 +712,6 @@ mplist__init ()
 void
 mplist__fini (void)
 {
-  mdebug__report_object ("Plist", &plist_table);
 }
 
 
@@ -606,7 +736,7 @@ mplist__from_plist (MPlist *plist)
       key = MPLIST_SYMBOL (plist);
       plist = MPLIST_NEXT (plist);
       type = MPLIST_KEY (plist);
-      if (type->managing_key)
+      if (type->managing_key && MPLIST_VAL (plist))
 	M17N_OBJECT_REF (MPLIST_VAL (plist));
       MPLIST_SET_ADVANCE (p, key, MPLIST_VAL (plist));
       plist = MPLIST_NEXT (plist);
@@ -644,7 +774,7 @@ mplist__from_alist (MPlist *plist)
 
 
 MPlist *
-mplist__from_file (FILE *fp)
+mplist__from_file (FILE *fp, MPlist *keys)
 {
   MPlist *plist, *pl;
   MStream st;
@@ -654,7 +784,7 @@ mplist__from_file (FILE *fp)
   st.p = st.pend = st.buffer;
   MPLIST_NEW (plist);
   pl = plist;
-  while ((pl = read_element (pl, &st)));
+  while ((pl = read_element (pl, &st, keys)));
   return plist;
 }
 
@@ -703,7 +833,7 @@ mplist__from_string (unsigned char *str, int n)
   st.pend = str + n;
   MPLIST_NEW (plist);
   pl = plist;
-  while ((pl = read_element (pl, &st)));
+  while ((pl = read_element (pl, &st, NULL)));
   return plist;
 }
 
@@ -721,6 +851,55 @@ mplist__serialize (MText *mt, MPlist *plist)
   return 0;
 }
 
+/**en
+    @brief Concatenate two plists.
+
+    The mplist__conc () function concatenates plist $TAIL at the end of
+    plist $PLIST and return $PLIST.  If $TAIL is empty, return $PLIST
+    without modifying it.  */
+
+MPlist *
+mplist__conc (MPlist *plist, MPlist *tail)
+{
+  MPlist *pl;
+
+  if (MPLIST_TAIL_P (tail))
+    return plist;
+  MPLIST_DO (pl, plist);
+  MPLIST_KEY (pl) = MPLIST_KEY (tail);
+  MPLIST_VAL (pl) = MPLIST_VAL (tail);
+  if (MPLIST_KEY (pl)->managing_key)
+    M17N_OBJECT_REF (MPLIST_VAL (pl));
+  tail = MPLIST_NEXT (tail);
+  MPLIST_NEXT (pl) = tail;
+  M17N_OBJECT_REF (tail);
+  return plist;
+}
+
+/*=*/
+/**en
+    @brief Discard a property at the beginning of a property list.
+
+    The mplist__pop_unref () function removes a property at the
+    beginning of property list $PLIST, and if the property value is a
+    managed object, unref it.  As a result, the second key and value
+    of the original $PLIST become the first of those of the new
+    $PLIST.  */
+
+void
+mplist__pop_unref (MPlist *plist)
+{
+  MSymbol key;
+  void *val;
+
+  if (MPLIST_TAIL_P (plist))
+    return;
+  key = MPLIST_KEY (plist);
+  val = mplist_pop (plist);
+  if (key->managing_key)
+    M17N_OBJECT_UNREF (val);
+}
+
 /*** @} */
 #endif /* !FOR_DOXYGEN || DOXYGEN_INTERNAL_MODULE */
 
@@ -734,7 +913,7 @@ mplist__serialize (MText *mt, MPlist *plist)
 /***en
     @brief Symbol whose name is "integer".
 
-    The symbol @c Minteger has the name <tt>"integer"</tt>.  A value
+    The symbol @c Minteger has the name <tt>"integer"</tt>.  The value
     of a property whose key is @c Minteger must be an integer.  */
 /***ja
     @brief "integer" ̾Ȥƻĥܥ.
@@ -754,9 +933,9 @@ MSymbol Minteger;
 /***ja
     @brief "plist" ̾Ȥƻĥܥ.
 
-    ܥ @c Mplist  <tt>"plist"</tt> Ȥ̾ġϴ
-    Ǥ롣 @c Mplist Ǥץѥƥͤ plist Ǥʤ
-    ƤϤʤʤ  */
+    ܥ @c Mplist  <tt>"plist"</tt> 
+    Ȥ̾ġϴǤ롣 @c Mplist 
+    Ǥץѥƥͤ plist ǤʤƤϤʤʤ  */
 
 MSymbol Mplist;
 /*=*/
@@ -771,9 +950,9 @@ MSymbol Mplist;
 /***ja
     @brief "mtext" ̾Ȥƻĥܥ.
 
-    ܥ @c Mtext  <tt>"mtext"</tt> Ȥ̾ĴǤ
-    롣 @c Mtext Ǥץѥƥͤ M-text ǤʤƤϤʤ
-          */
+    ܥ @c Mtext  <tt>"mtext"</tt>
+    Ȥ̾ĴǤ롣 @c Mtext 
+    Ǥץѥƥͤ M-text ǤʤƤϤʤʤ      */
 
 MSymbol Mtext;
 
@@ -792,8 +971,7 @@ MSymbol Mtext;
 /***ja
     @brief ץѥƥꥹȥ֥Ȥ.
 
-    ؿ mplist () Ĺ 0 ο줿ץѥƥꥹȥ֥
-    Ȥ֤
+    ؿ mplist () Ĺ 0 Υץѥƥꥹȥ֥Ȥ򿷤ä֤
 
     @returns
     δؿϿ줿ץѥƥꥹȥ֥Ȥ֤
@@ -826,8 +1004,8 @@ mplist ()
 /***ja
     @brief ץѥƥꥹȤ򥳥ԡ.
 
-    ؿ mplist_copy () ϥץѥƥꥹ $PLIST 򥳥ԡ롣ԡΤ٤Ƥ
-    ͤϥԡ $PLIST ͤƱǤ롣
+    ؿ mplist_copy () ϥץѥƥꥹ $PLIST 
+    򥳥ԡ롣ԡΤ٤Ƥͤϥԡ $PLIST ͤƱǤ롣
 
     @return
     δؿϿ줿$PLIST ΥԡǤץѥƥꥹȤ֤    
@@ -869,18 +1047,18 @@ mplist_copy (MPlist *plist)
     @brief ץѥƥꥹΥץѥƥͤꤹ.
 
     ؿ mplist_put () ϥץѥƥꥹ $PLIST Ϥᤫõơ
-     $KEY Ǥץѥƥ򸫤Ĥ롣ĤСͤ $VALUE 
-    ѹ롣ĤʤС $KEY ͤ $VALUE Ǥ뿷
-    ץѥƥ$PLIST ɲä롣$KEY  $VAL Ф¤
-    ĤƤϡmplist_add () 򻲾ȡ
+    $KEY Ǥץѥƥ򸫤Ĥ롣ĤСͤ $VALUE 
+    ѹ롣ĤʤС $KEY ͤ $VALUE 
+    Ǥ뿷ץѥƥ $PLIST ɲä롣$KEY  $VAL
+    Ф¤ˤĤƤϡmplist_add () 򻲾ȡ
 
-    $KEY ʤС$VAL ϴ֥ȤǤʤƤϤʤʤ
-    ξ硢Ťͤλȿ @c NULL Ǥʤ 1 餵졢$VAL 
-    ȿ 1 䤵롣
+    $KEY ʤС
+    $VAL ϴ֥ȤǤʤƤϤʤʤξ硢Ťͤλȿ 
+    @c NULL Ǥʤ 1 餵졢$VAL λȿ 1 䤵롣
 
     @return 
-      mplist_put () ѹ줿ɲä줿Ǥ
-    ޤ $PLIST ʬꥹȤ֤Ǥʤ @c NULL ֤   */
+     mplist_put () ѹ줿ɲä줿ǤϤޤ
+    $PLIST ʬꥹȤ֤Ǥʤ @c NULL ֤   */
 
 MPlist *
 mplist_put (MPlist *plist, MSymbol key, void *val)
@@ -892,7 +1070,8 @@ mplist_put (MPlist *plist, MSymbol key, void *val)
     {
       if (! MPLIST_TAIL_P (plist))
 	M17N_OBJECT_UNREF (MPLIST_VAL (plist));
-      M17N_OBJECT_REF (val);
+      if (val)
+	M17N_OBJECT_REF (val);
     }
   MPLIST_SET (plist, key, val);
   return plist;
@@ -917,12 +1096,12 @@ mplist_put (MPlist *plist, MSymbol key, void *val)
     @brief ץѥƥꥹΥץѥƥͤ.
 
     ؿ mplist_get () ϡץѥƥꥹ $PLIST Ϥᤫõơ
-     $KEY Ǥץѥƥ򸫤Ĥ롣ĤСͤؤΥݥ
-    󥿤 <tt>(void *)</tt> ֤Ĥʤ @c NULL ֤
+    $KEY Ǥץѥƥ򸫤Ĥ롣ĤСͤؤΥݥ󥿤
+    <tt>(void *)</tt> ֤Ĥʤ @c NULL ֤
 
-    @c NULL ֤äݤˤĤβǽ:嵭Τ褦˥ץѥƥ
-    ĤʤäȡץѥƥĤꡢͤ @c NULL 
-    Ǥ롣̤ɬפˤϴؿ 
+    @c NULL ֤äݤˤĤβǽ: 
+    嵭Τ褦˥ץѥƥĤʤäȡץѥƥĤꡢͤ
+    @c NULL ǤǤ롣̤ɬפˤϴؿ 
     mplist_find_by_key () Ȥȡ  */
 
 /***
@@ -942,8 +1121,8 @@ mplist_get (MPlist *plist, MSymbol key)
     @brief Add a property at the end of a property list.
 
     The mplist_add () function appends at the end of property list
-     $PLIST a property whose key is $KEY and value is $VAL.  $KEY can
-     be any symbol other than @c Mnil.
+    $PLIST a property whose key is $KEY and value is $VAL.  $KEY can
+    be any symbol other than @c Mnil.
 
     If $KEY is a managing key, $VAL must be a managed object.  In this
     case, the reference count of $VAL is incremented by one.
@@ -956,14 +1135,13 @@ mplist_get (MPlist *plist, MSymbol key)
     @brief ץѥƥꥹ˥ץѥƥɲä.
 
     ؿ mplist_add () ϡץѥƥꥹ $PLIST ˥ $KEY 
-    ͤ $VAL Ǥץѥƥɲä롣$KEY ϡ@c Mnil ʳǤ
-    դΥܥǤ褤
+    ͤ $VAL Ǥץѥƥɲä롣$KEY ϡ@c Mnil ʳǤդΥܥǤ褤
 
-    $KEY ʤС$VAL ϴ֥ȤǤʤƤϤʤʤ
-    ξ硢$VAL λȿ 1 䤵롣
+    $KEY ʤС$VAL ϴ֥ȤǤʤƤϤʤʤξ硢
+    $VAL λȿ 1 䤵롣
 
     @return
-      mplist_add () ɲä줿ǤϤޤ $PLIST 
+     mplist_add () ɲä줿ǤϤޤ $PLIST 
     ʬꥹȤ֤Ǥʤ @c NULL ֤  */
 
 MPlist *
@@ -972,7 +1150,7 @@ mplist_add (MPlist *plist, MSymbol key, void *val)
   if (key == Mnil)
     MERROR (MERROR_PLIST, NULL);
   MPLIST_FIND (plist, Mnil);
-  if (key->managing_key)
+  if (val && key->managing_key)
     M17N_OBJECT_REF (val);
   MPLIST_KEY (plist) = key;
   MPLIST_VAL (plist) = val;
@@ -1000,8 +1178,8 @@ mplist_add (MPlist *plist, MSymbol key, void *val)
     ؿ mplist_push () ϥץѥƥꥹ $PLIST Ƭ˥ $KEY 
     ͤ $VAL Ǥ륪֥Ȥ롣
 
-    $KEY ʤС$VAL ϴ֥ȤǤʤƤϤʤʤ
-    ξ硢$VAL λȿ 1 䤵롣
+    $KEY ʤС$VAL ϴ֥ȤǤʤƤϤʤʤξ硢
+    $VAL λȿ 1 䤵롣
 
     @return
     Фδؿ $PLIST ֤Ǥʤ@c NULL 
@@ -1019,7 +1197,7 @@ mplist_push (MPlist *plist, MSymbol key, void *val)
   MPLIST_VAL (pl) = MPLIST_VAL (plist);
   MPLIST_NEXT (pl) = MPLIST_NEXT (plist);
   plist->next = pl;
-  if (key->managing_key)
+  if (val && key->managing_key)
     M17N_OBJECT_REF (val);
   MPLIST_KEY (plist) = key;
   MPLIST_VAL (plist) = val;
@@ -1041,13 +1219,13 @@ mplist_push (MPlist *plist, MSymbol key, void *val)
 /***ja
     @brief ץѥƥꥹȤƬץѥƥ.
 
-    ؿ mplist_pop () ϥץѥƥꥹ $PLIST ƬΥץѥƥ
-    롣̤Ȥơ $PLIST 2ܤΥͤ 
+    ؿ mplist_pop () ϥץѥƥꥹ $PLIST 
+    ƬΥץѥƥ롣̤Ȥơ $PLIST 2ܤΥͤ 
     $PLIST ƬΥͤˤʤ롣
 
     @return 
-    СδؿϺ줿ץѥƥ֤ͤ
-    Ǥʤ @c NULL ֤  */
+    СδؿϺ줿ץѥƥ֤ͤǤʤ
+    @c NULL ֤  */
 
 void *
 mplist_pop (MPlist *plist)
@@ -1086,13 +1264,12 @@ mplist_pop (MPlist *plist)
 /***ja
     @brief ץѥƥꥹ椫Υĥץѥƥõ.
 
-    ؿ mplist_find_by_key () ϥץѥƥꥹ $PLIST Ϥᤫõ
-    ơ $KEY Ǥץѥƥ򸫤Ĥ롣ĤСΥץ
-    ѥƥϤޤ $PLIST ʬꥹȤ֤Ǥʤ@c NULL 
-    ֤
+    ؿ mplist_find_by_key () ϥץѥƥꥹ $PLIST 
+    Ϥᤫõ ơ $KEY 
+    Ǥץѥƥ򸫤Ĥ롣ĤСΥץѥƥϤޤ
+    $PLIST ʬꥹȤ֤Ǥʤ @c NULL ֤
 
-    $KEY  @c Mnil ʤС$PLIST κǸǤϤޤʬꥹȤ
-    ֤  */
+    $KEY  @c Mnil ʤС$PLIST κǸǤϤޤʬꥹȤ֤  */
 
 MPlist *
 mplist_find_by_key (MPlist *plist, MSymbol key)
@@ -1114,10 +1291,10 @@ mplist_find_by_key (MPlist *plist, MSymbol key)
 /***ja
     @brief ץѥƥꥹ椫ͤĥץѥƥõ.
 
-    ؿ mplist_find_by_value () ϥץѥƥꥹ $PLIST Ϥᤫ
-    õơͤ $VAL Ǥץѥƥ򸫤Ĥ롣ĤСΥץ
-    ѥƥϤޤ $PLIST ʬꥹȤ֤Ǥʤ@c NULL 
-    ֤ */
+    ؿ mplist_find_by_value () ϥץѥƥꥹ $PLIST 
+    Ϥᤫõơͤ $VAL 
+    Ǥץѥƥ򸫤Ĥ롣ĤСΥץѥƥϤޤ
+    $PLIST ʬꥹȤ֤Ǥʤ @c NULL ֤ */
 
 MPlist *
 mplist_find_by_value (MPlist *plist, void *val)
@@ -1141,9 +1318,9 @@ mplist_find_by_value (MPlist *plist, void *val)
 /***ja
     @brief ץѥƥꥹȤμʬꥹȤ֤.
 
-    ؿ mplist_next () ϥץѥƥꥹ $PLIST Σ
-    ǤϤޤʬꥹȤؤΥݥ󥿤֤$PLIST Ĺ 0 
-     @c NULL ֤  */
+    ؿ mplist_next () ϥץѥƥꥹ $PLIST  2 
+    ܤǤϤޤʬꥹȤؤΥݥ󥿤֤$PLIST Ĺ 0 
+    ʤ @c NULL ֤  */
 
 MPlist *
 mplist_next (MPlist *plist)
@@ -1167,13 +1344,12 @@ mplist_next (MPlist *plist)
 /***ja
     @brief ץѥƥꥹȤκǽΥץѥƥꤹ.
 
-    ؿ mplist_set () ϥץѥƥꥹ $PLIST κǽΥץѥƥ
-    ͤ򤽤줾 $KEY  $VALUE ꤹ롣$KEY  $VAL Ф
-    ¤ˤĤƤϡmplist_add () 򻲾ȡ
+    ؿ mplist_set () ϥץѥƥꥹ $PLIST 
+    κǽΥץѥƥΥͤ򤽤줾 $KEY  $VALUE ꤹ롣
+    $KEY  $VAL Ф¤ˤĤƤϡmplist_add () 򻲾ȡ
 
     @return
-     mplist_set ()  $PLIST ֤Ǥʤ
-    @c NULL ֤  */
+     mplist_set ()  $PLIST ֤Ǥʤ @c NULL ֤  */
 
 MPlist *
 mplist_set (MPlist *plist, MSymbol key, void * val)
@@ -1185,19 +1361,18 @@ mplist_set (MPlist *plist, MSymbol key, void * val)
 	  key = MPLIST_KEY (plist);
 	  M17N_OBJECT_UNREF (MPLIST_NEXT (plist));
 	  MPLIST_KEY (plist) = Mnil;
-	  if (key->managing_key && MPLIST_VAL (plist))
+	  if (key->managing_key)
 	    M17N_OBJECT_UNREF (MPLIST_VAL (plist));
 	  plist->next = NULL;
 	}
     }
   else
     {
-      if (! MPLIST_TAIL_P (plist)
-	  && MPLIST_KEY (plist)->managing_key
-	  && MPLIST_VAL (plist))
-	M17N_OBJECT_UNREF (MPLIST_VAL (plist));
-      if (key->managing_key)
+      if (val && key->managing_key)
 	M17N_OBJECT_REF (val);
+      if (! MPLIST_TAIL_P (plist)
+	  && MPLIST_KEY (plist)->managing_key)
+	M17N_OBJECT_UNREF (MPLIST_VAL (plist));
       MPLIST_SET (plist, key, val);
     }
   return plist;
@@ -1213,8 +1388,7 @@ mplist_set (MPlist *plist, MSymbol key, void * val)
 /***ja
     @brief ץѥƥꥹȤĹ֤.
 
-    ؿ mplist_length () ϥץѥƥꥹ $PLIST Υץѥƥο
-    ֤  */
+    ؿ mplist_length () ϥץѥƥꥹ $PLIST Υץѥƥο֤  */
 
 int
 mplist_length (MPlist *plist)
@@ -1236,8 +1410,8 @@ mplist_length (MPlist *plist)
 /***ja
     @brief ץѥƥꥹκǽΥץѥƥΥ֤.
 
-    ؿ mplist_key () ϡץѥƥꥹ $PLIST κ
-    ΥץѥƥΥ֤$PLIST Ĺ 0 ʤС @c Mnil 
+    ؿ mplist_key () ϡץѥƥꥹ $PLIST 
+    κǽΥץѥƥΥ֤$PLIST Ĺ 0 ʤС @c Mnil 
     ֤  */
 
 MSymbol
@@ -1257,9 +1431,8 @@ mplist_key (MPlist *plist)
 /***ja
     @brief ץѥƥꥹκǽΥץѥƥ֤ͤ.
 
-    ؿ mplist_value () ϡץѥƥꥹ $PLIST 
-    ǽΥץѥƥ֤ͤ$PLIST Ĺ 0 ʤС @c Mnil 
-    ֤  */
+    ؿ mplist_value () ϡץѥƥꥹ $PLIST κǽΥץѥƥ֤ͤ
+    $PLIST Ĺ 0 ʤС @c Mnil ֤  */
 
 void *
 mplist_value (MPlist *plist)
@@ -1289,15 +1462,14 @@ mplist_value (MPlist *plist)
     Each alternatives of @c ELEMENT is assigned one of these keys: @c
     Msymbol, @c Minteger, @c Mtext, @c Mplist
 
-    In an ascii-character-sequence, a backslush (\) is used as the escape
+    In an ascii-character-sequence, a backslash (\) is used as the escape
     character, which means that, for instance, <tt>"abc\ def"</tt>
     produces a symbol whose name is of length seven with the fourth
     character being a space.  */
 /***ja
     @brief M-text ǥꥢ饤ƥץѥƥꥹȤ.
 
-    ؿ mplist_deserialize ()  M-text $MT Ϥƥץѥƥꥹ
-    Ȥ֤
+    ؿ mplist_deserialize ()  M-text $MT ϤƥץѥƥꥹȤ֤
 
     $MT Υ󥿥åϰʲ̤ꡣ
 
@@ -1315,9 +1487,9 @@ mplist_value (MPlist *plist)
     @c ELEMENT γϥ@c Msymbol, @c Minteger, @c Mtext,
     @c Mplist Τ줫ƤƤ롣
 
-    ʸǤϡХåå (\) ʸȤ
-    Ѥ롣Ȥ <tt>"abc\ def"</tt> ϣʸܤʸǤ
-    ĹǤ̾ĥܥ롣   */
+    ʸǤϡХåå (\) ʸȤѤ롣Ȥ
+    <tt>"abc\ def"</tt>  4 ʸܤʸǤĹ 7 
+    Ǥ̾ĥܥ롣   */
 
 MPlist *
 mplist_deserialize (MText *mt)
@@ -1356,9 +1528,8 @@ mplist_deserialize (MText *mt)
 /***ja
     @brief ץѥƥꥹȤפ.
 
-    ؿ mdebug_dump_plist () ϥץѥƥꥹ $PLIST  stderr ˿
-    ֤˲ɤ ǰ롣 $UNDENT ϣܰʹߤΥǥȤꤹ
-    롣
+    ؿ mdebug_dump_plist () ϥץѥƥꥹ $PLIST  stderr 
+    ˿ʹ֤˲ɤʷǰ롣 $INDENT ϣܰʹߤΥǥȤꤹ롣
 
     @return
     δؿ $PLIST ֤  */
