@@ -52,26 +52,24 @@
 
     @brief ܥ륪֥ȤȤ˴ؤ API.
 
-    m17n 饤֥ϰդ˷ޤ뼱̻ҤȤ @e ܥ ȸƤ֥֥
-    ȤѤ롣ܥ X 饤֥ΥȥȻƤ뤬0 İ
-     @e ܥץѥƥ ĤȤǤ롣ܥץѥƥ
-     @e   @e  ʤ롣Ϥ켫ΥܥǤꡢͤ 
+    m17n 饤֥ϰդ˷ޤ뼱̻ҤȤ @e ܥ 
+    ȸƤ֥֥ȤѤ롣ܥ X 饤֥ΥȥȻƤ뤬
+    0 İʾ @e ܥץѥƥ ĤȤǤ롣ܥץѥƥ
+    @e   @e  ʤ롣Ϥ켫ΥܥǤꡢͤ 
     <tt>(void *)</tt> ˥㥹ȤǤΤʤ鲿Ǥ褤֥ܥ 
     S ĥܥץѥƥΤ K ΤΡפñˡS  K 
     ץѥƥפȸƤ֤Ȥ롣
 
     ܥӤϼ˰ʲ3̤Ǥ롣
 
-    @li ܥץѥƥ¾ΥץѥƥΥɽ魯
+    @li ܥץѥƥ¾ΥץѥƥΥɽ
 
     @li ʸåȡɷϡեȥåȤʤɤγƼ索֥Ȥɽ
-    魯
 
     @li m17n 饤֥ؿΰȤʤꡢؿεư椹롣
 
-    @e  ȸƤФ̤ʥܥ뤬ꡢ򥭡Ȥƻ
-    ĥץѥƥͤ @e ֥ ǤʤƤϤʤʤܺ
-     @ref m17nObject ȡ
+    @e  ȸƤФ̤ʥܥ뤬ꡢ򥭡Ȥƻĥץѥƥͤ
+    @e ֥ ǤʤƤϤʤʤܺ٤ @ref m17nObject ȡ
 */
 
 /*=*/
@@ -150,8 +148,7 @@ void
 msymbol__fini ()
 {
   int i;
-  MSymbol sym, next;
-  int freed_symbols = 0;
+  MSymbol sym;
 
   for (i = 0; i < SYMBOL_TABLE_SIZE; i++)
     for (sym = symbol_table[i]; sym; sym = sym->next)
@@ -160,7 +157,17 @@ msymbol__fini ()
 	  if (sym->plist.key->managing_key)
 	    M17N_OBJECT_UNREF (sym->plist.val);
 	  M17N_OBJECT_UNREF (sym->plist.next);
+	  sym->plist.key = Mnil;
 	}
+}
+
+void
+msymbol__free_table ()
+{
+  int i;
+  MSymbol sym, next;
+  int freed_symbols = 0;
+
   for (i = 0; i < SYMBOL_TABLE_SIZE; i++)
     {
       for (sym = symbol_table[i]; sym; sym = next)
@@ -189,6 +196,24 @@ msymbol__with_len (const char *name, int len)
   return msymbol (p);
 }
 
+/** Return a plist of symbols that has non-NULL property PROP.  If
+    PROP is Mnil, return a plist of all symbols.  Values of the plist
+    is NULL.  */
+
+MPlist *
+msymbol__list (MSymbol prop)
+{
+  MPlist *plist = mplist ();
+  int i;
+  MSymbol sym;
+
+  for (i = 0; i < SYMBOL_TABLE_SIZE; i++)
+    for (sym = symbol_table[i]; sym; sym = sym->next)
+      if (prop == Mnil || msymbol_get (sym, prop))
+	mplist_push (plist, sym, NULL);
+  return plist;
+}
+
 
 /** Canonicalize the name of SYM, and return a symbol of the
     canonicalized name.  Canonicalization is done by this rule:
@@ -267,12 +292,12 @@ MTextPropDeserializeFunc msymbol__deserializer = deserialize_symbol;
     The symbol #Mnil has the name <tt>"nil"</tt> and, in general,
     represents @e false or @e no.  When coerced to "int", its value is
     zero.  #Mnil can't have any symbol property.  */
-
 /***ja
     @brief "nil" ̾Ȥƻĥܥ.
 
-    ܥ #Mnil  <tt>"nil"</tt> Ȥ̤̾ˡֵפ
-    ϡפ̣롣"int" Ѵ줿硢ͤ 0 Ǥ롣
+    ܥ #Mnil  <tt>"nil"</tt> 
+    Ȥ̤̾ˡֵפޤϡפ̣롣
+    "int" Ѵ줿硢ͤ 0 Ǥ롣
     #Mnil ȤϤʤ륷ܥץѥƥʤ  */
 
 MSymbol Mnil;
@@ -284,12 +309,10 @@ MSymbol Mnil;
 
     The symbol #Mt has the name <tt>"t"</tt> and, in general,
     represents @e true or @e yes.  */
-
 /***ja
     @brief "t" ̾Ȥƻĥܥ.
 
-    ܥ #Mt  <tt>"t"</tt> Ȥ̤̾ˡֿפޤ
-    ֹפ̣롣  */
+    ܥ #Mt  <tt>"t"</tt> Ȥ̤̾ˡֿפޤϡֹפ̣롣  */
 
 MSymbol Mt;
 
@@ -301,12 +324,11 @@ MSymbol Mt;
     The symbol #Mstring has the name <tt>"string"</tt> and is used
     as an argument of the functions mchar_define_property (),
     etc.  */
-
 /***ja
     @brief "string" ̾Ȥƻĥܥ.
 
     ܥ #Mstring  <tt>"string"</tt> Ȥ̾ؿ 
-    mchar_define_property () ΰȤѤ롣  */
+    mchar_define_property () ʤɤΰȤѤ롣  */
 
 MSymbol Mstring;
 
@@ -318,12 +340,11 @@ MSymbol Mstring;
     The symbol #Msymbol has the name <tt>"symbol"</tt> and is used
     as an argument of the functions mchar_define_property (),
     etc.  */
-
 /***ja
     @brief "symbol" ̾Ȥƻĥܥ.
 
-    Ѥߥܥ #Msymbol  <tt>"symbol"</tt> Ȥ̾
-    ؿ mchar_define_property () ΰȤƻȤ롣  */
+    Ѥߥܥ #Msymbol  <tt>"symbol"</tt> Ȥ̾ؿ
+    mchar_define_property () ʤɤΰȤƻȤ롣  */
 
 MSymbol Msymbol;
 
@@ -344,16 +365,13 @@ MSymbol Msymbol;
 
     @errors
     This function never fails.  */
-
 /***ja
     @brief ܥ.
 
-    ؿ msymbol ()  $NAME Ȥ̾줿ܥ
-    Τ褦ʥܥ뤬¸ߤʤˤϡ롣줿
-    ܥϴǤϤʤ
+    ؿ msymbol ()  $NAME 
+    Ȥ̾줿ܥ֤Τ褦ʥܥ뤬¸ߤʤˤϡ롣줿ܥϴǤϤʤ
 
-    ʸĤǻϤޤ륷ܥ m17n 饤֥ѤǤꡢŪˤ
-    Ѥ롣
+    ʸĤǻϤޤ륷ܥ m17n 饤֥ѤǤꡢŪˤΤѤ롣
 
     @return
     δؿϸĤܥ֤
@@ -411,12 +429,11 @@ msymbol (const char *name)
 /***ja
     @brief .
 
-    ؿ msymbol_as_managing_key () ̾ $NAME Ŀ줿
-    ֤Ǥ̾ $NAME ĥܥ뤬С#Mnil 
-    ֤
+    ؿ msymbol_as_managing_key () ̾ $NAME 
+    Ŀ줿֤Ǥ̾ $NAME ĥܥ뤬С
+    #Mnil ֤
 
-    ʸĤǻϤޤ륷ܥ m17n 饤֥ѤǤꡢŪˤ
-    Ѥ롣
+    ʸĤǻϤޤ륷ܥ m17n 饤֥ѤǤꡢŪˤΤѤ롣
 
     @return
     Сδؿܥ֤Ǥʤ
@@ -460,6 +477,23 @@ msymbol_as_managing_key (const char *name)
 
 /*=*/
 
+/***en
+    @brief Check if a symbol is a managing key.
+
+    The msymbol_is_managing_key () function checks if the sysmbol
+    $SYMBOL is a managing key or not.
+
+    @return Return 1 if the symbol is a managing key.  Otherwise,
+    return 0.  */
+
+int
+msymbol_is_managing_key (MSymbol symbol)
+{
+  return (symbol->managing_key == 1);
+}
+
+/*=*/
+
 /***en
     @brief Search for a symbol that has a specified name.
 
@@ -472,15 +506,14 @@ msymbol_as_managing_key (const char *name)
 
     @errors
     This function never fails.  */
-
 /***ja
     @brief ꤵ줿̾ĥܥõ.
 
     ؿ msymbol_exist ()  $NAME Ȥ̾ĥܥõ
 
     @return
-    ⤷Τ褦ʥܥ뤬¸ߤʤФΥܥ֤Ǥ
-    СѤߥܥ #Mnil ֤  
+    ⤷Τ褦ʥܥ뤬¸ߤʤФΥܥ֤ǤʤСѤߥܥ
+    #Mnil ֤  
 
     @errors
     δؿϷ褷ƼԤʤ     */
@@ -521,8 +554,8 @@ msymbol_exist (const char *name)
 /***ja
     @brief ܥ̾.
 
-    ؿ msymbol_name () ϻꤵ줿ܥ $SYMBOL ̾ޤʸ
-    ؤΥݥ󥿤֤
+    ؿ msymbol_name () ϻꤵ줿ܥ $SYMBOL 
+    ̾ޤʸؤΥݥ󥿤֤
 
     @errors
     δؿϷ褷ƼԤʤ     */
@@ -553,18 +586,15 @@ msymbol_name (MSymbol symbol)
     If the operation was successful, msymbol_put () returns 0.
     Otherwise it returns -1 and assigns an error code to the external
     variable #merror_code.  */
-
 /***ja
     @brief ܥץѥƥͤꤹ.
 
-    ؿ msymbol_put () ϡܥ $SYMBOL ǥ $KEY Ǥ륷
-    ܥץѥƥͤ $VAL ꤹ롣Υܥץѥƥˤ
-    Ǥͤо񤭤롣$SYMBOL, $KEY Ȥ #Mnil ǤäƤϤʤ
-    ʤ
+    ؿ msymbol_put () ϡܥ $SYMBOL ǥ $KEY Ǥ륷ܥץѥƥͤ
+    $VAL ꤹ롣ΥܥץѥƥˤǤͤо񤭤롣
+    $SYMBOL, $KEY Ȥ #Mnil ǤäƤϤʤʤ
 
-    $KEY ʤС$VAL ϴ֥ȤǤʤƤϤʤʤ
-    ξ硢Ťͤλȿ @c NULL Ǥʤ 1 餵졢$VAL 
-    ȿ 1 䤵롣
+    $KEY ʤС$VAL ϴ֥ȤǤʤƤϤʤʤξ硢Ťͤλȿ
+    @c NULL Ǥʤ 1 餵졢$VAL λȿ 1 䤵롣
 
     @return
     Сmsymbol_put ()  0 ֤Ǥʤ -1 
@@ -599,17 +629,17 @@ msymbol_put (MSymbol symbol, MSymbol key, void *val)
     @return
     If an error is detected, msymbol_get () returns @c NULL and
     assigns an error code to the external variable #merror_code.  */
-
 /***ja
     @brief ܥץѥƥͤ.
 
-    ؿ msymbol_get () ϡܥ $SYMBOL ĥܥץѥƥ
-    Τ $KEY ǤΤõ⤷륷ܥץѥƥ
-    ¸ߤС֤ͤǤʤ @c NULL ֤
+    ؿ msymbol_get () ϡܥ $SYMBOL 
+    ĥܥץѥƥΤ $KEY 
+    ǤΤõ⤷륷ܥץѥƥ¸ߤС֤ͤǤʤ
+    @c NULL ֤
 
     @return 
-    顼Ф줿硢msymbol_get ()  @c NULL ֤
-    ѿ #merror_code ˥顼ɤꤹ롣  */
+    顼Ф줿硢msymbol_get ()  @c NULL 
+    ֤ѿ #merror_code ˥顼ɤꤹ롣  */
 
 /***
     @errors
@@ -653,8 +683,8 @@ msymbol_get (MSymbol symbol, MSymbol key)
 /***ja
     @brief ܥפ.
 
-    ؿ mdebug_dump_symbol () ϥܥ $symbol  stderr ˿ʹ֤˲
-    ɤ ǰ롣 $UNDENT ϣܰʹߤΥǥȤꤹ롣
+    ؿ mdebug_dump_symbol () ϥܥ $symbol  stderr 
+    ˿ʹ֤˲ɤʷǰ롣 $INDENT ϣܰʹߤΥǥȤꤹ롣
 
     @return
     δؿ $SYMBOL ֤ 
@@ -705,7 +735,7 @@ mdebug_dump_symbol (MSymbol symbol, int indent)
     @brief ٤ƤΥܥ̾פ.
 
     ؿ mdebug_dump_all_symbols () ϡ٤ƤΥܥ̾ 
-    stderr ˰롣 $UNDENT ϣܰʹߤΥǥȤꤹ롣
+    stderr ˰롣 $INDENT ϣܰʹߤΥǥȤꤹ롣
 
     @return
     δؿ #Mnil ֤ 
@@ -718,7 +748,7 @@ MSymbol
 mdebug_dump_all_symbols (int indent)
 {
   char *prefix;
-  int i;
+  int i, n;
   MSymbol sym;
 
   if (indent < 0)
@@ -728,14 +758,15 @@ mdebug_dump_all_symbols (int indent)
   prefix[indent] = 0;
 
   fprintf (stderr, "(symbol-list");
-  for (i = 0; i < SYMBOL_TABLE_SIZE; i++)
+  for (i = n = 0; i < SYMBOL_TABLE_SIZE; i++)
     if ((sym = symbol_table[i]))
       {
 	fprintf (stderr, "\n%s  (%4d", prefix, i);
-	for (; sym; sym = sym->next)
+	for (; sym; sym = sym->next, n++)
 	  fprintf (stderr, " '%s'", sym->name);
 	fprintf (stderr, ")");
       }
+  fprintf (stderr, "\n%s  (total %d)", prefix, n);
   fprintf (stderr, ")");
   return Mnil;
 }
